# -----------------------------------------------------------------------------
#  pytermor [ANSI formatted terminal output toolset]
#  (c) 2022—2023. A. Shavykin <0.delameter@gmail.com>
# -----------------------------------------------------------------------------
# AUTOGENERATED MODULE
# 2023-10-16T00:29:40.255281
"""
Color preset list:

 -   16x `Color16` (16 unique)
 -  256x `Color256` (247 unique)
 - 2304x `ColorRGB` (2297 unique)
"""
from __future__ import annotations

from .ansi import IntCode
from .color import Color16, Color256, ColorRGB
import typing as t

class _ColorValues:
    """
    Filling of indexes and registries with all the preset colors takes quite
    a bit of time (~5 ms for `Color256` registry, ~25 ms for `ColorRGB`
    registry), which can be essential for applications that require fast import
    of the dependencies. Deferred instantiating can save that time, i.e. colors
    and supplementary structures do not actually exist until any of them is
    accessed for the first time. At the moment, this mechanism is enabled only
    for `cvr` registry, but not for `cv` (i.e. `Color256`), as the latter
    is always loaded automatically by the library itself anyway. Also it is
    harder to implement correctly considering these colors have relation with
    `Color16` instances.
    """
    def _instantiate(self):
        for name in sorted(dir(self)):
            if name.isupper():
                   if isinstance(attr := getattr(self, name), t.Callable):
                        setattr(self, name, attr())

    def load(self):
        self._instantiate()

class _ColorValuesXterm(_ColorValues):
    """
    ColorValuesXterm
    """

    # fmt: off
    BLACK       : Color16 = Color16(0x000000, IntCode.BLACK,       IntCode.BG_BLACK,       "black",        register=True)
    RED         : Color16 = Color16(0x800000, IntCode.RED,         IntCode.BG_RED,         "red",          register=True)
    GREEN       : Color16 = Color16(0x008000, IntCode.GREEN,       IntCode.BG_GREEN,       "green",        register=True)
    YELLOW      : Color16 = Color16(0x808000, IntCode.YELLOW,      IntCode.BG_YELLOW,      "yellow",       register=True)
    BLUE        : Color16 = Color16(0x000080, IntCode.BLUE,        IntCode.BG_BLUE,        "blue",         register=True)
    MAGENTA     : Color16 = Color16(0x800080, IntCode.MAGENTA,     IntCode.BG_MAGENTA,     "magenta",      register=True)
    CYAN        : Color16 = Color16(0x008080, IntCode.CYAN,        IntCode.BG_CYAN,        "cyan",         register=True)
    WHITE       : Color16 = Color16(0xc0c0c0, IntCode.WHITE,       IntCode.BG_WHITE,       "white",        register=True)
    GRAY        : Color16 = Color16(0x808080, IntCode.GRAY,        IntCode.BG_GRAY,        "gray",         register=True)
    HI_RED      : Color16 = Color16(0xff0000, IntCode.HI_RED,      IntCode.BG_HI_RED,      "hi-red",       register=True, aliases=['bright-red'])
    HI_GREEN    : Color16 = Color16(0x00ff00, IntCode.HI_GREEN,    IntCode.BG_HI_GREEN,    "hi-green",     register=True, aliases=['bright-green'])
    HI_YELLOW   : Color16 = Color16(0xffff00, IntCode.HI_YELLOW,   IntCode.BG_HI_YELLOW,   "hi-yellow",    register=True, aliases=['bright-yellow'])
    HI_BLUE     : Color16 = Color16(0x0000ff, IntCode.HI_BLUE,     IntCode.BG_HI_BLUE,     "hi-blue",      register=True, aliases=['bright-blue'])
    HI_MAGENTA  : Color16 = Color16(0xff00ff, IntCode.HI_MAGENTA,  IntCode.BG_HI_MAGENTA,  "hi-magenta",   register=True, aliases=['bright-magenta'])
    HI_CYAN     : Color16 = Color16(0x00ffff, IntCode.HI_CYAN,     IntCode.BG_HI_CYAN,     "hi-cyan",      register=True, aliases=['bright-cyan'])
    HI_WHITE    : Color16 = Color16(0xffffff, IntCode.HI_WHITE,    IntCode.BG_HI_WHITE,    "hi-white",     register=True, aliases=['bright-white'])

    __256A_16_BLACK      : Color256 = Color256(0x000000, 0,   "black",                 register=True, aliases=['color0', 'colour0', 'xterm0'], color16_equiv=BLACK)
    __256A_16_RED        : Color256 = Color256(0x800000, 1,   "maroon",                register=True, aliases=['color1', 'colour1', 'xterm1'], color16_equiv=RED)
    __256A_16_GREEN      : Color256 = Color256(0x008000, 2,   "green",                 register=True, aliases=['color2', 'colour2', 'xterm2'], color16_equiv=GREEN)
    __256A_16_YELLOW     : Color256 = Color256(0x808000, 3,   "olive",                 register=True, aliases=['color3', 'colour3', 'xterm3'], color16_equiv=YELLOW)
    __256A_16_BLUE       : Color256 = Color256(0x000080, 4,   "navy",                  register=True, aliases=['color4', 'colour4', 'xterm4'], color16_equiv=BLUE)
    __256A_16_MAGENTA    : Color256 = Color256(0x800080, 5,   "purple-5",              register=True, aliases=['color5', 'colour5', 'xterm5'], color16_equiv=MAGENTA)
    __256A_16_CYAN       : Color256 = Color256(0x008080, 6,   "teal",                  register=True, aliases=['color6', 'colour6', 'xterm6'], color16_equiv=CYAN)
    __256A_16_WHITE      : Color256 = Color256(0xc0c0c0, 7,   "silver",                register=True, aliases=['color7', 'colour7', 'xterm7'], color16_equiv=WHITE)
    __256A_16_GRAY       : Color256 = Color256(0x808080, 8,   "gray",                  register=True, aliases=['color8', 'colour8', 'xterm8'], color16_equiv=GRAY)
    __256A_16_HI_RED     : Color256 = Color256(0xff0000, 9,   "red",                   register=True, aliases=['color9', 'colour9', 'xterm9'], color16_equiv=HI_RED)
    __256A_16_HI_GREEN   : Color256 = Color256(0x00ff00, 10,  "lime",                  register=True, aliases=['color10', 'colour10', 'xterm10'], color16_equiv=HI_GREEN)
    __256A_16_HI_YELLOW  : Color256 = Color256(0xffff00, 11,  "yellow",                register=True, aliases=['color11', 'colour11', 'xterm11'], color16_equiv=HI_YELLOW)
    __256A_16_HI_BLUE    : Color256 = Color256(0x0000ff, 12,  "blue",                  register=True, aliases=['color12', 'colour12', 'xterm12'], color16_equiv=HI_BLUE)
    __256A_16_HI_MAGENTA : Color256 = Color256(0xff00ff, 13,  "fuchsia",               register=True, aliases=['color13', 'colour13', 'xterm13'], color16_equiv=HI_MAGENTA)
    __256A_16_HI_CYAN    : Color256 = Color256(0x00ffff, 14,  "aqua",                  register=True, aliases=['color14', 'colour14', 'xterm14'], color16_equiv=HI_CYAN)
    __256A_16_HI_WHITE   : Color256 = Color256(0xffffff, 15,  "white",                 register=True, aliases=['color15', 'colour15', 'xterm15'], color16_equiv=HI_WHITE)
    AQUAMARINE_1         : Color256 = Color256(0x87ffd7, 122, "aquamarine-1",          register=True, aliases=['color122', 'colour122', 'xterm122'])
    AQUAMARINE_2         : Color256 = Color256(0x5fffd7, 86,  "aquamarine-2",          register=True, aliases=['color86', 'colour86', 'xterm86'])
    AQUAMARINE_3         : Color256 = Color256(0x5fd7af, 79,  "aquamarine-3",          register=True, aliases=['color79', 'colour79', 'xterm79'])
    BLUE_1               : Color256 = Color256(0x0000ff, 21,  "blue-1",                register=True, aliases=['color21', 'colour21', 'xterm21'])
    BLUE_2               : Color256 = Color256(0x0000d7, 20,  "blue-2",                register=True, aliases=['color20', 'colour20', 'xterm20'])
    BLUE_3               : Color256 = Color256(0x0000af, 19,  "blue-3",                register=True, aliases=['color19', 'colour19', 'xterm19'])
    BLUE_VIOLET          : Color256 = Color256(0x5f00ff, 57,  "blue-violet",           register=True, aliases=['color57', 'colour57', 'xterm57'])
    CADET_BLUE           : Color256 = Color256(0x5fafaf, 73,  "cadet-blue",            register=True, aliases=['color73', 'colour73', 'xterm73'])
    CADET_BLUE_2         : Color256 = Color256(0x5faf87, 72,  "cadet-blue-2",          register=True, aliases=['color72', 'colour72', 'xterm72'])
    CHARTREUSE_1         : Color256 = Color256(0x87ff00, 118, "chartreuse-1",          register=True, aliases=['color118', 'colour118', 'xterm118'])
    CHARTREUSE_2         : Color256 = Color256(0x5fff00, 82,  "chartreuse-2",          register=True, aliases=['color82', 'colour82', 'xterm82'])
    CHARTREUSE_3         : Color256 = Color256(0x87d700, 112, "chartreuse-3",          register=True, aliases=['color112', 'colour112', 'xterm112'])
    CHARTREUSE_4         : Color256 = Color256(0x5fd700, 76,  "chartreuse-4",          register=True, aliases=['color76', 'colour76', 'xterm76'])
    CHARTREUSE_5         : Color256 = Color256(0x5faf00, 70,  "chartreuse-5",          register=True, aliases=['color70', 'colour70', 'xterm70'])
    CHARTREUSE_6         : Color256 = Color256(0x5f8700, 64,  "chartreuse-6",          register=True, aliases=['color64', 'colour64', 'xterm64'])
    CORNFLOWER_BLUE      : Color256 = Color256(0x5f87ff, 69,  "cornflower-blue",       register=True, aliases=['color69', 'colour69', 'xterm69'])
    CORNSILK_1           : Color256 = Color256(0xffffd7, 230, "cornsilk-1",            register=True, aliases=['color230', 'colour230', 'xterm230'])
    CYAN_1               : Color256 = Color256(0x00ffff, 51,  "cyan-1",                register=True, aliases=['color51', 'colour51', 'xterm51'])
    CYAN_2               : Color256 = Color256(0x00ffd7, 50,  "cyan-2",                register=True, aliases=['color50', 'colour50', 'xterm50'])
    CYAN_3               : Color256 = Color256(0x00d7af, 43,  "cyan-3",                register=True, aliases=['color43', 'colour43', 'xterm43'])
    DARK_BLUE            : Color256 = Color256(0x000087, 18,  "dark-blue",             register=True, aliases=['color18', 'colour18', 'xterm18'])
    DARK_CYAN            : Color256 = Color256(0x00af87, 36,  "dark-cyan",             register=True, aliases=['color36', 'colour36', 'xterm36'])
    DARK_GOLDENROD       : Color256 = Color256(0xaf8700, 136, "dark-goldenrod",        register=True, aliases=['color136', 'colour136', 'xterm136'])
    DARK_GREEN           : Color256 = Color256(0x005f00, 22,  "dark-green",            register=True, aliases=['color22', 'colour22', 'xterm22'])
    DARK_KHAKI           : Color256 = Color256(0xafaf5f, 143, "dark-khaki",            register=True, aliases=['color143', 'colour143', 'xterm143'])
    DARK_MAGENTA         : Color256 = Color256(0x8700af, 91,  "dark-magenta",          register=True, aliases=['color91', 'colour91', 'xterm91'])
    DARK_MAGENTA_2       : Color256 = Color256(0x870087, 90,  "dark-magenta-2",        register=True, aliases=['color90', 'colour90', 'xterm90'])
    DARK_OLIVE_GREEN_1   : Color256 = Color256(0xd7ff87, 192, "dark-olive-green-1",    register=True, aliases=['color192', 'colour192', 'xterm192'])
    DARK_OLIVE_GREEN_2   : Color256 = Color256(0xd7ff5f, 191, "dark-olive-green-2",    register=True, aliases=['color191', 'colour191', 'xterm191'])
    DARK_OLIVE_GREEN_3   : Color256 = Color256(0xafff5f, 155, "dark-olive-green-3",    register=True, aliases=['color155', 'colour155', 'xterm155'])
    DARK_OLIVE_GREEN_4   : Color256 = Color256(0x87d75f, 113, "dark-olive-green-4",    register=True, aliases=['color113', 'colour113', 'xterm113'])
    DARK_OLIVE_GREEN_5   : Color256 = Color256(0xafd75f, 149, "dark-olive-green-5",    register=True, aliases=['color149', 'colour149', 'xterm149'])
    DARK_OLIVE_GREEN_6   : Color256 = Color256(0x87af5f, 107, "dark-olive-green-6",    register=True, aliases=['color107', 'colour107', 'xterm107'])
    DARK_ORANGE          : Color256 = Color256(0xff8700, 208, "dark-orange",           register=True, aliases=['color208', 'colour208', 'xterm208'])
    DARK_ORANGE_2        : Color256 = Color256(0xd75f00, 166, "dark-orange-2",         register=True, aliases=['color166', 'colour166', 'xterm166'])
    DARK_ORANGE_3        : Color256 = Color256(0xaf5f00, 130, "dark-orange-3",         register=True, aliases=['color130', 'colour130', 'xterm130'])
    DARK_RED             : Color256 = Color256(0x870000, 88,  "dark-red",              register=True, aliases=['color88', 'colour88', 'xterm88'])
    DARK_RED_2           : Color256 = Color256(0x5f0000, 52,  "dark-red-2",            register=True, aliases=['color52', 'colour52', 'xterm52'])
    DARK_SEA_GREEN_1     : Color256 = Color256(0xafffd7, 158, "dark-sea-green-1",      register=True, aliases=['color158', 'colour158', 'xterm158'])
    DARK_SEA_GREEN_2     : Color256 = Color256(0xd7ffaf, 193, "dark-sea-green-2",      register=True, aliases=['color193', 'colour193', 'xterm193'])
    DARK_SEA_GREEN_3     : Color256 = Color256(0xafffaf, 157, "dark-sea-green-3",      register=True, aliases=['color157', 'colour157', 'xterm157'])
    DARK_SEA_GREEN_4     : Color256 = Color256(0xafd7af, 151, "dark-sea-green-4",      register=True, aliases=['color151', 'colour151', 'xterm151'])
    DARK_SEA_GREEN_5     : Color256 = Color256(0x87d7af, 115, "dark-sea-green-5",      register=True, aliases=['color115', 'colour115', 'xterm115'])
    DARK_SEA_GREEN_6     : Color256 = Color256(0xafd787, 150, "dark-sea-green-6",      register=True, aliases=['color150', 'colour150', 'xterm150'])
    DARK_SEA_GREEN_7     : Color256 = Color256(0x87af87, 108, "dark-sea-green-7",      register=True, aliases=['color108', 'colour108', 'xterm108'])
    DARK_SEA_GREEN_8     : Color256 = Color256(0x5faf5f, 71,  "dark-sea-green-8",      register=True, aliases=['color71', 'colour71', 'xterm71'])
    DARK_SEA_GREEN_9     : Color256 = Color256(0x5f875f, 65,  "dark-sea-green-9",      register=True, aliases=['color65', 'colour65', 'xterm65'])
    DARK_SLATE_GRAY_1    : Color256 = Color256(0x87ffff, 123, "dark-slate-gray-1",     register=True, aliases=['color123', 'colour123', 'xterm123'])
    DARK_SLATE_GRAY_2    : Color256 = Color256(0x5fffff, 87,  "dark-slate-gray-2",     register=True, aliases=['color87', 'colour87', 'xterm87'])
    DARK_SLATE_GRAY_3    : Color256 = Color256(0x87d7d7, 116, "dark-slate-gray-3",     register=True, aliases=['color116', 'colour116', 'xterm116'])
    DARK_TURQUOISE       : Color256 = Color256(0x00d7d7, 44,  "dark-turquoise",        register=True, aliases=['color44', 'colour44', 'xterm44'])
    DARK_VIOLET          : Color256 = Color256(0xaf00d7, 128, "dark-violet",           register=True, aliases=['color128', 'colour128', 'xterm128'])
    DARK_VIOLET_2        : Color256 = Color256(0x8700d7, 92,  "dark-violet-2",         register=True, aliases=['color92', 'colour92', 'xterm92'])
    DEEP_PINK_1          : Color256 = Color256(0xff00af, 199, "deep-pink-1",           register=True, aliases=['color199', 'colour199', 'xterm199'])
    DEEP_PINK_2          : Color256 = Color256(0xff0087, 198, "deep-pink-2",           register=True, aliases=['color198', 'colour198', 'xterm198'])
    DEEP_PINK_3          : Color256 = Color256(0xd70087, 162, "deep-pink-3",           register=True, aliases=['color162', 'colour162', 'xterm162'])
    DEEP_PINK_4          : Color256 = Color256(0xff005f, 197, "deep-pink-4",           register=True, aliases=['color197', 'colour197', 'xterm197'])
    DEEP_PINK_5          : Color256 = Color256(0xd7005f, 161, "deep-pink-5",           register=True, aliases=['color161', 'colour161', 'xterm161'])
    DEEP_PINK_6          : Color256 = Color256(0xaf005f, 125, "deep-pink-6",           register=True, aliases=['color125', 'colour125', 'xterm125'])
    DEEP_PINK_7          : Color256 = Color256(0x87005f, 89,  "deep-pink-7",           register=True, aliases=['color89', 'colour89', 'xterm89'])
    DEEP_PINK_8          : Color256 = Color256(0x5f005f, 53,  "deep-pink-8",           register=True, aliases=['color53', 'colour53', 'xterm53'])
    DEEP_SKY_BLUE_1      : Color256 = Color256(0x00afff, 39,  "deep-sky-blue-1",       register=True, aliases=['color39', 'colour39', 'xterm39'])
    DEEP_SKY_BLUE_2      : Color256 = Color256(0x00afd7, 38,  "deep-sky-blue-2",       register=True, aliases=['color38', 'colour38', 'xterm38'])
    DEEP_SKY_BLUE_3      : Color256 = Color256(0x0087d7, 32,  "deep-sky-blue-3",       register=True, aliases=['color32', 'colour32', 'xterm32'])
    DEEP_SKY_BLUE_4      : Color256 = Color256(0x0087af, 31,  "deep-sky-blue-4",       register=True, aliases=['color31', 'colour31', 'xterm31'])
    DEEP_SKY_BLUE_5      : Color256 = Color256(0x005faf, 25,  "deep-sky-blue-5",       register=True, aliases=['color25', 'colour25', 'xterm25'])
    DEEP_SKY_BLUE_6      : Color256 = Color256(0x005f87, 24,  "deep-sky-blue-6",       register=True, aliases=['color24', 'colour24', 'xterm24'])
    DEEP_SKY_BLUE_7      : Color256 = Color256(0x005f5f, 23,  "deep-sky-blue-7",       register=True, aliases=['color23', 'colour23', 'xterm23'])
    DODGER_BLUE_1        : Color256 = Color256(0x0087ff, 33,  "dodger-blue-1",         register=True, aliases=['color33', 'colour33', 'xterm33'])
    DODGER_BLUE_2        : Color256 = Color256(0x005fff, 27,  "dodger-blue-2",         register=True, aliases=['color27', 'colour27', 'xterm27'])
    DODGER_BLUE_3        : Color256 = Color256(0x005fd7, 26,  "dodger-blue-3",         register=True, aliases=['color26', 'colour26', 'xterm26'])
    GOLD_1               : Color256 = Color256(0xffd700, 220, "gold-1",                register=True, aliases=['color220', 'colour220', 'xterm220'])
    GOLD_2               : Color256 = Color256(0xd7af00, 178, "gold-2",                register=True, aliases=['color178', 'colour178', 'xterm178'])
    GOLD_3               : Color256 = Color256(0xafaf00, 142, "gold-3",                register=True, aliases=['color142', 'colour142', 'xterm142'])
    GRAY_0               : Color256 = Color256(0x000000, 16,  "gray-0",                register=True, aliases=['color16', 'colour16', 'xterm16'])
    GRAY_100             : Color256 = Color256(0xffffff, 231, "gray-100",              register=True, aliases=['color231', 'colour231', 'xterm231'])
    GRAY_11              : Color256 = Color256(0x1c1c1c, 234, "gray-11",               register=True, aliases=['color234', 'colour234', 'xterm234'])
    GRAY_15              : Color256 = Color256(0x262626, 235, "gray-15",               register=True, aliases=['color235', 'colour235', 'xterm235'])
    GRAY_19              : Color256 = Color256(0x303030, 236, "gray-19",               register=True, aliases=['color236', 'colour236', 'xterm236'])
    GRAY_23              : Color256 = Color256(0x3a3a3a, 237, "gray-23",               register=True, aliases=['color237', 'colour237', 'xterm237'])
    GRAY_27              : Color256 = Color256(0x444444, 238, "gray-27",               register=True, aliases=['color238', 'colour238', 'xterm238'])
    GRAY_3               : Color256 = Color256(0x080808, 232, "gray-3",                register=True, aliases=['color232', 'colour232', 'xterm232'])
    GRAY_30              : Color256 = Color256(0x4e4e4e, 239, "gray-30",               register=True, aliases=['color239', 'colour239', 'xterm239'])
    GRAY_35              : Color256 = Color256(0x585858, 240, "gray-35",               register=True, aliases=['color240', 'colour240', 'xterm240'])
    GRAY_37              : Color256 = Color256(0x5f5f5f, 59,  "gray-37",               register=True, aliases=['color59', 'colour59', 'xterm59'])
    GRAY_39              : Color256 = Color256(0x626262, 241, "gray-39",               register=True, aliases=['color241', 'colour241', 'xterm241'])
    GRAY_42              : Color256 = Color256(0x6c6c6c, 242, "gray-42",               register=True, aliases=['color242', 'colour242', 'xterm242'])
    GRAY_46              : Color256 = Color256(0x767676, 243, "gray-46",               register=True, aliases=['color243', 'colour243', 'xterm243'])
    GRAY_50              : Color256 = Color256(0x808080, 244, "gray-50",               register=True, aliases=['color244', 'colour244', 'xterm244'])
    GRAY_53              : Color256 = Color256(0x878787, 102, "gray-53",               register=True, aliases=['color102', 'colour102', 'xterm102'])
    GRAY_54              : Color256 = Color256(0x8a8a8a, 245, "gray-54",               register=True, aliases=['color245', 'colour245', 'xterm245'])
    GRAY_58              : Color256 = Color256(0x949494, 246, "gray-58",               register=True, aliases=['color246', 'colour246', 'xterm246'])
    GRAY_62              : Color256 = Color256(0x9e9e9e, 247, "gray-62",               register=True, aliases=['color247', 'colour247', 'xterm247'])
    GRAY_63              : Color256 = Color256(0xaf87af, 139, "gray-63",               register=True, aliases=['color139', 'colour139', 'xterm139'])
    GRAY_66              : Color256 = Color256(0xa8a8a8, 248, "gray-66",               register=True, aliases=['color248', 'colour248', 'xterm248'])
    GRAY_69              : Color256 = Color256(0xafafaf, 145, "gray-69",               register=True, aliases=['color145', 'colour145', 'xterm145'])
    GRAY_7               : Color256 = Color256(0x121212, 233, "gray-7",                register=True, aliases=['color233', 'colour233', 'xterm233'])
    GRAY_70              : Color256 = Color256(0xb2b2b2, 249, "gray-70",               register=True, aliases=['color249', 'colour249', 'xterm249'])
    GRAY_74              : Color256 = Color256(0xbcbcbc, 250, "gray-74",               register=True, aliases=['color250', 'colour250', 'xterm250'])
    GRAY_78              : Color256 = Color256(0xc6c6c6, 251, "gray-78",               register=True, aliases=['color251', 'colour251', 'xterm251'])
    GRAY_82              : Color256 = Color256(0xd0d0d0, 252, "gray-82",               register=True, aliases=['color252', 'colour252', 'xterm252'])
    GRAY_84              : Color256 = Color256(0xd7d7d7, 188, "gray-84",               register=True, aliases=['color188', 'colour188', 'xterm188'])
    GRAY_85              : Color256 = Color256(0xdadada, 253, "gray-85",               register=True, aliases=['color253', 'colour253', 'xterm253'])
    GRAY_89              : Color256 = Color256(0xe4e4e4, 254, "gray-89",               register=True, aliases=['color254', 'colour254', 'xterm254'])
    GRAY_93              : Color256 = Color256(0xeeeeee, 255, "gray-93",               register=True, aliases=['color255', 'colour255', 'xterm255'])
    GREEN_2              : Color256 = Color256(0x00ff00, 46,  "green-2",               register=True, aliases=['color46', 'colour46', 'xterm46'])
    GREEN_3              : Color256 = Color256(0x00d700, 40,  "green-3",               register=True, aliases=['color40', 'colour40', 'xterm40'])
    GREEN_4              : Color256 = Color256(0x00af00, 34,  "green-4",               register=True, aliases=['color34', 'colour34', 'xterm34'])
    GREEN_5              : Color256 = Color256(0x008700, 28,  "green-5",               register=True, aliases=['color28', 'colour28', 'xterm28'])
    GREEN_YELLOW         : Color256 = Color256(0xafff00, 154, "green-yellow",          register=True, aliases=['color154', 'colour154', 'xterm154'])
    HONEYDEW_2           : Color256 = Color256(0xd7ffd7, 194, "honeydew-2",            register=True, aliases=['color194', 'colour194', 'xterm194'])
    HOT_PINK             : Color256 = Color256(0xff5fd7, 206, "hot-pink",              register=True, aliases=['color206', 'colour206', 'xterm206'])
    HOT_PINK_2           : Color256 = Color256(0xff5faf, 205, "hot-pink-2",            register=True, aliases=['color205', 'colour205', 'xterm205'])
    HOT_PINK_3           : Color256 = Color256(0xd75faf, 169, "hot-pink-3",            register=True, aliases=['color169', 'colour169', 'xterm169'])
    HOT_PINK_4           : Color256 = Color256(0xd75f87, 168, "hot-pink-4",            register=True, aliases=['color168', 'colour168', 'xterm168'])
    HOT_PINK_5           : Color256 = Color256(0xaf5f87, 132, "hot-pink-5",            register=True, aliases=['color132', 'colour132', 'xterm132'])
    INDIAN_RED_1         : Color256 = Color256(0xff5f5f, 203, "indian-red-1",          register=True, aliases=['color203', 'colour203', 'xterm203'])
    INDIAN_RED_2         : Color256 = Color256(0xff5f87, 204, "indian-red-2",          register=True, aliases=['color204', 'colour204', 'xterm204'])
    INDIAN_RED_3         : Color256 = Color256(0xd75f5f, 167, "indian-red-3",          register=True, aliases=['color167', 'colour167', 'xterm167'])
    INDIAN_RED_4         : Color256 = Color256(0xaf5f5f, 131, "indian-red-4",          register=True, aliases=['color131', 'colour131', 'xterm131'])
    KHAKI_1              : Color256 = Color256(0xffff87, 228, "khaki-1",               register=True, aliases=['color228', 'colour228', 'xterm228'])
    KHAKI_3              : Color256 = Color256(0xd7d75f, 185, "khaki-3",               register=True, aliases=['color185', 'colour185', 'xterm185'])
    LIGHT_CORAL          : Color256 = Color256(0xff8787, 210, "light-coral",           register=True, aliases=['color210', 'colour210', 'xterm210'])
    LIGHT_CYAN_1         : Color256 = Color256(0xd7ffff, 195, "light-cyan-1",          register=True, aliases=['color195', 'colour195', 'xterm195'])
    LIGHT_CYAN_3         : Color256 = Color256(0xafd7d7, 152, "light-cyan-3",          register=True, aliases=['color152', 'colour152', 'xterm152'])
    LIGHT_GOLDENROD_1    : Color256 = Color256(0xffff5f, 227, "light-goldenrod-1",     register=True, aliases=['color227', 'colour227', 'xterm227'])
    LIGHT_GOLDENROD_2    : Color256 = Color256(0xffd787, 222, "light-goldenrod-2",     register=True, aliases=['color222', 'colour222', 'xterm222'])
    LIGHT_GOLDENROD_3    : Color256 = Color256(0xd7d787, 186, "light-goldenrod-3",     register=True, aliases=['color186', 'colour186', 'xterm186'])
    LIGHT_GOLDENROD_4    : Color256 = Color256(0xffd75f, 221, "light-goldenrod-4",     register=True, aliases=['color221', 'colour221', 'xterm221'])
    LIGHT_GOLDENROD_5    : Color256 = Color256(0xd7af5f, 179, "light-goldenrod-5",     register=True, aliases=['color179', 'colour179', 'xterm179'])
    LIGHT_GREEN          : Color256 = Color256(0x87ff87, 120, "light-green",           register=True, aliases=['color120', 'colour120', 'xterm120'])
    LIGHT_GREEN_2        : Color256 = Color256(0x87ff5f, 119, "light-green-2",         register=True, aliases=['color119', 'colour119', 'xterm119'])
    LIGHT_PINK_1         : Color256 = Color256(0xffafaf, 217, "light-pink-1",          register=True, aliases=['color217', 'colour217', 'xterm217'])
    LIGHT_PINK_2         : Color256 = Color256(0xd78787, 174, "light-pink-2",          register=True, aliases=['color174', 'colour174', 'xterm174'])
    LIGHT_PINK_3         : Color256 = Color256(0x875f5f, 95,  "light-pink-3",          register=True, aliases=['color95', 'colour95', 'xterm95'])
    LIGHT_SALMON_1       : Color256 = Color256(0xffaf87, 216, "light-salmon-1",        register=True, aliases=['color216', 'colour216', 'xterm216'])
    LIGHT_SALMON_2       : Color256 = Color256(0xd7875f, 173, "light-salmon-2",        register=True, aliases=['color173', 'colour173', 'xterm173'])
    LIGHT_SALMON_3       : Color256 = Color256(0xaf875f, 137, "light-salmon-3",        register=True, aliases=['color137', 'colour137', 'xterm137'])
    LIGHT_SEA_GREEN      : Color256 = Color256(0x00afaf, 37,  "light-sea-green",       register=True, aliases=['color37', 'colour37', 'xterm37'])
    LIGHT_SKY_BLUE_1     : Color256 = Color256(0xafd7ff, 153, "light-sky-blue-1",      register=True, aliases=['color153', 'colour153', 'xterm153'])
    LIGHT_SKY_BLUE_2     : Color256 = Color256(0x87afd7, 110, "light-sky-blue-2",      register=True, aliases=['color110', 'colour110', 'xterm110'])
    LIGHT_SKY_BLUE_3     : Color256 = Color256(0x87afaf, 109, "light-sky-blue-3",      register=True, aliases=['color109', 'colour109', 'xterm109'])
    LIGHT_SLATE_BLUE     : Color256 = Color256(0x8787ff, 105, "light-slate-blue",      register=True, aliases=['color105', 'colour105', 'xterm105'])
    LIGHT_SLATE_GRAY     : Color256 = Color256(0x8787af, 103, "light-slate-gray",      register=True, aliases=['color103', 'colour103', 'xterm103'])
    LIGHT_STEEL_BLUE_1   : Color256 = Color256(0xd7d7ff, 189, "light-steel-blue-1",    register=True, aliases=['color189', 'colour189', 'xterm189'])
    LIGHT_STEEL_BLUE_2   : Color256 = Color256(0xafafff, 147, "light-steel-blue-2",    register=True, aliases=['color147', 'colour147', 'xterm147'])
    LIGHT_STEEL_BLUE_3   : Color256 = Color256(0xafafd7, 146, "light-steel-blue-3",    register=True, aliases=['color146', 'colour146', 'xterm146'])
    LIGHT_YELLOW_3       : Color256 = Color256(0xd7d7af, 187, "light-yellow-3",        register=True, aliases=['color187', 'colour187', 'xterm187'])
    MAGENTA_1            : Color256 = Color256(0xff00ff, 201, "magenta-1",             register=True, aliases=['color201', 'colour201', 'xterm201'])
    MAGENTA_2            : Color256 = Color256(0xff00d7, 200, "magenta-2",             register=True, aliases=['color200', 'colour200', 'xterm200'])
    MAGENTA_3            : Color256 = Color256(0xd700af, 163, "magenta-3",             register=True, aliases=['color163', 'colour163', 'xterm163'])
    MAGENTA_4            : Color256 = Color256(0xd700ff, 165, "magenta-4",             register=True, aliases=['color165', 'colour165', 'xterm165'])
    MAGENTA_5            : Color256 = Color256(0xd700d7, 164, "magenta-5",             register=True, aliases=['color164', 'colour164', 'xterm164'])
    MAGENTA_6            : Color256 = Color256(0xaf00af, 127, "magenta-6",             register=True, aliases=['color127', 'colour127', 'xterm127'])
    MEDIUM_ORCHID_1      : Color256 = Color256(0xff5fff, 207, "medium-orchid-1",       register=True, aliases=['color207', 'colour207', 'xterm207'])
    MEDIUM_ORCHID_2      : Color256 = Color256(0xd75fff, 171, "medium-orchid-2",       register=True, aliases=['color171', 'colour171', 'xterm171'])
    MEDIUM_ORCHID_3      : Color256 = Color256(0xaf5fd7, 134, "medium-orchid-3",       register=True, aliases=['color134', 'colour134', 'xterm134'])
    MEDIUM_ORCHID_4      : Color256 = Color256(0xaf5faf, 133, "medium-orchid-4",       register=True, aliases=['color133', 'colour133', 'xterm133'])
    MEDIUM_PURPLE_1      : Color256 = Color256(0xaf87ff, 141, "medium-purple-1",       register=True, aliases=['color141', 'colour141', 'xterm141'])
    MEDIUM_PURPLE_2      : Color256 = Color256(0xaf5fff, 135, "medium-purple-2",       register=True, aliases=['color135', 'colour135', 'xterm135'])
    MEDIUM_PURPLE_3      : Color256 = Color256(0xaf87d7, 140, "medium-purple-3",       register=True, aliases=['color140', 'colour140', 'xterm140'])
    MEDIUM_PURPLE_4      : Color256 = Color256(0x8787d7, 104, "medium-purple-4",       register=True, aliases=['color104', 'colour104', 'xterm104'])
    MEDIUM_PURPLE_5      : Color256 = Color256(0x875fd7, 98,  "medium-purple-5",       register=True, aliases=['color98', 'colour98', 'xterm98'])
    MEDIUM_PURPLE_6      : Color256 = Color256(0x875faf, 97,  "medium-purple-6",       register=True, aliases=['color97', 'colour97', 'xterm97'])
    MEDIUM_PURPLE_7      : Color256 = Color256(0x5f5f87, 60,  "medium-purple-7",       register=True, aliases=['color60', 'colour60', 'xterm60'])
    MEDIUM_SPRING_GREEN  : Color256 = Color256(0x00ffaf, 49,  "medium-spring-green",   register=True, aliases=['color49', 'colour49', 'xterm49'])
    MEDIUM_TURQUOISE     : Color256 = Color256(0x5fd7d7, 80,  "medium-turquoise",      register=True, aliases=['color80', 'colour80', 'xterm80'])
    MEDIUM_VIOLET_RED    : Color256 = Color256(0xaf0087, 126, "medium-violet-red",     register=True, aliases=['color126', 'colour126', 'xterm126'])
    MISTY_ROSE_1         : Color256 = Color256(0xffd7d7, 224, "misty-rose-1",          register=True, aliases=['color224', 'colour224', 'xterm224'])
    MISTY_ROSE_3         : Color256 = Color256(0xd7afaf, 181, "misty-rose-3",          register=True, aliases=['color181', 'colour181', 'xterm181'])
    NAVAJO_WHITE_1       : Color256 = Color256(0xffd7af, 223, "navajo-white-1",        register=True, aliases=['color223', 'colour223', 'xterm223'])
    NAVAJO_WHITE_3       : Color256 = Color256(0xafaf87, 144, "navajo-white-3",        register=True, aliases=['color144', 'colour144', 'xterm144'])
    NAVY_BLUE            : Color256 = Color256(0x00005f, 17,  "navy-blue",             register=True, aliases=['color17', 'colour17', 'xterm17'])
    ORANGE_1             : Color256 = Color256(0xffaf00, 214, "orange-1",              register=True, aliases=['color214', 'colour214', 'xterm214'])
    ORANGE_2             : Color256 = Color256(0xd78700, 172, "orange-2",              register=True, aliases=['color172', 'colour172', 'xterm172'])
    ORANGE_3             : Color256 = Color256(0x875f00, 94,  "orange-3",              register=True, aliases=['color94', 'colour94', 'xterm94'])
    ORANGE_4             : Color256 = Color256(0x5f5f00, 58,  "orange-4",              register=True, aliases=['color58', 'colour58', 'xterm58'])
    ORANGE_RED_1         : Color256 = Color256(0xff5f00, 202, "orange-red-1",          register=True, aliases=['color202', 'colour202', 'xterm202'])
    ORCHID_1             : Color256 = Color256(0xff87ff, 213, "orchid-1",              register=True, aliases=['color213', 'colour213', 'xterm213'])
    ORCHID_2             : Color256 = Color256(0xff87d7, 212, "orchid-2",              register=True, aliases=['color212', 'colour212', 'xterm212'])
    ORCHID_3             : Color256 = Color256(0xd75fd7, 170, "orchid-3",              register=True, aliases=['color170', 'colour170', 'xterm170'])
    PALE_GREEN_1         : Color256 = Color256(0x87ffaf, 121, "pale-green-1",          register=True, aliases=['color121', 'colour121', 'xterm121'])
    PALE_GREEN_2         : Color256 = Color256(0xafff87, 156, "pale-green-2",          register=True, aliases=['color156', 'colour156', 'xterm156'])
    PALE_GREEN_3         : Color256 = Color256(0x87d787, 114, "pale-green-3",          register=True, aliases=['color114', 'colour114', 'xterm114'])
    PALE_GREEN_4         : Color256 = Color256(0x5fd75f, 77,  "pale-green-4",          register=True, aliases=['color77', 'colour77', 'xterm77'])
    PALE_TURQUOISE_1     : Color256 = Color256(0xafffff, 159, "pale-turquoise-1",      register=True, aliases=['color159', 'colour159', 'xterm159'])
    PALE_TURQUOISE_4     : Color256 = Color256(0x5f8787, 66,  "pale-turquoise-4",      register=True, aliases=['color66', 'colour66', 'xterm66'])
    PALE_VIOLET_RED_1    : Color256 = Color256(0xff87af, 211, "pale-violet-red-1",     register=True, aliases=['color211', 'colour211', 'xterm211'])
    PINK_1               : Color256 = Color256(0xffafd7, 218, "pink-1",                register=True, aliases=['color218', 'colour218', 'xterm218'])
    PINK_3               : Color256 = Color256(0xd787af, 175, "pink-3",                register=True, aliases=['color175', 'colour175', 'xterm175'])
    PLUM_1               : Color256 = Color256(0xffafff, 219, "plum-1",                register=True, aliases=['color219', 'colour219', 'xterm219'])
    PLUM_2               : Color256 = Color256(0xd7afff, 183, "plum-2",                register=True, aliases=['color183', 'colour183', 'xterm183'])
    PLUM_3               : Color256 = Color256(0xd787d7, 176, "plum-3",                register=True, aliases=['color176', 'colour176', 'xterm176'])
    PLUM_4               : Color256 = Color256(0x875f87, 96,  "plum-4",                register=True, aliases=['color96', 'colour96', 'xterm96'])
    PURPLE               : Color256 = Color256(0xaf00ff, 129, "purple",                register=True, aliases=['color129', 'colour129', 'xterm129'])
    PURPLE_2             : Color256 = Color256(0x8700ff, 93,  "purple-2",              register=True, aliases=['color93', 'colour93', 'xterm93'])
    PURPLE_3             : Color256 = Color256(0x5f00d7, 56,  "purple-3",              register=True, aliases=['color56', 'colour56', 'xterm56'])
    PURPLE_4             : Color256 = Color256(0x5f00af, 55,  "purple-4",              register=True, aliases=['color55', 'colour55', 'xterm55'])
    PURPLE_6             : Color256 = Color256(0x5f0087, 54,  "purple-6",              register=True, aliases=['color54', 'colour54', 'xterm54'])
    RED_1                : Color256 = Color256(0xff0000, 196, "red-1",                 register=True, aliases=['color196', 'colour196', 'xterm196'])
    RED_3                : Color256 = Color256(0xd70000, 160, "red-3",                 register=True, aliases=['color160', 'colour160', 'xterm160'])
    RED_4                : Color256 = Color256(0xaf0000, 124, "red-4",                 register=True, aliases=['color124', 'colour124', 'xterm124'])
    ROSY_BROWN           : Color256 = Color256(0xaf8787, 138, "rosy-brown",            register=True, aliases=['color138', 'colour138', 'xterm138'])
    ROYAL_BLUE_1         : Color256 = Color256(0x5f5fff, 63,  "royal-blue-1",          register=True, aliases=['color63', 'colour63', 'xterm63'])
    SALMON_1             : Color256 = Color256(0xff875f, 209, "salmon-1",              register=True, aliases=['color209', 'colour209', 'xterm209'])
    SANDY_BROWN          : Color256 = Color256(0xffaf5f, 215, "sandy-brown",           register=True, aliases=['color215', 'colour215', 'xterm215'])
    SEA_GREEN_1          : Color256 = Color256(0x5fffaf, 85,  "sea-green-1",           register=True, aliases=['color85', 'colour85', 'xterm85'])
    SEA_GREEN_2          : Color256 = Color256(0x5fff87, 84,  "sea-green-2",           register=True, aliases=['color84', 'colour84', 'xterm84'])
    SEA_GREEN_3          : Color256 = Color256(0x5fd787, 78,  "sea-green-3",           register=True, aliases=['color78', 'colour78', 'xterm78'])
    SEA_GREEN_4          : Color256 = Color256(0x5fff5f, 83,  "sea-green-4",           register=True, aliases=['color83', 'colour83', 'xterm83'])
    SKY_BLUE_1           : Color256 = Color256(0x87d7ff, 117, "sky-blue-1",            register=True, aliases=['color117', 'colour117', 'xterm117'])
    SKY_BLUE_2           : Color256 = Color256(0x87afff, 111, "sky-blue-2",            register=True, aliases=['color111', 'colour111', 'xterm111'])
    SKY_BLUE_3           : Color256 = Color256(0x5fafd7, 74,  "sky-blue-3",            register=True, aliases=['color74', 'colour74', 'xterm74'])
    SLATE_BLUE_1         : Color256 = Color256(0x875fff, 99,  "slate-blue-1",          register=True, aliases=['color99', 'colour99', 'xterm99'])
    SLATE_BLUE_2         : Color256 = Color256(0x5f5fd7, 62,  "slate-blue-2",          register=True, aliases=['color62', 'colour62', 'xterm62'])
    SLATE_BLUE_3         : Color256 = Color256(0x5f5faf, 61,  "slate-blue-3",          register=True, aliases=['color61', 'colour61', 'xterm61'])
    SPRING_GREEN_1       : Color256 = Color256(0x00ff87, 48,  "spring-green-1",        register=True, aliases=['color48', 'colour48', 'xterm48'])
    SPRING_GREEN_2       : Color256 = Color256(0x00ff5f, 47,  "spring-green-2",        register=True, aliases=['color47', 'colour47', 'xterm47'])
    SPRING_GREEN_3       : Color256 = Color256(0x00d75f, 41,  "spring-green-3",        register=True, aliases=['color41', 'colour41', 'xterm41'])
    SPRING_GREEN_4       : Color256 = Color256(0x00875f, 29,  "spring-green-4",        register=True, aliases=['color29', 'colour29', 'xterm29'])
    SPRING_GREEN_5       : Color256 = Color256(0x00af5f, 35,  "spring-green-5",        register=True, aliases=['color35', 'colour35', 'xterm35'])
    SPRING_GREEN_6       : Color256 = Color256(0x00d787, 42,  "spring-green-6",        register=True, aliases=['color42', 'colour42', 'xterm42'])
    STEEL_BLUE           : Color256 = Color256(0x5f87af, 67,  "steel-blue",            register=True, aliases=['color67', 'colour67', 'xterm67'])
    STEEL_BLUE_1         : Color256 = Color256(0x5fd7ff, 81,  "steel-blue-1",          register=True, aliases=['color81', 'colour81', 'xterm81'])
    STEEL_BLUE_2         : Color256 = Color256(0x5fafff, 75,  "steel-blue-2",          register=True, aliases=['color75', 'colour75', 'xterm75'])
    STEEL_BLUE_3         : Color256 = Color256(0x5f87d7, 68,  "steel-blue-3",          register=True, aliases=['color68', 'colour68', 'xterm68'])
    TAN                  : Color256 = Color256(0xd7af87, 180, "tan",                   register=True, aliases=['color180', 'colour180', 'xterm180'])
    THISTLE_1            : Color256 = Color256(0xffd7ff, 225, "thistle-1",             register=True, aliases=['color225', 'colour225', 'xterm225'])
    THISTLE_3            : Color256 = Color256(0xd7afd7, 182, "thistle-3",             register=True, aliases=['color182', 'colour182', 'xterm182'])
    TURQUOISE_2          : Color256 = Color256(0x00d7ff, 45,  "turquoise-2",           register=True, aliases=['color45', 'colour45', 'xterm45'])
    TURQUOISE_4          : Color256 = Color256(0x008787, 30,  "turquoise-4",           register=True, aliases=['color30', 'colour30', 'xterm30'])
    VIOLET               : Color256 = Color256(0xd787ff, 177, "violet",                register=True, aliases=['color177', 'colour177', 'xterm177'])
    WHEAT_1              : Color256 = Color256(0xffffaf, 229, "wheat-1",               register=True, aliases=['color229', 'colour229', 'xterm229'])
    WHEAT_4              : Color256 = Color256(0x87875f, 101, "wheat-4",               register=True, aliases=['color101', 'colour101', 'xterm101'])
    YELLOW_1             : Color256 = Color256(0xffff00, 226, "yellow-1",              register=True, aliases=['color226', 'colour226', 'xterm226'])
    YELLOW_2             : Color256 = Color256(0xd7ff00, 190, "yellow-2",              register=True, aliases=['color190', 'colour190', 'xterm190'])
    YELLOW_3             : Color256 = Color256(0xd7d700, 184, "yellow-3",              register=True, aliases=['color184', 'colour184', 'xterm184'])
    YELLOW_4             : Color256 = Color256(0x87af00, 106, "yellow-4",              register=True, aliases=['color106', 'colour106', 'xterm106'])
    YELLOW_5             : Color256 = Color256(0xafd700, 148, "yellow-5",              register=True, aliases=['color148', 'colour148', 'xterm148'])
    YELLOW_6             : Color256 = Color256(0x878700, 100, "yellow-6",              register=True, aliases=['color100', 'colour100', 'xterm100'])
    # fmt: on


class _ColorValuesRGB(_ColorValues):
    """
    ColorValuesRGB
    """

    # fmt: off
    ABSOLUTE_ZERO                        : ColorRGB = lambda *_: ColorRGB(0x0048ba, "absolute-zero",                         register=True)
    ACAJOU                               : ColorRGB = lambda *_: ColorRGB(0x4c2f27, "acajou",                                register=True)
    ACID_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x8ffe09, "acid-green",                            register=True)
    ADOBE                                : ColorRGB = lambda *_: ColorRGB(0xbd6c48, "adobe",                                 register=True)
    AERO                                 : ColorRGB = lambda *_: ColorRGB(0x7cb9e8, "aero",                                  register=True)
    AERO_BLUE                            : ColorRGB = lambda *_: ColorRGB(0xc9ffe5, "aero-blue",                             register=True)
    AFRICAN_VIOLET                       : ColorRGB = lambda *_: ColorRGB(0xb284be, "african-violet",                        register=True)
    AIR_FORCE_BLUE                       : ColorRGB = lambda *_: ColorRGB(0x5d8aa8, "air-force-blue",                        register=True, variation_map={
             0x00308f:   "usaf" })
    AIR_SUPERIORITY_BLUE                 : ColorRGB = lambda *_: ColorRGB(0x72a0c1, "air-superiority-blue",                  register=True)
    ALABAMA_CRIMSON                      : ColorRGB = lambda *_: ColorRGB(0xaf002a, "alabama-crimson",                       register=True)
    ALABASTER                            : ColorRGB = lambda *_: ColorRGB(0xf2f0e6, "alabaster",                             register=True)
    ALEXANDRIA                           : ColorRGB = lambda *_: ColorRGB(0xff8f73, "alexandria",                            register=True)
    ALGAE                                : ColorRGB = lambda *_: ColorRGB(0x54ac68, "algae",                                 register=True)
    ALGAE_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x21c36f, "algae-green",                           register=True)
    ALICE_BLUE                           : ColorRGB = lambda *_: ColorRGB(0xf0f8ff, "alice-blue",                            register=True)
    ALIZARIN_CRIMSON                     : ColorRGB = lambda *_: ColorRGB(0xe32636, "alizarin-crimson",                      register=True)
    ALLOY_ORANGE                         : ColorRGB = lambda *_: ColorRGB(0xc46210, "alloy-orange",                          register=True)
    ALMOND                               : ColorRGB = lambda *_: ColorRGB(0xefdecd, "almond",                                register=True)
    ALMOST_BLACK                         : ColorRGB = lambda *_: ColorRGB(0x070d0d, "almost-black",                          register=True)
    ALOESWOOD_BROWN                      : ColorRGB = lambda *_: ColorRGB(0x5a6457, "aloeswood-brown",                       register=True)
    ALOEWOOD                             : ColorRGB = lambda *_: ColorRGB(0x6a432d, "aloewood",                              register=True)
    ALUMINUM                             : ColorRGB = lambda *_: ColorRGB(0xd6d6d6, "aluminum",                              register=True)
    ALUMINUM_FOIL                        : ColorRGB = lambda *_: ColorRGB(0xd2d9db, "aluminum-foil",                         register=True)
    AMARANTH                             : ColorRGB = lambda *_: ColorRGB(0xe52b50, "amaranth",                              register=True)
    AMARANTH_DEEP_PURPLE                 : ColorRGB = lambda *_: ColorRGB(0x9f2b68, "amaranth-deep-purple",                  register=True)
    AMARANTH_PINK                        : ColorRGB = lambda *_: ColorRGB(0xf19cbb, "amaranth-pink",                         register=True)
    AMARANTH_PURPLE                      : ColorRGB = lambda *_: ColorRGB(0xab274f, "amaranth-purple",                       register=True)
    AMARANTH_RED                         : ColorRGB = lambda *_: ColorRGB(0xd3212d, "amaranth-red",                          register=True)
    AMAZON                               : ColorRGB = lambda *_: ColorRGB(0x3b7a57, "amazon",                                register=True)
    AMBER                                : ColorRGB = lambda *_: ColorRGB(0xfeb308, "amber",                                 register=True, variation_map={
             0xca6924:   "kohaku-iro", 
             0xff7e00:   "sae-ece" })
    AMERICAN_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x3b3b6d, "american-blue",                         register=True)
    AMERICAN_BRONZE                      : ColorRGB = lambda *_: ColorRGB(0x391802, "american-bronze",                       register=True)
    AMERICAN_BROWN                       : ColorRGB = lambda *_: ColorRGB(0x804040, "american-brown",                        register=True)
    AMERICAN_GOLD                        : ColorRGB = lambda *_: ColorRGB(0xd3af37, "american-gold",                         register=True)
    AMERICAN_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x34b334, "american-green",                        register=True)
    AMERICAN_ORANGE                      : ColorRGB = lambda *_: ColorRGB(0xff8b00, "american-orange",                       register=True)
    AMERICAN_PINK                        : ColorRGB = lambda *_: ColorRGB(0xff9899, "american-pink",                         register=True)
    AMERICAN_PURPLE                      : ColorRGB = lambda *_: ColorRGB(0x431c53, "american-purple",                       register=True)
    AMERICAN_RED                         : ColorRGB = lambda *_: ColorRGB(0xb32134, "american-red",                          register=True)
    AMERICAN_ROSE                        : ColorRGB = lambda *_: ColorRGB(0xff033e, "american-rose",                         register=True)
    AMERICAN_SILVER                      : ColorRGB = lambda *_: ColorRGB(0xcfcfcf, "american-silver",                       register=True)
    AMERICAN_VIOLET                      : ColorRGB = lambda *_: ColorRGB(0x551b8c, "american-violet",                       register=True)
    AMERICAN_YELLOW                      : ColorRGB = lambda *_: ColorRGB(0xf2b400, "american-yellow",                       register=True)
    AMETHYST                             : ColorRGB = lambda *_: ColorRGB(0x9b5fc0, "amethyst",                              register=True)
    AMUR_CORK_TREE                       : ColorRGB = lambda *_: ColorRGB(0xf3c13a, "amur-cork-tree",                        register=True)
    ANTI_FLASH_WHITE                     : ColorRGB = lambda *_: ColorRGB(0xf2f3f4, "anti-flash-white",                      register=True)
    ANTIQUE_BRASS                        : ColorRGB = lambda *_: ColorRGB(0xcd9575, "antique-brass",                         register=True)
    ANTIQUE_BRONZE                       : ColorRGB = lambda *_: ColorRGB(0x665d1e, "antique-bronze",                        register=True)
    ANTIQUE_FUCHSIA                      : ColorRGB = lambda *_: ColorRGB(0x915c83, "antique-fuchsia",                       register=True)
    ANTIQUE_RUBY                         : ColorRGB = lambda *_: ColorRGB(0x841b2d, "antique-ruby",                          register=True)
    ANTIQUE_WHITE                        : ColorRGB = lambda *_: ColorRGB(0xfaebd7, "antique-white",                         register=True)
    APPLE                                : ColorRGB = lambda *_: ColorRGB(0x6ecb3c, "apple",                                 register=True)
    APPLE_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x76cd26, "apple-green",                           register=True)
    APRICOT                              : ColorRGB = lambda *_: ColorRGB(0xffb16d, "apricot",                               register=True)
    AQUA                                 : ColorRGB = lambda *_: ColorRGB(0x13eac9, "aqua",                                  register=True)
    AQUA_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x02d8e9, "aqua-blue",                             register=True, variation_map={
             0x86aba5:   "mizu-iro" })
    AQUA_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x12e193, "aqua-green",                            register=True)
    AQUA_MARINE                          : ColorRGB = lambda *_: ColorRGB(0x2ee8bb, "aqua-marine",                           register=True)
    AQUAMARINE                           : ColorRGB = lambda *_: ColorRGB(0x04d8b2, "aquamarine",                            register=True)
    ARCTIC_CHILL                         : ColorRGB = lambda *_: ColorRGB(0xb3f5ff, "arctic-chill",                          register=True)
    ARCTIC_LIME                          : ColorRGB = lambda *_: ColorRGB(0xd0ff14, "arctic-lime",                           register=True)
    ARGENT                               : ColorRGB = lambda *_: ColorRGB(0xc0c0c0, "argent",                                register=True)
    ARMY_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x4b5d16, "army-green",                            register=True)
    ARTICHOKE                            : ColorRGB = lambda *_: ColorRGB(0x8f9779, "artichoke",                             register=True)
    ARVO_BREEZE                          : ColorRGB = lambda *_: ColorRGB(0x4c9aff, "arvo-breeze",                           register=True)
    ARYLIDE_YELLOW                       : ColorRGB = lambda *_: ColorRGB(0xe9d66b, "arylide-yellow",                        register=True)
    ASDA_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x7dc242, "asda-green",                            register=True, variation_map={
             0x00dea4:   "1985", 
             0x80c197:   "1994", 
             0x66c992:   "1999", 
             0x32ad61:   "2002" })
    ASDA_ORANGE                          : ColorRGB = lambda *_: ColorRGB(0xc86500, "asda-orange",                           register=True, variation_map={
             0xca7309:   "1968" })
    ASH_GRAY                             : ColorRGB = lambda *_: ColorRGB(0xb2beb5, "ash-gray",                              register=True)
    ASPARAGUS                            : ColorRGB = lambda *_: ColorRGB(0x77ab56, "asparagus",                             register=True)
    ASPEN_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x687652, "aspen-green",                           register=True)
    ATENEO_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x003a6c, "ateneo-blue",                           register=True)
    ATOMIC_TANGERINE                     : ColorRGB = lambda *_: ColorRGB(0xff9966, "atomic-tangerine",                      register=True)
    AUBERGINE                            : ColorRGB = lambda *_: ColorRGB(0x3d0734, "aubergine",                             register=True)
    AUBURN                               : ColorRGB = lambda *_: ColorRGB(0x9a3001, "auburn",                                register=True)
    AUREOLIN                             : ColorRGB = lambda *_: ColorRGB(0xfdee00, "aureolin",                              register=True)
    AURO_METAL_SAURUS                    : ColorRGB = lambda *_: ColorRGB(0x6e7f80, "auro-metal-saurus",                     register=True)
    AVOCADO                              : ColorRGB = lambda *_: ColorRGB(0x90b134, "avocado",                               register=True)
    AVOCADO_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x87a922, "avocado-green",                         register=True)
    AWESOME                              : ColorRGB = lambda *_: ColorRGB(0xff2052, "awesome",                               register=True)
    AXOLOTL                              : ColorRGB = lambda *_: ColorRGB(0x63775b, "axolotl",                               register=True)
    AZUL                                 : ColorRGB = lambda *_: ColorRGB(0x1d5dec, "azul",                                  register=True)
    AZURE                                : ColorRGB = lambda *_: ColorRGB(0x069af3, "azure",                                 register=True)
    AZURE_MIST                           : ColorRGB = lambda *_: ColorRGB(0xf0ffff, "azure-mist",                            register=True)
    AZUREISH_WHITE                       : ColorRGB = lambda *_: ColorRGB(0xdbe9f4, "azureish-white",                        register=True)
    BABY_BLUE                            : ColorRGB = lambda *_: ColorRGB(0xa2cffe, "baby-blue",                             register=True)
    BABY_BLUE_EYES                       : ColorRGB = lambda *_: ColorRGB(0xa1caf1, "baby-blue-eyes",                        register=True)
    BABY_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x8cff9e, "baby-green",                            register=True)
    BABY_PINK                            : ColorRGB = lambda *_: ColorRGB(0xffb7ce, "baby-pink",                             register=True)
    BABY_POO                             : ColorRGB = lambda *_: ColorRGB(0xab9004, "baby-poo",                              register=True)
    BABY_POOP                            : ColorRGB = lambda *_: ColorRGB(0x937c00, "baby-poop",                             register=True)
    BABY_POOP_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x8f9805, "baby-poop-green",                       register=True)
    BABY_POWDER                          : ColorRGB = lambda *_: ColorRGB(0xfefefa, "baby-powder",                           register=True)
    BABY_PUKE_GREEN                      : ColorRGB = lambda *_: ColorRGB(0xb6c406, "baby-puke-green",                       register=True)
    BABY_PURPLE                          : ColorRGB = lambda *_: ColorRGB(0xca9bf7, "baby-purple",                           register=True)
    BABY_SHIT_BROWN                      : ColorRGB = lambda *_: ColorRGB(0xad900d, "baby-shit-brown",                       register=True)
    BABY_SHIT_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x889717, "baby-shit-green",                       register=True)
    BAIKO_BROWN                          : ColorRGB = lambda *_: ColorRGB(0x857c55, "baiko-brown",                           register=True)
    BAKER_MILLER_PINK                    : ColorRGB = lambda *_: ColorRGB(0xff91af, "baker-miller-pink",                     register=True)
    BALL_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x21abcd, "ball-blue",                             register=True)
    BAMBOO_CREAM                         : ColorRGB = lambda *_: ColorRGB(0xf2e096, "bamboo-cream",                          register=True)
    BANANA                               : ColorRGB = lambda *_: ColorRGB(0xffff7e, "banana",                                register=True)
    BANANA_MANIA                         : ColorRGB = lambda *_: ColorRGB(0xfae7b5, "banana-mania",                          register=True)
    BANANA_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xfafe4b, "banana-yellow",                         register=True)
    BANFF_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x83a9c1, "banff-blue",                            register=True)
    BANGLADESH_GREEN                     : ColorRGB = lambda *_: ColorRGB(0x006a4e, "bangladesh-green",                      register=True)
    BARBIE_PINK                          : ColorRGB = lambda *_: ColorRGB(0xfe46a5, "barbie-pink",                           register=True, variation_map={
             0xe85395:   "1959-1975", 
             0xd3419d:   "1975-1990", 
             0xf364a2:   "1990-1999", 
             0xf7238a:   "1999-2004", 
             0xff3988:   "2004-2005", 
             0xfc419a:   "2005-2009", 
             0xe0218a:   "pantone" })
    BARF_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x94ac02, "barf-green",                            register=True)
    BARN_RED                             : ColorRGB = lambda *_: ColorRGB(0x7c0a02, "barn-red",                              register=True)
    BARNEY                               : ColorRGB = lambda *_: ColorRGB(0xac1db8, "barney",                                register=True)
    BARNEY_PURPLE                        : ColorRGB = lambda *_: ColorRGB(0xa00498, "barney-purple",                         register=True)
    BATTERY_CHARGED_BLUE                 : ColorRGB = lambda *_: ColorRGB(0x1dacd6, "battery-charged-blue",                  register=True)
    BATTLESHIP_GRAY                      : ColorRGB = lambda *_: ColorRGB(0x848482, "battleship-gray",                       register=True)
    BATTLESHIP_GREY                      : ColorRGB = lambda *_: ColorRGB(0x6b7c85, "battleship-grey",                       register=True)
    BAYSIDE                              : ColorRGB = lambda *_: ColorRGB(0x5fc9bf, "bayside",                               register=True)
    BAZAAR                               : ColorRGB = lambda *_: ColorRGB(0x98777b, "bazaar",                                register=True)
    BDAZZLED_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x2e5894, "bdazzled-blue",                         register=True)
    BEAU_BLUE                            : ColorRGB = lambda *_: ColorRGB(0xbcd4e6, "beau-blue",                             register=True)
    BEAVER                               : ColorRGB = lambda *_: ColorRGB(0x9f8170, "beaver",                                register=True)
    BEER                                 : ColorRGB = lambda *_: ColorRGB(0xf28e1c, "beer",                                  register=True)
    BEGONIA                              : ColorRGB = lambda *_: ColorRGB(0xfa6e79, "begonia",                               register=True)
    BEIGE                                : ColorRGB = lambda *_: ColorRGB(0xe6daa6, "beige",                                 register=True)
    BELLFLOWER                           : ColorRGB = lambda *_: ColorRGB(0x5d3f6a, "bellflower",                            register=True)
    BERRY                                : ColorRGB = lambda *_: ColorRGB(0x990f4b, "berry",                                 register=True)
    BETEL_NUT_DYE                        : ColorRGB = lambda *_: ColorRGB(0x352925, "betel-nut-dye",                         register=True)
    BIG_DIP_O_RUBY                       : ColorRGB = lambda *_: ColorRGB(0x9c2542, "big-dip-o-ruby",                        register=True)
    BIG_FOOT_FEET                        : ColorRGB = lambda *_: ColorRGB(0xe88e5a, "big-foot-feet",                         register=True)
    BILE                                 : ColorRGB = lambda *_: ColorRGB(0xb5c306, "bile",                                  register=True)
    BIRCH_BROWN                          : ColorRGB = lambda *_: ColorRGB(0xb14a30, "birch-brown",                           register=True)
    BISQUE                               : ColorRGB = lambda *_: ColorRGB(0xffe4c4, "bisque",                                register=True)
    BISTRE                               : ColorRGB = lambda *_: ColorRGB(0x3d2b1f, "bistre",                                register=True)
    BISTRE_BROWN                         : ColorRGB = lambda *_: ColorRGB(0x967117, "bistre-brown",                          register=True)
    BITTER_LEMON                         : ColorRGB = lambda *_: ColorRGB(0xcae00d, "bitter-lemon",                          register=True)
    BITTER_LIME                          : ColorRGB = lambda *_: ColorRGB(0xbfff00, "bitter-lime",                           register=True)
    BITTERSWEET                          : ColorRGB = lambda *_: ColorRGB(0xfe6f5e, "bittersweet",                           register=True)
    BITTERSWEET_SHIMMER                  : ColorRGB = lambda *_: ColorRGB(0xbf4f51, "bittersweet-shimmer",                   register=True)
    BLACK                                : ColorRGB = lambda *_: ColorRGB(0x000000, "black",                                 register=True, variation_map={
             0x171412:   "kokushoku" })
    BLACK_BEAN                           : ColorRGB = lambda *_: ColorRGB(0x3d0c02, "black-bean",                            register=True)
    BLACK_CHESTNUT_OAK                   : ColorRGB = lambda *_: ColorRGB(0x252321, "black-chestnut-oak",                    register=True)
    BLACK_CHOCOLATE                      : ColorRGB = lambda *_: ColorRGB(0x1b1811, "black-chocolate",                       register=True)
    BLACK_COFFEE                         : ColorRGB = lambda *_: ColorRGB(0x3b2f2f, "black-coffee",                          register=True)
    BLACK_CORAL                          : ColorRGB = lambda *_: ColorRGB(0x54626f, "black-coral",                           register=True)
    BLACK_KALM                           : ColorRGB = lambda *_: ColorRGB(0x181a1e, "black-kalm",                            register=True)
    BLACK_KITE                           : ColorRGB = lambda *_: ColorRGB(0x351e1c, "black-kite",                            register=True)
    BLACK_LEATHER_JACKET                 : ColorRGB = lambda *_: ColorRGB(0x253529, "black-leather-jacket",                  register=True)
    BLACK_OLIVE                          : ColorRGB = lambda *_: ColorRGB(0x3b3c36, "black-olive",                           register=True)
    BLACK_PANIK                          : ColorRGB = lambda *_: ColorRGB(0x101112, "black-panik",                           register=True)
    BLACK_SHADOWS                        : ColorRGB = lambda *_: ColorRGB(0xbfafb2, "black-shadows",                         register=True)
    BLANCHE                              : ColorRGB = lambda *_: ColorRGB(0xa5adba, "blanche",                               register=True)
    BLANCHED_ALMOND                      : ColorRGB = lambda *_: ColorRGB(0xffebcd, "blanched-almond",                       register=True)
    BLAND                                : ColorRGB = lambda *_: ColorRGB(0xafa88b, "bland",                                 register=True)
    BLAST_OFF_BRONZE                     : ColorRGB = lambda *_: ColorRGB(0xa57164, "blast-off-bronze",                      register=True)
    BLEU_DE_FRANCE                       : ColorRGB = lambda *_: ColorRGB(0x318ce7, "bleu-de-france",                        register=True)
    BLING_BLING                          : ColorRGB = lambda *_: ColorRGB(0x6b778c, "bling-bling",                           register=True)
    BLIZZARD_BLUE                        : ColorRGB = lambda *_: ColorRGB(0xace5ee, "blizzard-blue",                         register=True)
    BLOND                                : ColorRGB = lambda *_: ColorRGB(0xfaf0be, "blond",                                 register=True)
    BLOOD                                : ColorRGB = lambda *_: ColorRGB(0x770001, "blood",                                 register=True, variation_map={
             0xa41313:   "animal", 
             0x630f0f:   "organ" })
    BLOOD_ORANGE                         : ColorRGB = lambda *_: ColorRGB(0xfe4b03, "blood-orange",                          register=True)
    BLOOD_RED                            : ColorRGB = lambda *_: ColorRGB(0x980002, "blood-red",                             register=True, variation_map={
             0xf35336:   "benihi" })
    BLUE                                 : ColorRGB = lambda *_: ColorRGB(0x0343df, "blue",                                  register=True, variation_map={
             0x1f75fe:   "crayola", 
             0x0093af:   "munsell", 
             0x0087bd:   "ncs", 
             0x0018a8:   "pantone", 
             0x333399:   "pigment", 
             0x0247fe:   "ryb" })
    BLUE_BELL                            : ColorRGB = lambda *_: ColorRGB(0xa2a2d0, "blue-bell",                             register=True)
    BLUE_BLACK_CRAYFISH                  : ColorRGB = lambda *_: ColorRGB(0x52593b, "blue-black-crayfish",                   register=True)
    BLUE_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x2242c7, "blue-blue",                             register=True)
    BLUE_BOLT                            : ColorRGB = lambda *_: ColorRGB(0x00b9fb, "blue-bolt",                             register=True)
    BLUE_COLA                            : ColorRGB = lambda *_: ColorRGB(0x0088dc, "blue-cola",                             register=True)
    BLUE_GRAY                            : ColorRGB = lambda *_: ColorRGB(0x6699cc, "blue-gray",                             register=True)
    BLUE_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x137e6d, "blue-green",                            register=True, variation_map={
             0x064e40:   "color-wheel", 
             0x3a6960:   "seiheki" })
    BLUE_GREY                            : ColorRGB = lambda *_: ColorRGB(0x607c8e, "blue-grey",                             register=True)
    BLUE_JEANS                           : ColorRGB = lambda *_: ColorRGB(0x5dadec, "blue-jeans",                            register=True)
    BLUE_KALM                            : ColorRGB = lambda *_: ColorRGB(0x5f819d, "blue-kalm",                             register=True)
    BLUE_LAGOON                          : ColorRGB = lambda *_: ColorRGB(0x4cb7a5, "blue-lagoon",                           register=True)
    BLUE_MAGENTA_VIOLET                  : ColorRGB = lambda *_: ColorRGB(0x553592, "blue-magenta-violet",                   register=True)
    BLUE_PANIK                           : ColorRGB = lambda *_: ColorRGB(0x5786fe, "blue-panik",                            register=True)
    BLUE_PURPLE                          : ColorRGB = lambda *_: ColorRGB(0x5729ce, "blue-purple",                           register=True)
    BLUE_RASPBERRY                       : ColorRGB = lambda *_: ColorRGB(0x0cbfe9, "blue-raspberry",                        register=True)
    BLUE_SAPPHIRE                        : ColorRGB = lambda *_: ColorRGB(0x126180, "blue-sapphire",                         register=True)
    BLUE_VIOLET                          : ColorRGB = lambda *_: ColorRGB(0x5d06e9, "blue-violet",                           register=True, variation_map={
             0x4d1a7f:   "color-wheel", 
             0x7366bd:   "crayola", 
             0x2b2028:   "shikon" })
    BLUE_WITH_A_HINT_OF_PURPLE           : ColorRGB = lambda *_: ColorRGB(0x533cc6, "blue-with-a-hint-of-purple",            register=True)
    BLUE_YONDER                          : ColorRGB = lambda *_: ColorRGB(0x5072a7, "blue-yonder",                           register=True)
    BLUEBERRY                            : ColorRGB = lambda *_: ColorRGB(0x464196, "blueberry",                             register=True)
    BLUEBONNET                           : ColorRGB = lambda *_: ColorRGB(0x1c1cf0, "bluebonnet",                            register=True)
    BLUEGREEN                            : ColorRGB = lambda *_: ColorRGB(0x017a79, "bluegreen",                             register=True)
    BLUEGREY                             : ColorRGB = lambda *_: ColorRGB(0x85a3b2, "bluegrey",                              register=True)
    BLUEY_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x2bb179, "bluey-green",                           register=True)
    BLUEY_GREY                           : ColorRGB = lambda *_: ColorRGB(0x89a0b0, "bluey-grey",                            register=True)
    BLUEY_PURPLE                         : ColorRGB = lambda *_: ColorRGB(0x6241c7, "bluey-purple",                          register=True)
    BLUISH                               : ColorRGB = lambda *_: ColorRGB(0x2976bb, "bluish",                                register=True)
    BLUISH_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x10a674, "bluish-green",                          register=True)
    BLUISH_GREY                          : ColorRGB = lambda *_: ColorRGB(0x748b97, "bluish-grey",                           register=True)
    BLUISH_PURPLE                        : ColorRGB = lambda *_: ColorRGB(0x703be7, "bluish-purple",                         register=True)
    BLURPLE                              : ColorRGB = lambda *_: ColorRGB(0x5539cc, "blurple",                               register=True)
    BLUSH                                : ColorRGB = lambda *_: ColorRGB(0xf29e8e, "blush",                                 register=True)
    BLUSH_PINK                           : ColorRGB = lambda *_: ColorRGB(0xfe828c, "blush-pink",                            register=True)
    BOILED_RED_BEAN_BROWN                : ColorRGB = lambda *_: ColorRGB(0x542d24, "boiled-red-bean-brown",                 register=True)
    BOLE                                 : ColorRGB = lambda *_: ColorRGB(0x79443b, "bole",                                  register=True)
    BONDI_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x0095b6, "bondi-blue",                            register=True)
    BONDI_SUNBURN                        : ColorRGB = lambda *_: ColorRGB(0xffbdad, "bondi-sunburn",                         register=True)
    BONE                                 : ColorRGB = lambda *_: ColorRGB(0xe3dac9, "bone",                                  register=True)
    BOOGER                               : ColorRGB = lambda *_: ColorRGB(0x9bb53c, "booger",                                register=True)
    BOOGER_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x96b403, "booger-green",                          register=True)
    BORDEAUX                             : ColorRGB = lambda *_: ColorRGB(0x7b002c, "bordeaux",                              register=True)
    BORED_ACCENT_GREEN                   : ColorRGB = lambda *_: ColorRGB(0xdde26a, "bored-accent-green",                    register=True)
    BORING_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x63b365, "boring-green",                          register=True)
    BOTTLE_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x044a05, "bottle-green",                          register=True)
    BOY_RED                              : ColorRGB = lambda *_: ColorRGB(0x0e9ca5, "boy-red",                               register=True)
    BOYSENBERRY                          : ColorRGB = lambda *_: ColorRGB(0x873260, "boysenberry",                           register=True)
    BRANDEIS_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x0070ff, "brandeis-blue",                         register=True)
    BRANDY                               : ColorRGB = lambda *_: ColorRGB(0x87413f, "brandy",                                register=True)
    BRASS                                : ColorRGB = lambda *_: ColorRGB(0xb5a642, "brass",                                 register=True)
    BRAVE_ORANGE                         : ColorRGB = lambda *_: ColorRGB(0xff631c, "brave-orange",                          register=True)
    BREWED_MUSTARD_BROWN                 : ColorRGB = lambda *_: ColorRGB(0xe68364, "brewed-mustard-brown",                  register=True)
    BRICK                                : ColorRGB = lambda *_: ColorRGB(0xa03623, "brick",                                 register=True)
    BRICK_ORANGE                         : ColorRGB = lambda *_: ColorRGB(0xc14a09, "brick-orange",                          register=True)
    BRICK_RED                            : ColorRGB = lambda *_: ColorRGB(0x8f1402, "brick-red",                             register=True)
    BRIGHT_AQUA                          : ColorRGB = lambda *_: ColorRGB(0x0bf9ea, "bright-aqua",                           register=True)
    BRIGHT_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x0165fc, "bright-blue",                           register=True)
    BRIGHT_CYAN                          : ColorRGB = lambda *_: ColorRGB(0x41fdfe, "bright-cyan",                           register=True)
    BRIGHT_GRAY                          : ColorRGB = lambda *_: ColorRGB(0xebecf0, "bright-gray",                           register=True)
    BRIGHT_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x01ff07, "bright-green",                          register=True)
    BRIGHT_LAVENDER                      : ColorRGB = lambda *_: ColorRGB(0xc760ff, "bright-lavender",                       register=True)
    BRIGHT_LIGHT_BLUE                    : ColorRGB = lambda *_: ColorRGB(0x26f7fd, "bright-light-blue",                     register=True)
    BRIGHT_LIGHT_GREEN                   : ColorRGB = lambda *_: ColorRGB(0x2dfe54, "bright-light-green",                    register=True)
    BRIGHT_LILAC                         : ColorRGB = lambda *_: ColorRGB(0xc95efb, "bright-lilac",                          register=True)
    BRIGHT_LIME                          : ColorRGB = lambda *_: ColorRGB(0x87fd05, "bright-lime",                           register=True)
    BRIGHT_LIME_GREEN                    : ColorRGB = lambda *_: ColorRGB(0x65fe08, "bright-lime-green",                     register=True)
    BRIGHT_MAGENTA                       : ColorRGB = lambda *_: ColorRGB(0xff08e8, "bright-magenta",                        register=True)
    BRIGHT_MAROON                        : ColorRGB = lambda *_: ColorRGB(0xc32148, "bright-maroon",                         register=True)
    BRIGHT_NAVY_BLUE                     : ColorRGB = lambda *_: ColorRGB(0x1974d2, "bright-navy-blue",                      register=True)
    BRIGHT_OLIVE                         : ColorRGB = lambda *_: ColorRGB(0x9cbb04, "bright-olive",                          register=True)
    BRIGHT_ORANGE                        : ColorRGB = lambda *_: ColorRGB(0xff5b00, "bright-orange",                         register=True)
    BRIGHT_PINK                          : ColorRGB = lambda *_: ColorRGB(0xfe01b1, "bright-pink",                           register=True)
    BRIGHT_PURPLE                        : ColorRGB = lambda *_: ColorRGB(0xbe03fd, "bright-purple",                         register=True)
    BRIGHT_RED                           : ColorRGB = lambda *_: ColorRGB(0xff000d, "bright-red",                            register=True)
    BRIGHT_SEA_GREEN                     : ColorRGB = lambda *_: ColorRGB(0x05ffa6, "bright-sea-green",                      register=True)
    BRIGHT_SKY_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x02ccfe, "bright-sky-blue",                       register=True)
    BRIGHT_TEAL                          : ColorRGB = lambda *_: ColorRGB(0x01f9c6, "bright-teal",                           register=True)
    BRIGHT_TURQUOISE                     : ColorRGB = lambda *_: ColorRGB(0x0ffef9, "bright-turquoise",                      register=True)
    BRIGHT_UBE                           : ColorRGB = lambda *_: ColorRGB(0xd19fe8, "bright-ube",                            register=True)
    BRIGHT_VIOLET                        : ColorRGB = lambda *_: ColorRGB(0xad0afd, "bright-violet",                         register=True)
    BRIGHT_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xfffd01, "bright-yellow",                         register=True, variation_map={
             0xffaa1d:   "crayola" })
    BRIGHT_YELLOW_GREEN                  : ColorRGB = lambda *_: ColorRGB(0x9dff00, "bright-yellow-green",                   register=True)
    BRILLIANT_AZURE                      : ColorRGB = lambda *_: ColorRGB(0x3399ff, "brilliant-azure",                       register=True)
    BRILLIANT_LAVENDER                   : ColorRGB = lambda *_: ColorRGB(0xf4bbff, "brilliant-lavender",                    register=True)
    BRILLIANT_ROSE                       : ColorRGB = lambda *_: ColorRGB(0xff55a3, "brilliant-rose",                        register=True)
    BRINK_PINK                           : ColorRGB = lambda *_: ColorRGB(0xfb607f, "brink-pink",                            register=True)
    BRITISH_RACING_GREEN                 : ColorRGB = lambda *_: ColorRGB(0x05480d, "british-racing-green",                  register=True)
    BRONZE                               : ColorRGB = lambda *_: ColorRGB(0xa87900, "bronze",                                register=True, variation_map={
             0x88540b:   "2", 
             0xb08d57:   "metallic" })
    BRONZE_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0x737000, "bronze-yellow",                         register=True)
    BROOKMERE_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x3d504b, "brookmere-green",                       register=True)
    BROWN                                : ColorRGB = lambda *_: ColorRGB(0x653700, "brown",                                 register=True, variation_map={
             0xaf593e:   "crayola", 
             0x964b00:   "traditional" })
    BROWN_CHOCOLATE                      : ColorRGB = lambda *_: ColorRGB(0x5f1933, "brown-chocolate",                       register=True)
    BROWN_COFFEE                         : ColorRGB = lambda *_: ColorRGB(0x4a2c2a, "brown-coffee",                          register=True)
    BROWN_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x706c11, "brown-green",                           register=True)
    BROWN_GREY                           : ColorRGB = lambda *_: ColorRGB(0x8d8468, "brown-grey",                            register=True)
    BROWN_NOSE                           : ColorRGB = lambda *_: ColorRGB(0x6b4423, "brown-nose",                            register=True)
    BROWN_ORANGE                         : ColorRGB = lambda *_: ColorRGB(0xb96902, "brown-orange",                          register=True)
    BROWN_RAT_GRAY                       : ColorRGB = lambda *_: ColorRGB(0x4b3c39, "brown-rat-gray",                        register=True)
    BROWN_RED                            : ColorRGB = lambda *_: ColorRGB(0x922b05, "brown-red",                             register=True)
    BROWN_SUGAR                          : ColorRGB = lambda *_: ColorRGB(0xaf6e4d, "brown-sugar",                           register=True)
    BROWN_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xb29705, "brown-yellow",                          register=True)
    BROWNISH                             : ColorRGB = lambda *_: ColorRGB(0x9c6d57, "brownish",                              register=True)
    BROWNISH_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x6a6e09, "brownish-green",                        register=True)
    BROWNISH_GREY                        : ColorRGB = lambda *_: ColorRGB(0x86775f, "brownish-grey",                         register=True)
    BROWNISH_ORANGE                      : ColorRGB = lambda *_: ColorRGB(0xcb7723, "brownish-orange",                       register=True)
    BROWNISH_PINK                        : ColorRGB = lambda *_: ColorRGB(0xc27e79, "brownish-pink",                         register=True)
    BROWNISH_PURPLE                      : ColorRGB = lambda *_: ColorRGB(0x76424e, "brownish-purple",                       register=True)
    BROWNISH_RED                         : ColorRGB = lambda *_: ColorRGB(0x9e3623, "brownish-red",                          register=True)
    BROWNISH_YELLOW                      : ColorRGB = lambda *_: ColorRGB(0xc9b003, "brownish-yellow",                       register=True)
    BROWNY_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x6f6c0a, "browny-green",                          register=True)
    BROWNY_ORANGE                        : ColorRGB = lambda *_: ColorRGB(0xca6b02, "browny-orange",                         register=True)
    BRUISE                               : ColorRGB = lambda *_: ColorRGB(0x7e4071, "bruise",                                register=True)
    BRUNSWICK_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x1b4d3e, "brunswick-green",                       register=True)
    BRUSHWOOD_DYED                       : ColorRGB = lambda *_: ColorRGB(0x8c5939, "brushwood-dyed",                        register=True)
    BUBBLE_GUM                           : ColorRGB = lambda *_: ColorRGB(0xffc1cc, "bubble-gum",                            register=True)
    BUBBLE_GUM_PINK                      : ColorRGB = lambda *_: ColorRGB(0xff69af, "bubble-gum-pink",                       register=True)
    BUBBLEGUM                            : ColorRGB = lambda *_: ColorRGB(0xff6cb5, "bubblegum",                             register=True)
    BUBBLEGUM_PINK                       : ColorRGB = lambda *_: ColorRGB(0xfe83cc, "bubblegum-pink",                        register=True)
    BUBBLES                              : ColorRGB = lambda *_: ColorRGB(0xe7feff, "bubbles",                               register=True)
    BUD_GREEN                            : ColorRGB = lambda *_: ColorRGB(0x7bb661, "bud-green",                             register=True)
    BUFF                                 : ColorRGB = lambda *_: ColorRGB(0xfef69e, "buff",                                  register=True)
    BULGARIAN_ROSE                       : ColorRGB = lambda *_: ColorRGB(0x480607, "bulgarian-rose",                        register=True)
    BURGUNDY                             : ColorRGB = lambda *_: ColorRGB(0x610023, "burgundy",                              register=True)
    BURLYWOOD                            : ColorRGB = lambda *_: ColorRGB(0xdeb887, "burlywood",                             register=True)
    BURNISHED_BROWN                      : ColorRGB = lambda *_: ColorRGB(0xa17a74, "burnished-brown",                       register=True)
    BURNT_ORANGE                         : ColorRGB = lambda *_: ColorRGB(0xc04e01, "burnt-orange",                          register=True)
    BURNT_RED                            : ColorRGB = lambda *_: ColorRGB(0x9f2305, "burnt-red",                             register=True)
    BURNT_SIENA                          : ColorRGB = lambda *_: ColorRGB(0xb75203, "burnt-siena",                           register=True)
    BURNT_SIENNA                         : ColorRGB = lambda *_: ColorRGB(0xb04e0f, "burnt-sienna",                          register=True)
    BURNT_UMBER                          : ColorRGB = lambda *_: ColorRGB(0xa0450e, "burnt-umber",                           register=True)
    BURNT_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xd5ab09, "burnt-yellow",                          register=True)
    BURPLE                               : ColorRGB = lambda *_: ColorRGB(0x6832e3, "burple",                                register=True)
    BUTTER                               : ColorRGB = lambda *_: ColorRGB(0xffff81, "butter",                                register=True)
    BUTTER_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xfffd74, "butter-yellow",                         register=True)
    BUTTERSCOTCH                         : ColorRGB = lambda *_: ColorRGB(0xfdb147, "butterscotch",                          register=True)
    BUTTON_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x24a0ed, "button-blue",                           register=True)
    BYZANTINE                            : ColorRGB = lambda *_: ColorRGB(0xbd33a4, "byzantine",                             register=True)
    BYZANTIUM                            : ColorRGB = lambda *_: ColorRGB(0x702963, "byzantium",                             register=True)
    CADET                                : ColorRGB = lambda *_: ColorRGB(0x536872, "cadet",                                 register=True)
    CADET_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x4e7496, "cadet-blue",                            register=True, variation_map={
             0xa9b2c3:   "crayola" })
    CADET_GRAY                           : ColorRGB = lambda *_: ColorRGB(0x91a3b0, "cadet-gray",                            register=True)
    CADMIUM_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x0a1195, "cadmium-blue",                          register=True)
    CADMIUM_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x006b3c, "cadmium-green",                         register=True)
    CADMIUM_ORANGE                       : ColorRGB = lambda *_: ColorRGB(0xed872d, "cadmium-orange",                        register=True)
    CADMIUM_PURPLE                       : ColorRGB = lambda *_: ColorRGB(0xb60c26, "cadmium-purple",                        register=True)
    CADMIUM_RED                          : ColorRGB = lambda *_: ColorRGB(0xe30022, "cadmium-red",                           register=True)
    CADMIUM_VIOLET                       : ColorRGB = lambda *_: ColorRGB(0x7f3e98, "cadmium-violet",                        register=True)
    CADMIUM_YELLOW                       : ColorRGB = lambda *_: ColorRGB(0xfff600, "cadmium-yellow",                        register=True)
    CAFE_AU_LAIT                         : ColorRGB = lambda *_: ColorRGB(0xa67b5b, "cafe-au-lait",                          register=True)
    CAFE_NOIR                            : ColorRGB = lambda *_: ColorRGB(0x4b3621, "cafe-noir",                             register=True)
    CAL_POLY_POMONA_GREEN                : ColorRGB = lambda *_: ColorRGB(0x1e4d2b, "cal-poly-pomona-green",                 register=True)
    CALAMANSI                            : ColorRGB = lambda *_: ColorRGB(0xfcffa4, "calamansi",                             register=True)
    CAMBRIDGE_BLUE                       : ColorRGB = lambda *_: ColorRGB(0xa3c1ad, "cambridge-blue",                        register=True)
    CAMEL                                : ColorRGB = lambda *_: ColorRGB(0xc69f59, "camel",                                 register=True)
    CAMEO_PINK                           : ColorRGB = lambda *_: ColorRGB(0xefbbcc, "cameo-pink",                            register=True)
    CAMO                                 : ColorRGB = lambda *_: ColorRGB(0x7f8f4e, "camo",                                  register=True)
    CAMO_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x526525, "camo-green",                            register=True)
    CAMOUFLAGE_GREEN                     : ColorRGB = lambda *_: ColorRGB(0x4b6113, "camouflage-green",                      register=True)
    CANARY                               : ColorRGB = lambda *_: ColorRGB(0xfdff63, "canary",                                register=True)
    CANARY_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xfffe40, "canary-yellow",                         register=True)
    CANDY_APPLE_RED                      : ColorRGB = lambda *_: ColorRGB(0xff0800, "candy-apple-red",                       register=True)
    CANDY_PINK                           : ColorRGB = lambda *_: ColorRGB(0xff63e9, "candy-pink",                            register=True)
    CAPE_JASMINE_OR_GARDENIA             : ColorRGB = lambda *_: ColorRGB(0xffb95a, "cape-jasmine-or-gardenia",              register=True)
    CAPRI                                : ColorRGB = lambda *_: ColorRGB(0x00bfff, "capri",                                 register=True)
    CAPUT_MORTUUM                        : ColorRGB = lambda *_: ColorRGB(0x592720, "caput-mortuum",                         register=True)
    CARAMEL                              : ColorRGB = lambda *_: ColorRGB(0xaf6f09, "caramel",                               register=True)
    CARDINAL                             : ColorRGB = lambda *_: ColorRGB(0xc41e3a, "cardinal",                              register=True)
    CARIBBEAN_BLUE                       : ColorRGB = lambda *_: ColorRGB(0x1ac1dd, "caribbean-blue",                        register=True)
    CARIBBEAN_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x00cc99, "caribbean-green",                       register=True)
    CARMINE                              : ColorRGB = lambda *_: ColorRGB(0x9d0216, "carmine",                               register=True, variation_map={
             0xd70040:   "m-p" })
    CARMINE_PINK                         : ColorRGB = lambda *_: ColorRGB(0xeb4c42, "carmine-pink",                          register=True)
    CARMINE_RED                          : ColorRGB = lambda *_: ColorRGB(0xff0038, "carmine-red",                           register=True)
    CARNATION                            : ColorRGB = lambda *_: ColorRGB(0xfd798f, "carnation",                             register=True)
    CARNATION_PINK                       : ColorRGB = lambda *_: ColorRGB(0xff7fa7, "carnation-pink",                        register=True)
    CARNELIAN                            : ColorRGB = lambda *_: ColorRGB(0xb31b1b, "carnelian",                             register=True)
    CAROLINA_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x8ab8fe, "carolina-blue",                         register=True)
    CARROT_ORANGE                        : ColorRGB = lambda *_: ColorRGB(0xed9121, "carrot-orange",                         register=True)
    CASTLETON_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x00563f, "castleton-green",                       register=True)
    CATALINA_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x062a78, "catalina-blue",                         register=True)
    CATAWBA                              : ColorRGB = lambda *_: ColorRGB(0x703642, "catawba",                               register=True)
    CATTAIL                              : ColorRGB = lambda *_: ColorRGB(0xb64925, "cattail",                               register=True)
    CEDAR_CHEST                          : ColorRGB = lambda *_: ColorRGB(0xc95a49, "cedar-chest",                           register=True)
    CEIL                                 : ColorRGB = lambda *_: ColorRGB(0x92a1cf, "ceil",                                  register=True)
    CELADON                              : ColorRGB = lambda *_: ColorRGB(0xbefdb7, "celadon",                               register=True, variation_map={
             0x819c8b:   "seiji-iro" })
    CELADON_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x007ba7, "celadon-blue",                          register=True)
    CELADON_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x2f847c, "celadon-green",                         register=True)
    CELERY                               : ColorRGB = lambda *_: ColorRGB(0xc1fd95, "celery",                                register=True)
    CELESTE                              : ColorRGB = lambda *_: ColorRGB(0xb2ffff, "celeste",                               register=True)
    CELESTIAL_BLUE                       : ColorRGB = lambda *_: ColorRGB(0x4997d0, "celestial-blue",                        register=True)
    CELTIC_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x246bce, "celtic-blue",                           register=True)
    CEMENT                               : ColorRGB = lambda *_: ColorRGB(0xa5a391, "cement",                                register=True)
    CERISE                               : ColorRGB = lambda *_: ColorRGB(0xde0c62, "cerise",                                register=True)
    CERISE_PINK                          : ColorRGB = lambda *_: ColorRGB(0xec3b83, "cerise-pink",                           register=True)
    CERULEAN                             : ColorRGB = lambda *_: ColorRGB(0x0485d1, "cerulean",                              register=True)
    CERULEAN_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x056eee, "cerulean-blue",                         register=True)
    CERULEAN_FROST                       : ColorRGB = lambda *_: ColorRGB(0x6d9bc3, "cerulean-frost",                        register=True)
    CETACEAN_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x001440, "cetacean-blue",                         register=True)
    CG_BLUE                              : ColorRGB = lambda *_: ColorRGB(0x007aa5, "cg-blue",                               register=True)
    CG_RED                               : ColorRGB = lambda *_: ColorRGB(0xe03c31, "cg-red",                                register=True)
    CHAMOISEE                            : ColorRGB = lambda *_: ColorRGB(0xa0785a, "chamoisee",                             register=True)
    CHAMPAGNE                            : ColorRGB = lambda *_: ColorRGB(0xf7e7ce, "champagne",                             register=True)
    CHAMPAGNE_PINK                       : ColorRGB = lambda *_: ColorRGB(0xf1ddcf, "champagne-pink",                        register=True)
    CHARCOAL                             : ColorRGB = lambda *_: ColorRGB(0x343837, "charcoal",                              register=True)
    CHARCOAL_GREY                        : ColorRGB = lambda *_: ColorRGB(0x3c4142, "charcoal-grey",                         register=True)
    CHARLESTON_GREEN                     : ColorRGB = lambda *_: ColorRGB(0x232b2b, "charleston-green",                      register=True)
    CHARM                                : ColorRGB = lambda *_: ColorRGB(0xd0748b, "charm",                                 register=True)
    CHARM_PINK                           : ColorRGB = lambda *_: ColorRGB(0xe68fac, "charm-pink",                            register=True)
    CHARTREUSE                           : ColorRGB = lambda *_: ColorRGB(0xc1f80a, "chartreuse",                            register=True, variation_map={
             0xdfff00:   "traditional" })
    CHEESE                               : ColorRGB = lambda *_: ColorRGB(0xffa600, "cheese",                                register=True)
    CHEEZY_BLASTERS                      : ColorRGB = lambda *_: ColorRGB(0xff991f, "cheezy-blasters",                       register=True)
    CHERRY                               : ColorRGB = lambda *_: ColorRGB(0xcf0234, "cherry",                                register=True)
    CHERRY_BLOSSOM                       : ColorRGB = lambda *_: ColorRGB(0xfcc9b9, "cherry-blossom",                        register=True)
    CHERRY_BLOSSOM_MOUSE                 : ColorRGB = lambda *_: ColorRGB(0xac8181, "cherry-blossom-mouse",                  register=True)
    CHERRY_BLOSSOM_PINK                  : ColorRGB = lambda *_: ColorRGB(0xffb7c5, "cherry-blossom-pink",                   register=True)
    CHERRY_RED                           : ColorRGB = lambda *_: ColorRGB(0xf7022a, "cherry-red",                            register=True)
    CHESTNUT                             : ColorRGB = lambda *_: ColorRGB(0x742802, "chestnut",                              register=True)
    CHESTNUT_LEATHER_BROWN               : ColorRGB = lambda *_: ColorRGB(0x60281e, "chestnut-leather-brown",                register=True)
    CHICKADEE                            : ColorRGB = lambda *_: ColorRGB(0xffc34d, "chickadee",                             register=True)
    CHINA_PINK                           : ColorRGB = lambda *_: ColorRGB(0xde6fa1, "china-pink",                            register=True)
    CHINA_ROSE                           : ColorRGB = lambda *_: ColorRGB(0xa8516e, "china-rose",                            register=True)
    CHINESE_BLACK                        : ColorRGB = lambda *_: ColorRGB(0x141414, "chinese-black",                         register=True)
    CHINESE_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x365194, "chinese-blue",                          register=True)
    CHINESE_BRONZE                       : ColorRGB = lambda *_: ColorRGB(0xcd8032, "chinese-bronze",                        register=True)
    CHINESE_BROWN                        : ColorRGB = lambda *_: ColorRGB(0xab381f, "chinese-brown",                         register=True)
    CHINESE_GOLD                         : ColorRGB = lambda *_: ColorRGB(0xcc9900, "chinese-gold",                          register=True)
    CHINESE_GREEN                        : ColorRGB = lambda *_: ColorRGB(0xd0db61, "chinese-green",                         register=True)
    CHINESE_ORANGE                       : ColorRGB = lambda *_: ColorRGB(0xf37042, "chinese-orange",                        register=True)
    CHINESE_PINK                         : ColorRGB = lambda *_: ColorRGB(0xde70a1, "chinese-pink",                          register=True)
    CHINESE_PURPLE                       : ColorRGB = lambda *_: ColorRGB(0x720b98, "chinese-purple",                        register=True)
    CHINESE_RED                          : ColorRGB = lambda *_: ColorRGB(0xaa381e, "chinese-red",                           register=True, variation_map={
             0xcd071e:   "2" })
    CHINESE_SILVER                       : ColorRGB = lambda *_: ColorRGB(0xcccccc, "chinese-silver",                        register=True)
    CHINESE_VIOLET                       : ColorRGB = lambda *_: ColorRGB(0x856088, "chinese-violet",                        register=True)
    CHINESE_WHITE                        : ColorRGB = lambda *_: ColorRGB(0xe2e5de, "chinese-white",                         register=True)
    CHINESE_YELLOW                       : ColorRGB = lambda *_: ColorRGB(0xffb200, "chinese-yellow",                        register=True)
    CHINESE_YELLOW_TEA                   : ColorRGB = lambda *_: ColorRGB(0xb7702d, "chinese-yellow-tea",                    register=True)
    CHLOROPHYLL_GREEN                    : ColorRGB = lambda *_: ColorRGB(0x4aff00, "chlorophyll-green",                     register=True)
    CHOCOLATE                            : ColorRGB = lambda *_: ColorRGB(0x3d1c02, "chocolate",                             register=True, variation_map={
             0x7b3f00:   "traditional" })
    CHOCOLATE_BROWN                      : ColorRGB = lambda *_: ColorRGB(0x411900, "chocolate-brown",                       register=True)
    CHOCOLATE_COSMOS                     : ColorRGB = lambda *_: ColorRGB(0x58111a, "chocolate-cosmos",                      register=True)
    CHOCOLATE_KISSES                     : ColorRGB = lambda *_: ColorRGB(0x3c1421, "chocolate-kisses",                      register=True)
    CHORE_COAT                           : ColorRGB = lambda *_: ColorRGB(0x0747a6, "chore-coat",                            register=True)
    CHROME_ALUMINUM                      : ColorRGB = lambda *_: ColorRGB(0xa8a9ad, "chrome-aluminum",                       register=True)
    CHROME_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xffa700, "chrome-yellow",                         register=True)
    CINEREOUS                            : ColorRGB = lambda *_: ColorRGB(0x98817b, "cinereous",                             register=True)
    CINNABAR                             : ColorRGB = lambda *_: ColorRGB(0xe34234, "cinnabar",                              register=True)
    CINNAMON                             : ColorRGB = lambda *_: ColorRGB(0xac4f06, "cinnamon",                              register=True)
    CINNAMON_SATIN                       : ColorRGB = lambda *_: ColorRGB(0xcd607e, "cinnamon-satin",                        register=True)
    CITRINE                              : ColorRGB = lambda *_: ColorRGB(0xe4d00a, "citrine",                               register=True)
    CITRINE_BROWN                        : ColorRGB = lambda *_: ColorRGB(0x933709, "citrine-brown",                         register=True)
    CITRON                               : ColorRGB = lambda *_: ColorRGB(0x9fa91f, "citron",                                register=True)
    CLARET                               : ColorRGB = lambda *_: ColorRGB(0x680018, "claret",                                register=True)
    CLASSIC_ROSE                         : ColorRGB = lambda *_: ColorRGB(0xfbcce7, "classic-rose",                          register=True)
    CLAY                                 : ColorRGB = lambda *_: ColorRGB(0xb66a50, "clay",                                  register=True)
    CLAY_BROWN                           : ColorRGB = lambda *_: ColorRGB(0xb2713d, "clay-brown",                            register=True)
    CLEAR_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x247afd, "clear-blue",                            register=True)
    CLOONEY                              : ColorRGB = lambda *_: ColorRGB(0x5e6c84, "clooney",                               register=True)
    CLOUDY_BLUE                          : ColorRGB = lambda *_: ColorRGB(0xacc2d9, "cloudy-blue",                           register=True)
    CLOVE_BROWN                          : ColorRGB = lambda *_: ColorRGB(0x8f583c, "clove-brown",                           register=True)
    CLOVE_DYED                           : ColorRGB = lambda *_: ColorRGB(0xa96232, "clove-dyed",                            register=True)
    CLOVERLEAF                           : ColorRGB = lambda *_: ColorRGB(0x79f2c0, "cloverleaf",                            register=True)
    COARSE_WOOL                          : ColorRGB = lambda *_: ColorRGB(0x181b26, "coarse-wool",                           register=True)
    COBALT                               : ColorRGB = lambda *_: ColorRGB(0x1e488f, "cobalt",                                register=True)
    COBALT_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x030aa7, "cobalt-blue",                           register=True)
    COCOA                                : ColorRGB = lambda *_: ColorRGB(0x875f42, "cocoa",                                 register=True)
    COCONUT                              : ColorRGB = lambda *_: ColorRGB(0x965a3e, "coconut",                               register=True)
    COCONUT_WHITE                        : ColorRGB = lambda *_: ColorRGB(0xe9edf6, "coconut-white",                         register=True)
    CODIUM_FRAGILE_SEAWEED               : ColorRGB = lambda *_: ColorRGB(0x524b2a, "codium-fragile-seaweed",                register=True)
    COFFEE                               : ColorRGB = lambda *_: ColorRGB(0xa6814c, "coffee",                                register=True)
    COLA                                 : ColorRGB = lambda *_: ColorRGB(0x3c3024, "cola",                                  register=True)
    COLUMBIA_BLUE                        : ColorRGB = lambda *_: ColorRGB(0xc4d8e2, "columbia-blue",                         register=True)
    CONCRETE_JUNGLE                      : ColorRGB = lambda *_: ColorRGB(0x505f79, "concrete-jungle",                       register=True)
    CONDITIONER                          : ColorRGB = lambda *_: ColorRGB(0xffffcc, "conditioner",                           register=True)
    CONGO_PINK                           : ColorRGB = lambda *_: ColorRGB(0xf88379, "congo-pink",                            register=True)
    CONTEMPLATION_IN_A_TEA_GARDEN        : ColorRGB = lambda *_: ColorRGB(0x665343, "contemplation-in-a-tea-garden",         register=True)
    COOGEE                               : ColorRGB = lambda *_: ColorRGB(0x2684ff, "coogee",                                register=True)
    COOKIES_AND_CREAM                    : ColorRGB = lambda *_: ColorRGB(0xeee0b1, "cookies-and-cream",                     register=True)
    COOL_BLACK                           : ColorRGB = lambda *_: ColorRGB(0x002e63, "cool-black",                            register=True)
    COOL_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x4984b8, "cool-blue",                             register=True)
    COOL_GRAY                            : ColorRGB = lambda *_: ColorRGB(0x8c92ac, "cool-gray",                             register=True)
    COOL_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x33b864, "cool-green",                            register=True)
    COOL_GREY                            : ColorRGB = lambda *_: ColorRGB(0x95a3a6, "cool-grey",                             register=True)
    COPPER                               : ColorRGB = lambda *_: ColorRGB(0xb66325, "copper",                                register=True, variation_map={
             0xda8a67:   "crayola" })
    COPPER_PENNY                         : ColorRGB = lambda *_: ColorRGB(0xad6f69, "copper-penny",                          register=True)
    COPPER_RED                           : ColorRGB = lambda *_: ColorRGB(0xcb6d51, "copper-red",                            register=True)
    COPPER_ROSE                          : ColorRGB = lambda *_: ColorRGB(0x996666, "copper-rose",                           register=True)
    COQUELICOT                           : ColorRGB = lambda *_: ColorRGB(0xff3800, "coquelicot",                            register=True)
    CORAL                                : ColorRGB = lambda *_: ColorRGB(0xfc5a50, "coral",                                 register=True, variation_map={
             0xf8674f:   "sango-iro" })
    CORAL_PINK                           : ColorRGB = lambda *_: ColorRGB(0xff6163, "coral-pink",                            register=True)
    CORAL_RED                            : ColorRGB = lambda *_: ColorRGB(0xff4040, "coral-red",                             register=True)
    CORAL_REEF                           : ColorRGB = lambda *_: ColorRGB(0xfd7c6e, "coral-reef",                            register=True, variation_map={
             0xf6a494:   "valspar-paint" })
    CORDOVAN                             : ColorRGB = lambda *_: ColorRGB(0x893f45, "cordovan",                              register=True)
    CORN                                 : ColorRGB = lambda *_: ColorRGB(0xfbec5d, "corn",                                  register=True, variation_map={
             0xfaa945:   "tomorokoshi-iro" })
    CORNFLOWER                           : ColorRGB = lambda *_: ColorRGB(0x6a79f7, "cornflower",                            register=True)
    CORNFLOWER_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x5170d7, "cornflower-blue",                       register=True)
    CORNSILK                             : ColorRGB = lambda *_: ColorRGB(0xfff8dc, "cornsilk",                              register=True)
    COSMIC_COBALT                        : ColorRGB = lambda *_: ColorRGB(0x2e2d88, "cosmic-cobalt",                         register=True)
    COSMIC_LATTE                         : ColorRGB = lambda *_: ColorRGB(0xfff8e7, "cosmic-latte",                          register=True)
    COTTON_CANDY                         : ColorRGB = lambda *_: ColorRGB(0xffbcd9, "cotton-candy",                          register=True)
    COWBELL                              : ColorRGB = lambda *_: ColorRGB(0xffe380, "cowbell",                               register=True)
    COYOTE_BROWN                         : ColorRGB = lambda *_: ColorRGB(0x81613c, "coyote-brown",                          register=True)
    CRANBERRY                            : ColorRGB = lambda *_: ColorRGB(0x9e003a, "cranberry",                             register=True)
    CREAM                                : ColorRGB = lambda *_: ColorRGB(0xffffc2, "cream",                                 register=True)
    CREME                                : ColorRGB = lambda *_: ColorRGB(0xffffb6, "creme",                                 register=True)
    CRIMSON                              : ColorRGB = lambda *_: ColorRGB(0x8c000f, "crimson",                               register=True)
    CRIMSON_GLORY                        : ColorRGB = lambda *_: ColorRGB(0xbe0032, "crimson-glory",                         register=True)
    CRIMSON_RED                          : ColorRGB = lambda *_: ColorRGB(0x990000, "crimson-red",                           register=True)
    CRYSTAL                              : ColorRGB = lambda *_: ColorRGB(0xa7d8de, "crystal",                               register=True)
    CRYSTAL_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x68a0b0, "crystal-blue",                          register=True)
    CULTURED                             : ColorRGB = lambda *_: ColorRGB(0xf5f5f5, "cultured",                              register=True)
    CUSTARD                              : ColorRGB = lambda *_: ColorRGB(0xfffd78, "custard",                               register=True)
    CYAN                                 : ColorRGB = lambda *_: ColorRGB(0x00ffff, "cyan",                                  register=True, variation_map={
             0x00b7eb:   "process" })
    CYAN_AZURE                           : ColorRGB = lambda *_: ColorRGB(0x4e82b4, "cyan-azure",                            register=True)
    CYAN_BLUE_AZURE                      : ColorRGB = lambda *_: ColorRGB(0x4682bf, "cyan-blue-azure",                       register=True)
    CYAN_COBALT_BLUE                     : ColorRGB = lambda *_: ColorRGB(0x28589c, "cyan-cobalt-blue",                      register=True)
    CYAN_CORNFLOWER_BLUE                 : ColorRGB = lambda *_: ColorRGB(0x188bc2, "cyan-cornflower-blue",                  register=True)
    CYAN_KALM                            : ColorRGB = lambda *_: ColorRGB(0x5e8d87, "cyan-kalm",                             register=True)
    CYAN_PANIK                           : ColorRGB = lambda *_: ColorRGB(0x5dd6ed, "cyan-panik",                            register=True)
    CYBER_GRAPE                          : ColorRGB = lambda *_: ColorRGB(0x58427c, "cyber-grape",                           register=True)
    CYBER_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xffd300, "cyber-yellow",                          register=True)
    CYCLAMEN                             : ColorRGB = lambda *_: ColorRGB(0xf56fa1, "cyclamen",                              register=True)
    CYPRESS_BARK                         : ColorRGB = lambda *_: ColorRGB(0x752e23, "cypress-bark",                          register=True)
    CYPRESS_BARK_RED                     : ColorRGB = lambda *_: ColorRGB(0x6f3028, "cypress-bark-red",                      register=True)
    DA_JUICE                             : ColorRGB = lambda *_: ColorRGB(0x6554c0, "da-juice",                              register=True, aliases=['jira-violet'])
    DAFFODIL                             : ColorRGB = lambda *_: ColorRGB(0xffff31, "daffodil",                              register=True)
    DANDELION                            : ColorRGB = lambda *_: ColorRGB(0xfedf08, "dandelion",                             register=True, variation_map={
             0xfddb6d:   "crayola" })
    DANDELION_WHISPER                    : ColorRGB = lambda *_: ColorRGB(0xfff0b3, "dandelion-whisper",                     register=True)
    DARK                                 : ColorRGB = lambda *_: ColorRGB(0x1b2431, "dark",                                  register=True)
    DARK_AQUA                            : ColorRGB = lambda *_: ColorRGB(0x05696b, "dark-aqua",                             register=True)
    DARK_AQUAMARINE                      : ColorRGB = lambda *_: ColorRGB(0x017371, "dark-aquamarine",                       register=True)
    DARK_BEIGE                           : ColorRGB = lambda *_: ColorRGB(0xac9362, "dark-beige",                            register=True)
    DARK_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x00035b, "dark-blue",                             register=True, variation_map={
             0x192236:   "kon" })
    DARK_BLUE_GRAY                       : ColorRGB = lambda *_: ColorRGB(0x666699, "dark-blue-gray",                        register=True)
    DARK_BLUE_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x005249, "dark-blue-green",                       register=True)
    DARK_BLUE_GREY                       : ColorRGB = lambda *_: ColorRGB(0x1f3b4d, "dark-blue-grey",                        register=True)
    DARK_BLUE_LAPIS_LAZULI               : ColorRGB = lambda *_: ColorRGB(0x1b294b, "dark-blue-lapis-lazuli",                register=True)
    DARK_BRONZE                          : ColorRGB = lambda *_: ColorRGB(0x804a00, "dark-bronze",                           register=True, variation_map={
             0x514100:   "coin" })
    DARK_BROWN                           : ColorRGB = lambda *_: ColorRGB(0x341c02, "dark-brown",                            register=True)
    DARK_BROWN_TANGELO                   : ColorRGB = lambda *_: ColorRGB(0x88654e, "dark-brown-tangelo",                    register=True)
    DARK_BYZANTIUM                       : ColorRGB = lambda *_: ColorRGB(0x5d3954, "dark-byzantium",                        register=True)
    DARK_CANDY_APPLE_RED                 : ColorRGB = lambda *_: ColorRGB(0xa40000, "dark-candy-apple-red",                  register=True)
    DARK_CERULEAN                        : ColorRGB = lambda *_: ColorRGB(0x08457e, "dark-cerulean",                         register=True)
    DARK_CHARCOAL                        : ColorRGB = lambda *_: ColorRGB(0x333333, "dark-charcoal",                         register=True)
    DARK_CHESTNUT                        : ColorRGB = lambda *_: ColorRGB(0x986960, "dark-chestnut",                         register=True)
    DARK_CHOCOLATE                       : ColorRGB = lambda *_: ColorRGB(0x490206, "dark-chocolate",                        register=True, variation_map={
             0x3c1321:   "hersheys" })
    DARK_CORAL                           : ColorRGB = lambda *_: ColorRGB(0xcf524e, "dark-coral",                            register=True)
    DARK_CORNFLOWER_BLUE                 : ColorRGB = lambda *_: ColorRGB(0x26428b, "dark-cornflower-blue",                  register=True)
    DARK_CREAM                           : ColorRGB = lambda *_: ColorRGB(0xfff39a, "dark-cream",                            register=True)
    DARK_CYAN                            : ColorRGB = lambda *_: ColorRGB(0x0a888a, "dark-cyan",                             register=True)
    DARK_ELECTRIC_BLUE                   : ColorRGB = lambda *_: ColorRGB(0x536878, "dark-electric-blue",                    register=True)
    DARK_FOREST_GREEN                    : ColorRGB = lambda *_: ColorRGB(0x002d04, "dark-forest-green",                     register=True)
    DARK_FUCHSIA                         : ColorRGB = lambda *_: ColorRGB(0x9d0759, "dark-fuchsia",                          register=True)
    DARK_GOLD                            : ColorRGB = lambda *_: ColorRGB(0xb59410, "dark-gold",                             register=True)
    DARK_GOLDENROD                       : ColorRGB = lambda *_: ColorRGB(0xb8860b, "dark-goldenrod",                        register=True)
    DARK_GRASS_GREEN                     : ColorRGB = lambda *_: ColorRGB(0x388004, "dark-grass-green",                      register=True)
    DARK_GRAY                            : ColorRGB = lambda *_: ColorRGB(0xa9a9a9, "dark-gray",                             register=True)
    DARK_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x033500, "dark-green",                            register=True, variation_map={
             0x006400:   "x11" })
    DARK_GREEN_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x1f6357, "dark-green-blue",                       register=True)
    DARK_GREY                            : ColorRGB = lambda *_: ColorRGB(0x363737, "dark-grey",                             register=True)
    DARK_GREY_BLUE                       : ColorRGB = lambda *_: ColorRGB(0x29465b, "dark-grey-blue",                        register=True)
    DARK_GUNMETAL                        : ColorRGB = lambda *_: ColorRGB(0x1f262a, "dark-gunmetal",                         register=True)
    DARK_HOT_PINK                        : ColorRGB = lambda *_: ColorRGB(0xd90166, "dark-hot-pink",                         register=True)
    DARK_IMPERIAL_BLUE                   : ColorRGB = lambda *_: ColorRGB(0x00147e, "dark-imperial-blue",                    register=True, variation_map={
             0x00416a:   "2" })
    DARK_INDIGO                          : ColorRGB = lambda *_: ColorRGB(0x1f0954, "dark-indigo",                           register=True, variation_map={
             0x614e6e:   "futaai" })
    DARK_JUNGLE_GREEN                    : ColorRGB = lambda *_: ColorRGB(0x1a2421, "dark-jungle-green",                     register=True)
    DARK_KHAKI                           : ColorRGB = lambda *_: ColorRGB(0x9b8f55, "dark-khaki",                            register=True)
    DARK_LAVA                            : ColorRGB = lambda *_: ColorRGB(0x483c32, "dark-lava",                             register=True)
    DARK_LAVENDER                        : ColorRGB = lambda *_: ColorRGB(0x856798, "dark-lavender",                         register=True)
    DARK_LEMON_LIME                      : ColorRGB = lambda *_: ColorRGB(0x8bbe1b, "dark-lemon-lime",                       register=True)
    DARK_LILAC                           : ColorRGB = lambda *_: ColorRGB(0x9c6da5, "dark-lilac",                            register=True)
    DARK_LIME                            : ColorRGB = lambda *_: ColorRGB(0x84b701, "dark-lime",                             register=True)
    DARK_LIME_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x7ebd01, "dark-lime-green",                       register=True)
    DARK_LIVER                           : ColorRGB = lambda *_: ColorRGB(0x534b4f, "dark-liver",                            register=True, variation_map={
             0x543d37:   "horses" })
    DARK_MAGENTA                         : ColorRGB = lambda *_: ColorRGB(0x960056, "dark-magenta",                          register=True)
    DARK_MAROON                          : ColorRGB = lambda *_: ColorRGB(0x3c0008, "dark-maroon",                           register=True)
    DARK_MAUVE                           : ColorRGB = lambda *_: ColorRGB(0x874c62, "dark-mauve",                            register=True)
    DARK_MIDNIGHT_BLUE                   : ColorRGB = lambda *_: ColorRGB(0x003366, "dark-midnight-blue",                    register=True)
    DARK_MINT                            : ColorRGB = lambda *_: ColorRGB(0x48c072, "dark-mint",                             register=True)
    DARK_MINT_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x20c073, "dark-mint-green",                       register=True)
    DARK_MOSS_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x4a5d23, "dark-moss-green",                       register=True)
    DARK_MUSTARD                         : ColorRGB = lambda *_: ColorRGB(0xa88905, "dark-mustard",                          register=True)
    DARK_NAVY                            : ColorRGB = lambda *_: ColorRGB(0x000435, "dark-navy",                             register=True)
    DARK_NAVY_BLUE                       : ColorRGB = lambda *_: ColorRGB(0x00022e, "dark-navy-blue",                        register=True)
    DARK_OLIVE                           : ColorRGB = lambda *_: ColorRGB(0x373e02, "dark-olive",                            register=True)
    DARK_OLIVE_GREEN                     : ColorRGB = lambda *_: ColorRGB(0x3c4d03, "dark-olive-green",                      register=True)
    DARK_ORANGE                          : ColorRGB = lambda *_: ColorRGB(0xc65102, "dark-orange",                           register=True)
    DARK_ORCHID                          : ColorRGB = lambda *_: ColorRGB(0x9932cc, "dark-orchid",                           register=True)
    DARK_PASTEL_BLUE                     : ColorRGB = lambda *_: ColorRGB(0x779ecb, "dark-pastel-blue",                      register=True)
    DARK_PASTEL_GREEN                    : ColorRGB = lambda *_: ColorRGB(0x56ae57, "dark-pastel-green",                     register=True)
    DARK_PASTEL_PURPLE                   : ColorRGB = lambda *_: ColorRGB(0x966fd6, "dark-pastel-purple",                    register=True)
    DARK_PASTEL_RED                      : ColorRGB = lambda *_: ColorRGB(0xc23b22, "dark-pastel-red",                       register=True)
    DARK_PEACH                           : ColorRGB = lambda *_: ColorRGB(0xde7e5d, "dark-peach",                            register=True)
    DARK_PERIWINKLE                      : ColorRGB = lambda *_: ColorRGB(0x665fd1, "dark-periwinkle",                       register=True)
    DARK_PINK                            : ColorRGB = lambda *_: ColorRGB(0xcb416b, "dark-pink",                             register=True)
    DARK_PLUM                            : ColorRGB = lambda *_: ColorRGB(0x3f012c, "dark-plum",                             register=True)
    DARK_POWDER_BLUE                     : ColorRGB = lambda *_: ColorRGB(0x003399, "dark-powder-blue",                      register=True)
    DARK_PUCE                            : ColorRGB = lambda *_: ColorRGB(0x4f3a3c, "dark-puce",                             register=True)
    DARK_PURPLE                          : ColorRGB = lambda *_: ColorRGB(0x35063e, "dark-purple",                           register=True)
    DARK_RASPBERRY                       : ColorRGB = lambda *_: ColorRGB(0x872657, "dark-raspberry",                        register=True)
    DARK_RED                             : ColorRGB = lambda *_: ColorRGB(0x840000, "dark-red",                              register=True, variation_map={
             0x23191e:   "kurobeni" })
    DARK_ROSE                            : ColorRGB = lambda *_: ColorRGB(0xb5485d, "dark-rose",                             register=True)
    DARK_ROYAL_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x02066f, "dark-royal-blue",                       register=True)
    DARK_SAGE                            : ColorRGB = lambda *_: ColorRGB(0x598556, "dark-sage",                             register=True)
    DARK_SALMON                          : ColorRGB = lambda *_: ColorRGB(0xc85a53, "dark-salmon",                           register=True)
    DARK_SAND                            : ColorRGB = lambda *_: ColorRGB(0xa88f59, "dark-sand",                             register=True)
    DARK_SCARLET                         : ColorRGB = lambda *_: ColorRGB(0x560319, "dark-scarlet",                          register=True)
    DARK_SEA_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x11875d, "dark-sea-green",                        register=True)
    DARK_SEAFOAM                         : ColorRGB = lambda *_: ColorRGB(0x1fb57a, "dark-seafoam",                          register=True)
    DARK_SEAFOAM_GREEN                   : ColorRGB = lambda *_: ColorRGB(0x3eaf76, "dark-seafoam-green",                    register=True)
    DARK_SIENNA                          : ColorRGB = lambda *_: ColorRGB(0x3c1414, "dark-sienna",                           register=True)
    DARK_SILVER                          : ColorRGB = lambda *_: ColorRGB(0x71706e, "dark-silver",                           register=True)
    DARK_SKY_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x448ee4, "dark-sky-blue",                         register=True)
    DARK_SLATE_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x214761, "dark-slate-blue",                       register=True)
    DARK_SLATE_GRAY                      : ColorRGB = lambda *_: ColorRGB(0x2f4f4f, "dark-slate-gray",                       register=True)
    DARK_SPRING_GREEN                    : ColorRGB = lambda *_: ColorRGB(0x177245, "dark-spring-green",                     register=True)
    DARK_TAN                             : ColorRGB = lambda *_: ColorRGB(0xaf884a, "dark-tan",                              register=True)
    DARK_TANGERINE                       : ColorRGB = lambda *_: ColorRGB(0xffa812, "dark-tangerine",                        register=True)
    DARK_TAUPE                           : ColorRGB = lambda *_: ColorRGB(0x7f684e, "dark-taupe",                            register=True)
    DARK_TEAL                            : ColorRGB = lambda *_: ColorRGB(0x014d4e, "dark-teal",                             register=True)
    DARK_TERRA_COTTA                     : ColorRGB = lambda *_: ColorRGB(0xcc4e5c, "dark-terra-cotta",                      register=True)
    DARK_TURQUOISE                       : ColorRGB = lambda *_: ColorRGB(0x045c5a, "dark-turquoise",                        register=True)
    DARK_VANILLA                         : ColorRGB = lambda *_: ColorRGB(0xd1bea8, "dark-vanilla",                          register=True)
    DARK_VIOLET                          : ColorRGB = lambda *_: ColorRGB(0x34013f, "dark-violet",                           register=True)
    DARK_YELLOW                          : ColorRGB = lambda *_: ColorRGB(0xd5b60a, "dark-yellow",                           register=True)
    DARK_YELLOW_GREEN                    : ColorRGB = lambda *_: ColorRGB(0x728f02, "dark-yellow-green",                     register=True)
    DARKBLUE                             : ColorRGB = lambda *_: ColorRGB(0x030764, "darkblue",                              register=True)
    DARKGREEN                            : ColorRGB = lambda *_: ColorRGB(0x054907, "darkgreen",                             register=True)
    DARKISH_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x014182, "darkish-blue",                          register=True)
    DARKISH_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x287c37, "darkish-green",                         register=True)
    DARKISH_PINK                         : ColorRGB = lambda *_: ColorRGB(0xda467d, "darkish-pink",                          register=True)
    DARKISH_PURPLE                       : ColorRGB = lambda *_: ColorRGB(0x751973, "darkish-purple",                        register=True)
    DARKISH_RED                          : ColorRGB = lambda *_: ColorRGB(0xa90308, "darkish-red",                           register=True)
    DARTMOUTH_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x00703c, "dartmouth-green",                       register=True)
    DAVYS_GRAY                           : ColorRGB = lambda *_: ColorRGB(0x555555, "davys-gray",                            register=True)
    DAWN                                 : ColorRGB = lambda *_: ColorRGB(0xfa7b62, "dawn",                                  register=True)
    DAWN_GRAY                            : ColorRGB = lambda *_: ColorRGB(0x616359, "dawn-gray",                             register=True)
    DAYLILY                              : ColorRGB = lambda *_: ColorRGB(0xff8936, "daylily",                               register=True)
    DEBIAN_RED                           : ColorRGB = lambda *_: ColorRGB(0xd70a53, "debian-red",                            register=True)
    DEBRITO                              : ColorRGB = lambda *_: ColorRGB(0xff8b00, "debrito",                               register=True)
    DECAYING_LEAVES                      : ColorRGB = lambda *_: ColorRGB(0xd57835, "decaying-leaves",                       register=True)
    DEEP_AMETHYST                        : ColorRGB = lambda *_: ColorRGB(0x9c8aa4, "deep-amethyst",                         register=True)
    DEEP_AQUA                            : ColorRGB = lambda *_: ColorRGB(0x08787f, "deep-aqua",                             register=True)
    DEEP_AQUAMARINE                      : ColorRGB = lambda *_: ColorRGB(0x40826d, "deep-aquamarine",                       register=True)
    DEEP_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x040273, "deep-blue",                             register=True)
    DEEP_BROWN                           : ColorRGB = lambda *_: ColorRGB(0x410200, "deep-brown",                            register=True)
    DEEP_CARMINE                         : ColorRGB = lambda *_: ColorRGB(0xa9203e, "deep-carmine",                          register=True)
    DEEP_CARMINE_PINK                    : ColorRGB = lambda *_: ColorRGB(0xef3038, "deep-carmine-pink",                     register=True)
    DEEP_CARROT_ORANGE                   : ColorRGB = lambda *_: ColorRGB(0xe9692c, "deep-carrot-orange",                    register=True)
    DEEP_CERISE                          : ColorRGB = lambda *_: ColorRGB(0xda3287, "deep-cerise",                           register=True)
    DEEP_CHAMPAGNE                       : ColorRGB = lambda *_: ColorRGB(0xfad6a5, "deep-champagne",                        register=True)
    DEEP_CHESTNUT                        : ColorRGB = lambda *_: ColorRGB(0xb94e48, "deep-chestnut",                         register=True)
    DEEP_COFFEE                          : ColorRGB = lambda *_: ColorRGB(0x704241, "deep-coffee",                           register=True)
    DEEP_DUMPLING                        : ColorRGB = lambda *_: ColorRGB(0x9b351b, "deep-dumpling",                         register=True)
    DEEP_FUCHSIA                         : ColorRGB = lambda *_: ColorRGB(0xc154c1, "deep-fuchsia",                          register=True)
    DEEP_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x02590f, "deep-green",                            register=True)
    DEEP_GREEN_CYAN_TURQUOISE            : ColorRGB = lambda *_: ColorRGB(0x0e7c61, "deep-green-cyan-turquoise",             register=True)
    DEEP_JUNGLE_GREEN                    : ColorRGB = lambda *_: ColorRGB(0x004b49, "deep-jungle-green",                     register=True)
    DEEP_KOAMARU                         : ColorRGB = lambda *_: ColorRGB(0x333366, "deep-koamaru",                          register=True)
    DEEP_LAVENDER                        : ColorRGB = lambda *_: ColorRGB(0x8d5eb7, "deep-lavender",                         register=True)
    DEEP_LEMON                           : ColorRGB = lambda *_: ColorRGB(0xf5c71a, "deep-lemon",                            register=True)
    DEEP_LILAC                           : ColorRGB = lambda *_: ColorRGB(0x966ebd, "deep-lilac",                            register=True)
    DEEP_MAGENTA                         : ColorRGB = lambda *_: ColorRGB(0xa0025c, "deep-magenta",                          register=True)
    DEEP_MAROON                          : ColorRGB = lambda *_: ColorRGB(0x820000, "deep-maroon",                           register=True)
    DEEP_MAUVE                           : ColorRGB = lambda *_: ColorRGB(0xd473d4, "deep-mauve",                            register=True)
    DEEP_MOSS_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x355e3b, "deep-moss-green",                       register=True)
    DEEP_ORANGE                          : ColorRGB = lambda *_: ColorRGB(0xdc4d01, "deep-orange",                           register=True)
    DEEP_PEACH                           : ColorRGB = lambda *_: ColorRGB(0xffcba4, "deep-peach",                            register=True)
    DEEP_PINK                            : ColorRGB = lambda *_: ColorRGB(0xcb0162, "deep-pink",                             register=True)
    DEEP_PUCE                            : ColorRGB = lambda *_: ColorRGB(0xa95c68, "deep-puce",                             register=True)
    DEEP_PURPLE                          : ColorRGB = lambda *_: ColorRGB(0x36013f, "deep-purple",                           register=True, variation_map={
             0x3a243b:   "kokimurasaki" })
    DEEP_RED                             : ColorRGB = lambda *_: ColorRGB(0x9a0200, "deep-red",                              register=True)
    DEEP_ROSE                            : ColorRGB = lambda *_: ColorRGB(0xc74767, "deep-rose",                             register=True)
    DEEP_RUBY                            : ColorRGB = lambda *_: ColorRGB(0x843f5b, "deep-ruby",                             register=True)
    DEEP_SAFFRON                         : ColorRGB = lambda *_: ColorRGB(0xff9933, "deep-saffron",                          register=True)
    DEEP_SCARLET                         : ColorRGB = lambda *_: ColorRGB(0x7b3b3a, "deep-scarlet",                          register=True)
    DEEP_SEA_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x015482, "deep-sea-blue",                         register=True)
    DEEP_SKY_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x0d75f8, "deep-sky-blue",                         register=True)
    DEEP_SPACE_SPARKLE                   : ColorRGB = lambda *_: ColorRGB(0x4a646c, "deep-space-sparkle",                    register=True)
    DEEP_TAUPE                           : ColorRGB = lambda *_: ColorRGB(0x7e5e60, "deep-taupe",                            register=True)
    DEEP_TEAL                            : ColorRGB = lambda *_: ColorRGB(0x00555a, "deep-teal",                             register=True)
    DEEP_TURQUOISE                       : ColorRGB = lambda *_: ColorRGB(0x017374, "deep-turquoise",                        register=True)
    DEEP_TUSCAN_RED                      : ColorRGB = lambda *_: ColorRGB(0x66424d, "deep-tuscan-red",                       register=True)
    DEEP_VIOLET                          : ColorRGB = lambda *_: ColorRGB(0x490648, "deep-violet",                           register=True)
    DEER                                 : ColorRGB = lambda *_: ColorRGB(0xba8759, "deer",                                  register=True)
    DELICATE_GIRL_BLUE                   : ColorRGB = lambda *_: ColorRGB(0x6ab2ca, "delicate-girl-blue",                    register=True)
    DENIM                                : ColorRGB = lambda *_: ColorRGB(0x3b638c, "denim",                                 register=True)
    DENIM_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x3b5b92, "denim-blue",                            register=True)
    DESATURATED_CYAN                     : ColorRGB = lambda *_: ColorRGB(0x669999, "desaturated-cyan",                      register=True)
    DESERT                               : ColorRGB = lambda *_: ColorRGB(0xccad60, "desert",                                register=True)
    DESERT_SAND                          : ColorRGB = lambda *_: ColorRGB(0xedc9af, "desert-sand",                           register=True)
    DESIRE                               : ColorRGB = lambda *_: ColorRGB(0xea3c53, "desire",                                register=True)
    DIAMOND                              : ColorRGB = lambda *_: ColorRGB(0xb9f2ff, "diamond",                               register=True)
    DIARRHEA                             : ColorRGB = lambda *_: ColorRGB(0x9f8303, "diarrhea",                              register=True)
    DIM_GRAY                             : ColorRGB = lambda *_: ColorRGB(0x696969, "dim-gray",                              register=True)
    DINGY_DUNGEON                        : ColorRGB = lambda *_: ColorRGB(0xc53151, "dingy-dungeon",                         register=True)
    DIRT                                 : ColorRGB = lambda *_: ColorRGB(0x8a6e45, "dirt",                                  register=True)
    DIRT_BROWN                           : ColorRGB = lambda *_: ColorRGB(0x836539, "dirt-brown",                            register=True)
    DIRTY_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x3f829d, "dirty-blue",                            register=True)
    DIRTY_BROWN                          : ColorRGB = lambda *_: ColorRGB(0xb5651e, "dirty-brown",                           register=True)
    DIRTY_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x667e2c, "dirty-green",                           register=True)
    DIRTY_ORANGE                         : ColorRGB = lambda *_: ColorRGB(0xc87606, "dirty-orange",                          register=True)
    DIRTY_PINK                           : ColorRGB = lambda *_: ColorRGB(0xca7b80, "dirty-pink",                            register=True)
    DIRTY_PURPLE                         : ColorRGB = lambda *_: ColorRGB(0x734a65, "dirty-purple",                          register=True)
    DIRTY_WHITE                          : ColorRGB = lambda *_: ColorRGB(0xe8e4c9, "dirty-white",                           register=True)
    DIRTY_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xcdc50a, "dirty-yellow",                          register=True)
    DISAPPEARING_PURPLE                  : ColorRGB = lambda *_: ColorRGB(0x3f313a, "disappearing-purple",                   register=True)
    DODGER_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x3e82fc, "dodger-blue",                           register=True)
    DODIE_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xfef65b, "dodie-yellow",                          register=True)
    DOGWOOD_ROSE                         : ColorRGB = lambda *_: ColorRGB(0xd71868, "dogwood-rose",                          register=True)
    DOLLAR_BILL                          : ColorRGB = lambda *_: ColorRGB(0x85bb65, "dollar-bill",                           register=True)
    DOLPHIN_GRAY                         : ColorRGB = lambda *_: ColorRGB(0x828e84, "dolphin-gray",                          register=True)
    DOUBAN_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x2496cd, "douban-blue",                           register=True)
    DOUBAN_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x2e963d, "douban-green",                          register=True)
    DOUBAN_LIGHT_YELLOW                  : ColorRGB = lambda *_: ColorRGB(0xf7c58e, "douban-light-yellow",                   register=True)
    DOVE_FEATHER_GRAY                    : ColorRGB = lambda *_: ColorRGB(0x755d5b, "dove-feather-gray",                     register=True)
    DRAB                                 : ColorRGB = lambda *_: ColorRGB(0x828344, "drab",                                  register=True)
    DRAB_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x749551, "drab-green",                            register=True)
    DRAGONS_BLOOD                        : ColorRGB = lambda *_: ColorRGB(0xbf2600, "dragons-blood",                         register=True)
    DRIED_BLOOD                          : ColorRGB = lambda *_: ColorRGB(0x4b0101, "dried-blood",                           register=True)
    DUCK_EGG_BLUE                        : ColorRGB = lambda *_: ColorRGB(0xc3fbf4, "duck-egg-blue",                         register=True)
    DUKE_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x00009c, "duke-blue",                             register=True)
    DULL_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x49759c, "dull-blue",                             register=True, variation_map={
             0x4f4944:   "aonibi" })
    DULL_BROWN                           : ColorRGB = lambda *_: ColorRGB(0x876e4b, "dull-brown",                            register=True)
    DULL_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x74a662, "dull-green",                            register=True)
    DULL_ORANGE                          : ColorRGB = lambda *_: ColorRGB(0xd8863b, "dull-orange",                           register=True)
    DULL_PINK                            : ColorRGB = lambda *_: ColorRGB(0xd5869d, "dull-pink",                             register=True)
    DULL_PURPLE                          : ColorRGB = lambda *_: ColorRGB(0x84597e, "dull-purple",                           register=True)
    DULL_RED                             : ColorRGB = lambda *_: ColorRGB(0xbb3f3f, "dull-red",                              register=True)
    DULL_TEAL                            : ColorRGB = lambda *_: ColorRGB(0x5f9e8f, "dull-teal",                             register=True)
    DULL_YELLOW                          : ColorRGB = lambda *_: ColorRGB(0xeedc5b, "dull-yellow",                           register=True)
    DURIAN                               : ColorRGB = lambda *_: ColorRGB(0xb07939, "durian",                                register=True)
    DURIAN_WHITE                         : ColorRGB = lambda *_: ColorRGB(0xe6d0ab, "durian-white",                          register=True)
    DURIAN_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xe1bd27, "durian-yellow",                         register=True)
    DUSK                                 : ColorRGB = lambda *_: ColorRGB(0x4e5481, "dusk",                                  register=True)
    DUSK_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x26538d, "dusk-blue",                             register=True)
    DUSKY_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x475f94, "dusky-blue",                            register=True)
    DUSKY_PINK                           : ColorRGB = lambda *_: ColorRGB(0xcc7a8b, "dusky-pink",                            register=True)
    DUSKY_PURPLE                         : ColorRGB = lambda *_: ColorRGB(0x895b7b, "dusky-purple",                          register=True)
    DUSKY_ROSE                           : ColorRGB = lambda *_: ColorRGB(0xba6873, "dusky-rose",                            register=True)
    DUST                                 : ColorRGB = lambda *_: ColorRGB(0xb2996e, "dust",                                  register=True)
    DUST_STORM                           : ColorRGB = lambda *_: ColorRGB(0xe5ccc9, "dust-storm",                            register=True)
    DUSTY_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x5a86ad, "dusty-blue",                            register=True)
    DUSTY_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x76a973, "dusty-green",                           register=True)
    DUSTY_LAVENDER                       : ColorRGB = lambda *_: ColorRGB(0xac86a8, "dusty-lavender",                        register=True)
    DUSTY_ORANGE                         : ColorRGB = lambda *_: ColorRGB(0xf0833a, "dusty-orange",                          register=True)
    DUSTY_PINK                           : ColorRGB = lambda *_: ColorRGB(0xd58a94, "dusty-pink",                            register=True)
    DUSTY_PURPLE                         : ColorRGB = lambda *_: ColorRGB(0x825f87, "dusty-purple",                          register=True)
    DUSTY_RED                            : ColorRGB = lambda *_: ColorRGB(0xb9484e, "dusty-red",                             register=True)
    DUSTY_ROSE                           : ColorRGB = lambda *_: ColorRGB(0xc0737a, "dusty-rose",                            register=True)
    DUSTY_TEAL                           : ColorRGB = lambda *_: ColorRGB(0x4c9085, "dusty-teal",                            register=True)
    DUTCH_WHITE                          : ColorRGB = lambda *_: ColorRGB(0xefdfbb, "dutch-white",                           register=True)
    DYESTALK_RED                         : ColorRGB = lambda *_: ColorRGB(0x913225, "dyestalk-red",                          register=True)
    EARTH                                : ColorRGB = lambda *_: ColorRGB(0xa2653e, "earth",                                 register=True)
    EARTH_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xe1a95f, "earth-yellow",                          register=True)
    EARTHTONE                            : ColorRGB = lambda *_: ColorRGB(0x5d3a1a, "earthtone",                             register=True)
    EASTER_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x8cfd7e, "easter-green",                          register=True)
    EASTER_PURPLE                        : ColorRGB = lambda *_: ColorRGB(0xc071fe, "easter-purple",                         register=True)
    EBONY                                : ColorRGB = lambda *_: ColorRGB(0x555d50, "ebony",                                 register=True)
    ECRU                                 : ColorRGB = lambda *_: ColorRGB(0xfeffca, "ecru",                                  register=True)
    EERIE_BLACK                          : ColorRGB = lambda *_: ColorRGB(0x1b1b1b, "eerie-black",                           register=True)
    EGG                                  : ColorRGB = lambda *_: ColorRGB(0xffa631, "egg",                                   register=True)
    EGG_SHELL                            : ColorRGB = lambda *_: ColorRGB(0xfffcc4, "egg-shell",                             register=True)
    EGGPLANT                             : ColorRGB = lambda *_: ColorRGB(0x380835, "eggplant",                              register=True)
    EGGPLANT_PURPLE                      : ColorRGB = lambda *_: ColorRGB(0x430541, "eggplant-purple",                       register=True)
    EGGSHELL                             : ColorRGB = lambda *_: ColorRGB(0xffffd4, "eggshell",                              register=True)
    EGGSHELL_BLUE                        : ColorRGB = lambda *_: ColorRGB(0xc4fff7, "eggshell-blue",                         register=True)
    EGGSHELL_PAPER                       : ColorRGB = lambda *_: ColorRGB(0xe2be9f, "eggshell-paper",                        register=True)
    EGYPTIAN_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x1034a6, "egyptian-blue",                         register=True)
    ELDERBERRY                           : ColorRGB = lambda *_: ColorRGB(0x17182b, "elderberry",                            register=True)
    ELECTRIC_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x0652ff, "electric-blue",                         register=True)
    ELECTRIC_BROWN                       : ColorRGB = lambda *_: ColorRGB(0xb56257, "electric-brown",                        register=True)
    ELECTRIC_CRIMSON                     : ColorRGB = lambda *_: ColorRGB(0xff003f, "electric-crimson",                      register=True)
    ELECTRIC_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x21fc0d, "electric-green",                        register=True)
    ELECTRIC_INDIGO                      : ColorRGB = lambda *_: ColorRGB(0x6f00ff, "electric-indigo",                       register=True)
    ELECTRIC_LIME                        : ColorRGB = lambda *_: ColorRGB(0xa8ff04, "electric-lime",                         register=True)
    ELECTRIC_ORANGE                      : ColorRGB = lambda *_: ColorRGB(0xff3503, "electric-orange",                       register=True)
    ELECTRIC_PINK                        : ColorRGB = lambda *_: ColorRGB(0xff0490, "electric-pink",                         register=True)
    ELECTRIC_PURPLE                      : ColorRGB = lambda *_: ColorRGB(0xaa23ff, "electric-purple",                       register=True)
    ELECTRIC_RED                         : ColorRGB = lambda *_: ColorRGB(0xe60000, "electric-red",                          register=True)
    ELECTRIC_ULTRAMARINE                 : ColorRGB = lambda *_: ColorRGB(0x3f00ff, "electric-ultramarine",                  register=True)
    ELECTRIC_VIOLET                      : ColorRGB = lambda *_: ColorRGB(0x8f00ff, "electric-violet",                       register=True)
    ELECTRIC_YELLOW                      : ColorRGB = lambda *_: ColorRGB(0xffff33, "electric-yellow",                       register=True)
    EMERALD                              : ColorRGB = lambda *_: ColorRGB(0x01a049, "emerald",                               register=True)
    EMERALD_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x028f1e, "emerald-green",                         register=True)
    EMINENCE                             : ColorRGB = lambda *_: ColorRGB(0x6c3082, "eminence",                              register=True)
    ENGLISH_LAVENDER                     : ColorRGB = lambda *_: ColorRGB(0xb48395, "english-lavender",                      register=True)
    ENGLISH_RED                          : ColorRGB = lambda *_: ColorRGB(0xab4b52, "english-red",                           register=True)
    ENGLISH_VERMILLION                   : ColorRGB = lambda *_: ColorRGB(0xcc474b, "english-vermillion",                    register=True)
    ENGLISH_VIOLET                       : ColorRGB = lambda *_: ColorRGB(0x563c5c, "english-violet",                        register=True)
    ETON_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x96c8a2, "eton-blue",                             register=True)
    EUCALYPTUS                           : ColorRGB = lambda *_: ColorRGB(0x44d7a8, "eucalyptus",                            register=True)
    EVERGREEN                            : ColorRGB = lambda *_: ColorRGB(0x05472a, "evergreen",                             register=True)
    FACEBOOK_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x39569c, "facebook-blue",                         register=True)
    FADED_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x658cbb, "faded-blue",                            register=True)
    FADED_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x7bb274, "faded-green",                           register=True)
    FADED_ORANGE                         : ColorRGB = lambda *_: ColorRGB(0xf0944d, "faded-orange",                          register=True)
    FADED_PINK                           : ColorRGB = lambda *_: ColorRGB(0xde9dac, "faded-pink",                            register=True)
    FADED_PURPLE                         : ColorRGB = lambda *_: ColorRGB(0x916e99, "faded-purple",                          register=True)
    FADED_RED                            : ColorRGB = lambda *_: ColorRGB(0xd3494e, "faded-red",                             register=True)
    FADED_SEN_NO_RIKYUS_TEA              : ColorRGB = lambda *_: ColorRGB(0xb0927a, "faded-sen-no-rikyus-tea",               register=True)
    FADED_SPICY_RED_BROWN                : ColorRGB = lambda *_: ColorRGB(0x9b533f, "faded-spicy-red-brown",                 register=True)
    FADED_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xfeff7f, "faded-yellow",                          register=True)
    FAKE_PURPLE                          : ColorRGB = lambda *_: ColorRGB(0x43242a, "fake-purple",                           register=True)
    FALU_RED                             : ColorRGB = lambda *_: ColorRGB(0x801818, "falu-red",                              register=True)
    FANDANGO                             : ColorRGB = lambda *_: ColorRGB(0xb53389, "fandango",                              register=True)
    FANDANGO_PINK                        : ColorRGB = lambda *_: ColorRGB(0xde5285, "fandango-pink",                         register=True)
    FASHION_FUCHSIA                      : ColorRGB = lambda *_: ColorRGB(0xf400a1, "fashion-fuchsia",                       register=True)
    FAWN                                 : ColorRGB = lambda *_: ColorRGB(0xcfaf7b, "fawn",                                  register=True)
    FELDGRAU                             : ColorRGB = lambda *_: ColorRGB(0x4d5d53, "feldgrau",                              register=True)
    FELDSPAR                             : ColorRGB = lambda *_: ColorRGB(0xfdd5b1, "feldspar",                              register=True)
    FERN                                 : ColorRGB = lambda *_: ColorRGB(0x63a950, "fern",                                  register=True)
    FERN_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x548d44, "fern-green",                            register=True)
    FERRARI_RED                          : ColorRGB = lambda *_: ColorRGB(0xff2800, "ferrari-red",                           register=True)
    FIELD_DRAB                           : ColorRGB = lambda *_: ColorRGB(0x6c541e, "field-drab",                            register=True)
    FIERY_ROSE                           : ColorRGB = lambda *_: ColorRGB(0xff5470, "fiery-rose",                            register=True)
    FINCH_BROWN                          : ColorRGB = lambda *_: ColorRGB(0x957b38, "finch-brown",                           register=True)
    FINE_PINE                            : ColorRGB = lambda *_: ColorRGB(0x36b37e, "fine-pine",                             register=True, aliases=['jira-green'])
    FIRE_ENGINE_RED                      : ColorRGB = lambda *_: ColorRGB(0xfe0002, "fire-engine-red",                       register=True)
    FIRE_OPAL                            : ColorRGB = lambda *_: ColorRGB(0xe95c4b, "fire-opal",                             register=True)
    FIREBRICK                            : ColorRGB = lambda *_: ColorRGB(0xb22222, "firebrick",                             register=True)
    FLAME                                : ColorRGB = lambda *_: ColorRGB(0xe25822, "flame",                                 register=True)
    FLAMINGO_PINK                        : ColorRGB = lambda *_: ColorRGB(0xfc8eac, "flamingo-pink",                         register=True)
    FLAT_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x3c73a8, "flat-blue",                             register=True)
    FLAT_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x699d4c, "flat-green",                            register=True)
    FLAVESCENT                           : ColorRGB = lambda *_: ColorRGB(0xf7e98e, "flavescent",                            register=True)
    FLAX                                 : ColorRGB = lambda *_: ColorRGB(0xeedc82, "flax",                                  register=True)
    FLESH                                : ColorRGB = lambda *_: ColorRGB(0xffe9d1, "flesh",                                 register=True)
    FLICKR_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x216bd6, "flickr-blue",                           register=True)
    FLICKR_PINK                          : ColorRGB = lambda *_: ColorRGB(0xfb0081, "flickr-pink",                           register=True)
    FLIRT                                : ColorRGB = lambda *_: ColorRGB(0xa2006d, "flirt",                                 register=True)
    FLIRTATIOUS_INDIGO_TEA               : ColorRGB = lambda *_: ColorRGB(0x473f2d, "flirtatious-indigo-tea",                register=True)
    FLORAL_LEAF                          : ColorRGB = lambda *_: ColorRGB(0xffb94e, "floral-leaf",                           register=True)
    FLORAL_WHITE                         : ColorRGB = lambda *_: ColorRGB(0xfffaf0, "floral-white",                          register=True)
    FLOWER_GIRL                          : ColorRGB = lambda *_: ColorRGB(0xf498ad, "flower-girl",                           register=True)
    FLUORESCENT_BLUE                     : ColorRGB = lambda *_: ColorRGB(0x15f4ee, "fluorescent-blue",                      register=True)
    FLUORESCENT_GREEN                    : ColorRGB = lambda *_: ColorRGB(0x08ff08, "fluorescent-green",                     register=True)
    FLURO_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x0aff02, "fluro-green",                           register=True)
    FOAM_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x90fda9, "foam-green",                            register=True)
    FOLLY                                : ColorRGB = lambda *_: ColorRGB(0xff004f, "folly",                                 register=True)
    FOREIGN_CRIMSON                      : ColorRGB = lambda *_: ColorRGB(0xc91f37, "foreign-crimson",                       register=True)
    FOREST                               : ColorRGB = lambda *_: ColorRGB(0x0b5509, "forest",                                register=True)
    FOREST_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x06470c, "forest-green",                          register=True, variation_map={
             0x5fa777:   "crayola", 
             0x014421:   "traditional" })
    FORREST_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x154406, "forrest-green",                         register=True)
    FOX                                  : ColorRGB = lambda *_: ColorRGB(0x985629, "fox",                                   register=True)
    FRENCH_BISTRE                        : ColorRGB = lambda *_: ColorRGB(0x856d4d, "french-bistre",                         register=True)
    FRENCH_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x436bad, "french-blue",                           register=True)
    FRENCH_FUCHSIA                       : ColorRGB = lambda *_: ColorRGB(0xfd3f92, "french-fuchsia",                        register=True)
    FRENCH_LILAC                         : ColorRGB = lambda *_: ColorRGB(0x86608e, "french-lilac",                          register=True)
    FRENCH_LIME                          : ColorRGB = lambda *_: ColorRGB(0x9efd38, "french-lime",                           register=True)
    FRENCH_PINK                          : ColorRGB = lambda *_: ColorRGB(0xfd6c9e, "french-pink",                           register=True)
    FRENCH_PLUM                          : ColorRGB = lambda *_: ColorRGB(0x811453, "french-plum",                           register=True)
    FRENCH_PUCE                          : ColorRGB = lambda *_: ColorRGB(0x4e1609, "french-puce",                           register=True)
    FRENCH_RASPBERRY                     : ColorRGB = lambda *_: ColorRGB(0xc72c48, "french-raspberry",                      register=True)
    FRENCH_ROSE                          : ColorRGB = lambda *_: ColorRGB(0xf64a8a, "french-rose",                           register=True)
    FRENCH_SKY_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x77b5fe, "french-sky-blue",                       register=True)
    FRENCH_VIOLET                        : ColorRGB = lambda *_: ColorRGB(0x8806ce, "french-violet",                         register=True)
    FRENCH_WINE                          : ColorRGB = lambda *_: ColorRGB(0xac1e44, "french-wine",                           register=True)
    FRESH_AIR                            : ColorRGB = lambda *_: ColorRGB(0xa6e7ff, "fresh-air",                             register=True)
    FRESH_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x69d84f, "fresh-green",                           register=True)
    FRESH_ONION                          : ColorRGB = lambda *_: ColorRGB(0x5b8930, "fresh-onion",                           register=True)
    FROG_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x58bc08, "frog-green",                            register=True)
    FROSTBITE                            : ColorRGB = lambda *_: ColorRGB(0xe936a7, "frostbite",                             register=True)
    FUCHSIA                              : ColorRGB = lambda *_: ColorRGB(0xed0dd9, "fuchsia",                               register=True)
    FUCHSIA_PINK                         : ColorRGB = lambda *_: ColorRGB(0xff77ff, "fuchsia-pink",                          register=True)
    FUCHSIA_PURPLE                       : ColorRGB = lambda *_: ColorRGB(0xcc397b, "fuchsia-purple",                        register=True)
    FUCHSIA_ROSE                         : ColorRGB = lambda *_: ColorRGB(0xc74375, "fuchsia-rose",                          register=True)
    FULL_BLACK                           : ColorRGB = lambda *_: ColorRGB(0x000000, "full-black",                            register=True, aliases=['true-black', 'pure-black'])
    FULL_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x0000ff, "full-blue",                             register=True)
    FULL_CYAN                            : ColorRGB = lambda *_: ColorRGB(0x00ffff, "full-cyan",                             register=True)
    FULL_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x00ff00, "full-green",                            register=True)
    FULL_MAGENTA                         : ColorRGB = lambda *_: ColorRGB(0xff00ff, "full-magenta",                          register=True)
    FULL_RED                             : ColorRGB = lambda *_: ColorRGB(0xff0000, "full-red",                              register=True)
    FULL_WHITE                           : ColorRGB = lambda *_: ColorRGB(0xffffff, "full-white",                            register=True, aliases=['true-white', 'pure-white', 'doctor'])
    FULL_YELLOW                          : ColorRGB = lambda *_: ColorRGB(0xffff00, "full-yellow",                           register=True)
    FULVOUS                              : ColorRGB = lambda *_: ColorRGB(0xe48400, "fulvous",                               register=True)
    FUZZY_WUZZY                          : ColorRGB = lambda *_: ColorRGB(0xcc6666, "fuzzy-wuzzy",                           register=True)
    GAINSBORO                            : ColorRGB = lambda *_: ColorRGB(0xdcdcdc, "gainsboro",                             register=True)
    GAMBOGE                              : ColorRGB = lambda *_: ColorRGB(0xe49b0f, "gamboge",                               register=True, variation_map={
             0xffb61e:   "too" })
    GAMBOGE_ORANGE                       : ColorRGB = lambda *_: ColorRGB(0x996600, "gamboge-orange",                        register=True)
    GARGOYLE_GAS                         : ColorRGB = lambda *_: ColorRGB(0xffdf46, "gargoyle-gas",                          register=True)
    GARNET                               : ColorRGB = lambda *_: ColorRGB(0x733635, "garnet",                                register=True)
    GENERIC_VIRIDIAN                     : ColorRGB = lambda *_: ColorRGB(0x007f66, "generic-viridian",                      register=True)
    GHOST_WHITE                          : ColorRGB = lambda *_: ColorRGB(0xf8f8ff, "ghost-white",                           register=True)
    GIANTS_CLUB                          : ColorRGB = lambda *_: ColorRGB(0xb05c52, "giants-club",                           register=True)
    GIANTS_ORANGE                        : ColorRGB = lambda *_: ColorRGB(0xfe5a1d, "giants-orange",                         register=True)
    GIN                                  : ColorRGB = lambda *_: ColorRGB(0xd8e4bc, "gin",                                   register=True)
    GLAUCOUS                             : ColorRGB = lambda *_: ColorRGB(0x6082b6, "glaucous",                              register=True)
    GLAZED_PERSIMMON                     : ColorRGB = lambda *_: ColorRGB(0xd34e36, "glazed-persimmon",                      register=True)
    GLOSSY_GRAPE                         : ColorRGB = lambda *_: ColorRGB(0xab92b3, "glossy-grape",                          register=True)
    GO_GREEN                             : ColorRGB = lambda *_: ColorRGB(0x00ab66, "go-green",                              register=True)
    GOLD                                 : ColorRGB = lambda *_: ColorRGB(0xdbb40c, "gold",                                  register=True, variation_map={
             0xe6be8a:   "crayola", 
             0xd4af37:   "metallic" })
    GOLD_BROWN                           : ColorRGB = lambda *_: ColorRGB(0xcb7e1f, "gold-brown",                            register=True)
    GOLD_FOIL                            : ColorRGB = lambda *_: ColorRGB(0xbd9b16, "gold-foil",                             register=True)
    GOLD_FUSION                          : ColorRGB = lambda *_: ColorRGB(0x85754e, "gold-fusion",                           register=True)
    GOLDEN                               : ColorRGB = lambda *_: ColorRGB(0xf5bf03, "golden",                                register=True)
    GOLDEN_BROWN                         : ColorRGB = lambda *_: ColorRGB(0xb27a01, "golden-brown",                          register=True, variation_map={
             0xc66b27:   "kincha" })
    GOLDEN_FALLEN_LEAVES                 : ColorRGB = lambda *_: ColorRGB(0xe29c45, "golden-fallen-leaves",                  register=True)
    GOLDEN_GRAY_BAMBOO                   : ColorRGB = lambda *_: ColorRGB(0x7d4e2d, "golden-gray-bamboo",                    register=True)
    GOLDEN_OAK                           : ColorRGB = lambda *_: ColorRGB(0xbb8141, "golden-oak",                            register=True)
    GOLDEN_POPPY                         : ColorRGB = lambda *_: ColorRGB(0xfcc200, "golden-poppy",                          register=True)
    GOLDEN_ROD                           : ColorRGB = lambda *_: ColorRGB(0xf9bc08, "golden-rod",                            register=True)
    GOLDEN_STATE                         : ColorRGB = lambda *_: ColorRGB(0xffab00, "golden-state",                          register=True, aliases=['jira-yellow'])
    GOLDEN_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xfec615, "golden-yellow",                         register=True, variation_map={
             0xffa400:   "and-a-particular-rose-varietal" })
    GOLDENROD                            : ColorRGB = lambda *_: ColorRGB(0xfac205, "goldenrod",                             register=True)
    GOOGLE_CHROME_BLUE                   : ColorRGB = lambda *_: ColorRGB(0x4c8bf5, "google-chrome-blue",                    register=True)
    GOOGLE_CHROME_GREEN                  : ColorRGB = lambda *_: ColorRGB(0x1aa260, "google-chrome-green",                   register=True)
    GOOGLE_CHROME_RED                    : ColorRGB = lambda *_: ColorRGB(0xde5246, "google-chrome-red",                     register=True)
    GOOGLE_CHROME_YELLOW                 : ColorRGB = lambda *_: ColorRGB(0xffce44, "google-chrome-yellow",                  register=True)
    GORYEO_STOREROOM                     : ColorRGB = lambda *_: ColorRGB(0x203838, "goryeo-storeroom",                      register=True)
    GRAMS_HAIR                           : ColorRGB = lambda *_: ColorRGB(0xf4f5f7, "grams-hair",                            register=True)
    GRAMS_SOFA                           : ColorRGB = lambda *_: ColorRGB(0xe6fcff, "grams-sofa",                            register=True)
    GRANITE_GRAY                         : ColorRGB = lambda *_: ColorRGB(0x676767, "granite-gray",                          register=True)
    GRANNY_SMITH_APPLE                   : ColorRGB = lambda *_: ColorRGB(0xa8e4a0, "granny-smith-apple",                    register=True)
    GRAPE                                : ColorRGB = lambda *_: ColorRGB(0x6c3461, "grape",                                 register=True)
    GRAPE_MOUSE                          : ColorRGB = lambda *_: ColorRGB(0x63424b, "grape-mouse",                           register=True)
    GRAPE_PURPLE                         : ColorRGB = lambda *_: ColorRGB(0x5d1451, "grape-purple",                          register=True)
    GRAPEFRUIT                           : ColorRGB = lambda *_: ColorRGB(0xfd5956, "grapefruit",                            register=True)
    GRASS                                : ColorRGB = lambda *_: ColorRGB(0x5cac2d, "grass",                                 register=True)
    GRASS_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x3f9b0b, "grass-green",                           register=True)
    GRASSY_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x419c03, "grassy-green",                          register=True)
    GRAY                                 : ColorRGB = lambda *_: ColorRGB(0xbebebe, "gray",                                  register=True)
    GRAY_ASPARAGUS                       : ColorRGB = lambda *_: ColorRGB(0x465945, "gray-asparagus",                        register=True)
    GRAY_KALM                            : ColorRGB = lambda *_: ColorRGB(0x676c73, "gray-kalm",                             register=True)
    GRAY_PANIK                           : ColorRGB = lambda *_: ColorRGB(0x424242, "gray-panik",                            register=True)
    GRAYED_RED                           : ColorRGB = lambda *_: ColorRGB(0xbc2d29, "grayed-red",                            register=True)
    GRAYISH_DARK_GREEN                   : ColorRGB = lambda *_: ColorRGB(0x656255, "grayish-dark-green",                    register=True)
    GRECIAN_BRONZE                       : ColorRGB = lambda *_: ColorRGB(0x923916, "grecian-bronze",                        register=True)
    GREEN                                : ColorRGB = lambda *_: ColorRGB(0x15b01a, "green",                                 register=True, variation_map={
             0x1cac78:   "crayola", 
             0x2a603b:   "midori", 
             0x00a877:   "munsell", 
             0x009f6b:   "ncs", 
             0x00ad43:   "pantone", 
             0x00a550:   "pigment", 
             0x66b032:   "ryb" })
    GREEN_APPLE                          : ColorRGB = lambda *_: ColorRGB(0x5edc1f, "green-apple",                           register=True)
    GREEN_BAMBOO                         : ColorRGB = lambda *_: ColorRGB(0x006442, "green-bamboo",                          register=True)
    GREEN_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x06b48b, "green-blue",                            register=True, variation_map={
             0x2887c8:   "crayola" })
    GREEN_BROWN                          : ColorRGB = lambda *_: ColorRGB(0x544e03, "green-brown",                           register=True)
    GREEN_COLA                           : ColorRGB = lambda *_: ColorRGB(0x4c721d, "green-cola",                            register=True)
    GREEN_CYAN                           : ColorRGB = lambda *_: ColorRGB(0x009966, "green-cyan",                            register=True)
    GREEN_GREY                           : ColorRGB = lambda *_: ColorRGB(0x77926f, "green-grey",                            register=True)
    GREEN_KALM                           : ColorRGB = lambda *_: ColorRGB(0x769440, "green-kalm",                            register=True)
    GREEN_LIZARD                         : ColorRGB = lambda *_: ColorRGB(0xa7f432, "green-lizard",                          register=True)
    GREEN_PANIK                          : ColorRGB = lambda *_: ColorRGB(0x5af78d, "green-panik",                           register=True)
    GREEN_SHEEN                          : ColorRGB = lambda *_: ColorRGB(0x6eaea1, "green-sheen",                           register=True)
    GREEN_SLIME                          : ColorRGB = lambda *_: ColorRGB(0x65ff00, "green-slime",                           register=True)
    GREEN_TEA                            : ColorRGB = lambda *_: ColorRGB(0x824b35, "green-tea",                             register=True, variation_map={
             0x57d9a3:   "atlassian" })
    GREEN_TEAL                           : ColorRGB = lambda *_: ColorRGB(0x0cb577, "green-teal",                            register=True)
    GREEN_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xc9ff27, "green-yellow",                          register=True, variation_map={
             0xf0e891:   "crayola" })
    GREENBLUE                            : ColorRGB = lambda *_: ColorRGB(0x23c48b, "greenblue",                             register=True)
    GREENFINCH                           : ColorRGB = lambda *_: ColorRGB(0xbda928, "greenfinch",                            register=True)
    GREENISH                             : ColorRGB = lambda *_: ColorRGB(0x40a368, "greenish",                              register=True)
    GREENISH_BEIGE                       : ColorRGB = lambda *_: ColorRGB(0xc9d179, "greenish-beige",                        register=True)
    GREENISH_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x0b8b87, "greenish-blue",                         register=True)
    GREENISH_BROWN                       : ColorRGB = lambda *_: ColorRGB(0x696112, "greenish-brown",                        register=True)
    GREENISH_CYAN                        : ColorRGB = lambda *_: ColorRGB(0x2afeb7, "greenish-cyan",                         register=True)
    GREENISH_GREY                        : ColorRGB = lambda *_: ColorRGB(0x96ae8d, "greenish-grey",                         register=True)
    GREENISH_TAN                         : ColorRGB = lambda *_: ColorRGB(0xbccb7a, "greenish-tan",                          register=True)
    GREENISH_TEAL                        : ColorRGB = lambda *_: ColorRGB(0x32bf84, "greenish-teal",                         register=True)
    GREENISH_TURQUOISE                   : ColorRGB = lambda *_: ColorRGB(0x00fbb0, "greenish-turquoise",                    register=True)
    GREENISH_YELLOW                      : ColorRGB = lambda *_: ColorRGB(0xcdfd02, "greenish-yellow",                       register=True)
    GREENY_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x42b395, "greeny-blue",                           register=True)
    GREENY_BROWN                         : ColorRGB = lambda *_: ColorRGB(0x696006, "greeny-brown",                          register=True)
    GREENY_GREY                          : ColorRGB = lambda *_: ColorRGB(0x7ea07a, "greeny-grey",                           register=True)
    GREENY_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xc6f808, "greeny-yellow",                         register=True)
    GREY                                 : ColorRGB = lambda *_: ColorRGB(0x929591, "grey",                                  register=True)
    GREY_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x6b8ba4, "grey-blue",                             register=True)
    GREY_BROWN                           : ColorRGB = lambda *_: ColorRGB(0x7f7053, "grey-brown",                            register=True)
    GREY_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x789b73, "grey-green",                            register=True)
    GREY_PINK                            : ColorRGB = lambda *_: ColorRGB(0xc3909b, "grey-pink",                             register=True)
    GREY_PURPLE                          : ColorRGB = lambda *_: ColorRGB(0x826d8c, "grey-purple",                           register=True)
    GREY_TEAL                            : ColorRGB = lambda *_: ColorRGB(0x5e9b8a, "grey-teal",                             register=True)
    GREYBLUE                             : ColorRGB = lambda *_: ColorRGB(0x77a1b5, "greyblue",                              register=True)
    GREYISH                              : ColorRGB = lambda *_: ColorRGB(0xa8a495, "greyish",                               register=True)
    GREYISH_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x5e819d, "greyish-blue",                          register=True)
    GREYISH_BROWN                        : ColorRGB = lambda *_: ColorRGB(0x7a6a4f, "greyish-brown",                         register=True)
    GREYISH_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x82a67d, "greyish-green",                         register=True)
    GREYISH_PINK                         : ColorRGB = lambda *_: ColorRGB(0xc88d94, "greyish-pink",                          register=True)
    GREYISH_PURPLE                       : ColorRGB = lambda *_: ColorRGB(0x887191, "greyish-purple",                        register=True)
    GREYISH_TEAL                         : ColorRGB = lambda *_: ColorRGB(0x719f91, "greyish-teal",                          register=True)
    GROSS_GREEN                          : ColorRGB = lambda *_: ColorRGB(0xa0bf16, "gross-green",                           register=True)
    GRULLO                               : ColorRGB = lambda *_: ColorRGB(0xa99a86, "grullo",                                register=True)
    GUNMETAL                             : ColorRGB = lambda *_: ColorRGB(0x536267, "gunmetal",                              register=True)
    GUPPIE_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x00ff7f, "guppie-green",                          register=True)
    GUYABANO                             : ColorRGB = lambda *_: ColorRGB(0xf8f8f8, "guyabano",                              register=True)
    HAIRY_FAIRY                          : ColorRGB = lambda *_: ColorRGB(0x79e2f2, "hairy-fairy",                           register=True)
    HALAYA_UBE                           : ColorRGB = lambda *_: ColorRGB(0x663854, "halaya-ube",                            register=True)
    HALF                                 : ColorRGB = lambda *_: ColorRGB(0x8d608c, "half",                                  register=True)
    HALLOWEEN_ORANGE                     : ColorRGB = lambda *_: ColorRGB(0xeb6123, "halloween-orange",                      register=True)
    HAN_BLUE                             : ColorRGB = lambda *_: ColorRGB(0x446ccf, "han-blue",                              register=True)
    HAN_PURPLE                           : ColorRGB = lambda *_: ColorRGB(0x5218fa, "han-purple",                            register=True)
    HARBOR_RAT                           : ColorRGB = lambda *_: ColorRGB(0x757d75, "harbor-rat",                            register=True)
    HARLEQUIN                            : ColorRGB = lambda *_: ColorRGB(0x3fff00, "harlequin",                             register=True)
    HARLEQUIN_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x46cb18, "harlequin-green",                       register=True)
    HARMONIOUS_ROSE                      : ColorRGB = lambda *_: ColorRGB(0xf29cb7, "harmonious-rose",                       register=True)
    HARVEST_GOLD                         : ColorRGB = lambda *_: ColorRGB(0xda9100, "harvest-gold",                          register=True)
    HAWAII_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x00c3e3, "hawaii-blue",                           register=True)
    HAZEL                                : ColorRGB = lambda *_: ColorRGB(0x8e7618, "hazel",                                 register=True)
    HEART_GOLD                           : ColorRGB = lambda *_: ColorRGB(0x808000, "heart-gold",                            register=True)
    HEAT_WAVE                            : ColorRGB = lambda *_: ColorRGB(0xff7a00, "heat-wave",                             register=True)
    HEATHER                              : ColorRGB = lambda *_: ColorRGB(0xa484ac, "heather",                               register=True)
    HELIOTROPE                           : ColorRGB = lambda *_: ColorRGB(0xd94ff5, "heliotrope",                            register=True)
    HELIOTROPE_GRAY                      : ColorRGB = lambda *_: ColorRGB(0xaa98a9, "heliotrope-gray",                       register=True)
    HELIOTROPE_MAGENTA                   : ColorRGB = lambda *_: ColorRGB(0xaa00bb, "heliotrope-magenta",                    register=True)
    HERKY_JERKY                          : ColorRGB = lambda *_: ColorRGB(0x998dd9, "herky-jerky",                           register=True)
    HI_BLUE_KALM                         : ColorRGB = lambda *_: ColorRGB(0x81a2be, "hi-blue-kalm",                          register=True)
    HI_BLUE_PANIK                        : ColorRGB = lambda *_: ColorRGB(0xbeb4fe, "hi-blue-panik",                         register=True)
    HI_CYAN_KALM                         : ColorRGB = lambda *_: ColorRGB(0x8abeb7, "hi-cyan-kalm",                          register=True)
    HI_CYAN_PANIK                        : ColorRGB = lambda *_: ColorRGB(0xa1fffe, "hi-cyan-panik",                         register=True)
    HI_GREEN_KALM                        : ColorRGB = lambda *_: ColorRGB(0xa4bd68, "hi-green-kalm",                         register=True)
    HI_GREEN_PANIK                       : ColorRGB = lambda *_: ColorRGB(0xacf99f, "hi-green-panik",                        register=True)
    HI_MAGENTA_KALM                      : ColorRGB = lambda *_: ColorRGB(0xb294bb, "hi-magenta-kalm",                       register=True)
    HI_MAGENTA_PANIK                     : ColorRGB = lambda *_: ColorRGB(0xff90fd, "hi-magenta-panik",                      register=True)
    HI_RED_KALM                          : ColorRGB = lambda *_: ColorRGB(0xcc6666, "hi-red-kalm",                           register=True)
    HI_RED_PANIK                         : ColorRGB = lambda *_: ColorRGB(0xfe7f74, "hi-red-panik",                          register=True)
    HI_WHITE_KALM                        : ColorRGB = lambda *_: ColorRGB(0xdde3ec, "hi-white-kalm",                         register=True)
    HI_WHITE_PANIK                       : ColorRGB = lambda *_: ColorRGB(0xf1f1f0, "hi-white-panik",                        register=True)
    HI_YELLOW_KALM                       : ColorRGB = lambda *_: ColorRGB(0xf0c674, "hi-yellow-kalm",                        register=True)
    HI_YELLOW_PANIK                      : ColorRGB = lambda *_: ColorRGB(0xfffed4, "hi-yellow-panik",                       register=True)
    HIGHLIGHTER_GREEN                    : ColorRGB = lambda *_: ColorRGB(0x1bfc06, "highlighter-green",                     register=True)
    HONEYDEW                             : ColorRGB = lambda *_: ColorRGB(0xf0fff0, "honeydew",                              register=True)
    HONOLULU_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x006db0, "honolulu-blue",                         register=True)
    HOOKERS_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x49796b, "hookers-green",                         register=True)
    HORSETAIL                            : ColorRGB = lambda *_: ColorRGB(0x3d5d42, "horsetail",                             register=True)
    HOSPITAL_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x9be5aa, "hospital-green",                        register=True)
    HOT_GREEN                            : ColorRGB = lambda *_: ColorRGB(0x25ff29, "hot-green",                             register=True)
    HOT_MAGENTA                          : ColorRGB = lambda *_: ColorRGB(0xf504c9, "hot-magenta",                           register=True)
    HOT_PINK                             : ColorRGB = lambda *_: ColorRGB(0xff028d, "hot-pink",                              register=True)
    HOT_PURPLE                           : ColorRGB = lambda *_: ColorRGB(0xcb00f5, "hot-purple",                            register=True)
    HTTP_DELETE                          : ColorRGB = lambda *_: ColorRGB(0xc75450, "http-delete",                           register=True)
    HTTP_GET                             : ColorRGB = lambda *_: ColorRGB(0x499c54, "http-get",                              register=True)
    HTTP_PATCH                           : ColorRGB = lambda *_: ColorRGB(0xabb2bf, "http-patch",                            register=True)
    HTTP_POST                            : ColorRGB = lambda *_: ColorRGB(0xd9a343, "http-post",                             register=True)
    HTTP_PUT                             : ColorRGB = lambda *_: ColorRGB(0x6897bb, "http-put",                              register=True)
    HUAWEI_RED                           : ColorRGB = lambda *_: ColorRGB(0xcf0a2c, "huawei-red",                            register=True, variation_map={
             0xed1c24:   "1987-2018" })
    HUMBOLDT_FOG                         : ColorRGB = lambda *_: ColorRGB(0x7a869a, "humboldt-fog",                          register=True)
    HUNTER_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x0b4008, "hunter-green",                          register=True)
    IBIS_WING                            : ColorRGB = lambda *_: ColorRGB(0xf58f84, "ibis-wing",                             register=True)
    ICATHIAN_YELLOW                      : ColorRGB = lambda *_: ColorRGB(0xe7c899, "icathian-yellow",                       register=True)
    ICE                                  : ColorRGB = lambda *_: ColorRGB(0xd6fffa, "ice",                                   register=True)
    ICE_BLUE                             : ColorRGB = lambda *_: ColorRGB(0xd7fffe, "ice-blue",                              register=True)
    ICEBERG                              : ColorRGB = lambda *_: ColorRGB(0x71a6d2, "iceberg",                               register=True)
    ICED_TEA                             : ColorRGB = lambda *_: ColorRGB(0x923c01, "iced-tea",                              register=True)
    ICKY_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x8fae22, "icky-green",                            register=True)
    ICTERINE                             : ColorRGB = lambda *_: ColorRGB(0xfcf75e, "icterine",                              register=True)
    IGUANA_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x71bc78, "iguana-green",                          register=True)
    ILLUMINATING_EMERALD                 : ColorRGB = lambda *_: ColorRGB(0x319177, "illuminating-emerald",                  register=True)
    IMPERIAL                             : ColorRGB = lambda *_: ColorRGB(0x602f6b, "imperial",                              register=True)
    IMPERIAL_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x002395, "imperial-blue",                         register=True)
    IMPERIAL_PURPLE                      : ColorRGB = lambda *_: ColorRGB(0x66023c, "imperial-purple",                       register=True)
    IMPERIAL_RED                         : ColorRGB = lambda *_: ColorRGB(0xed2939, "imperial-red",                          register=True)
    INCHWORM                             : ColorRGB = lambda *_: ColorRGB(0xb2ec5d, "inchworm",                              register=True)
    INCOMPATIBILITY                      : ColorRGB = lambda *_: ColorRGB(0xffd19e, "incompatibility",                       register=True)
    INDEPENDENCE                         : ColorRGB = lambda *_: ColorRGB(0x4c516d, "independence",                          register=True)
    INDIA_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x138808, "india-green",                           register=True)
    INDIAN_RED                           : ColorRGB = lambda *_: ColorRGB(0x850e04, "indian-red",                            register=True)
    INDIAN_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xe3a857, "indian-yellow",                         register=True)
    INDIGO                               : ColorRGB = lambda *_: ColorRGB(0x380282, "indigo",                                register=True, variation_map={
             0x233067:   "rainbow" })
    INDIGO_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x3a18b1, "indigo-blue",                           register=True)
    INDIGO_CODIUM_FRAGILE_SEAWEED_BROWN  : ColorRGB = lambda *_: ColorRGB(0x2e372e, "indigo-codium-fragile-seaweed-brown",   register=True)
    INDIGO_DYE                           : ColorRGB = lambda *_: ColorRGB(0x091f92, "indigo-dye",                            register=True)
    INDIGO_INK_BROWN                     : ColorRGB = lambda *_: ColorRGB(0x393432, "indigo-ink-brown",                      register=True)
    INDIGO_WHITE                         : ColorRGB = lambda *_: ColorRGB(0xebf6f7, "indigo-white",                          register=True)
    INFRA_RED                            : ColorRGB = lambda *_: ColorRGB(0xff496c, "infra-red",                             register=True)
    INK                                  : ColorRGB = lambda *_: ColorRGB(0x27221f, "ink",                                   register=True)
    INSECT_SCREEN                        : ColorRGB = lambda *_: ColorRGB(0x2d4436, "insect-screen",                         register=True)
    INSIDE_OF_A_BOTTLE                   : ColorRGB = lambda *_: ColorRGB(0xc6c2b6, "inside-of-a-bottle",                    register=True)
    INTERDIMENSIONAL_BLUE                : ColorRGB = lambda *_: ColorRGB(0x360ccc, "interdimensional-blue",                 register=True)
    INTERNATIONAL_ORANGE                 : ColorRGB = lambda *_: ColorRGB(0xff4f00, "international-orange",                  register=True, variation_map={
             0xba160c:   "engineering", 
             0xc0362c:   "golden-gate-bridge" })
    IRIS                                 : ColorRGB = lambda *_: ColorRGB(0x6258c4, "iris",                                  register=True, variation_map={
             0x763568:   "ayame-iro" })
    IRISH_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x019529, "irish-green",                           register=True)
    IRON                                 : ColorRGB = lambda *_: ColorRGB(0xa19d94, "iron",                                  register=True, variation_map={
             0x2b3733:   "tetsu-iro" })
    IRON_HEAD_FLOWER                     : ColorRGB = lambda *_: ColorRGB(0x344d56, "iron-head-flower",                      register=True)
    IRON_STORAGE                         : ColorRGB = lambda *_: ColorRGB(0x2b3736, "iron-storage",                          register=True)
    IRRESISTIBLE                         : ColorRGB = lambda *_: ColorRGB(0xb3446c, "irresistible",                          register=True)
    ISABELLINE                           : ColorRGB = lambda *_: ColorRGB(0xf4f0ec, "isabelline",                            register=True)
    ISLAMIC_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x009000, "islamic-green",                         register=True)
    ISPAHAN_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x303940, "ispahan-blue",                          register=True)
    ITALIAN_ICE                          : ColorRGB = lambda *_: ColorRGB(0xe9f6ef, "italian-ice",                           register=True)
    ITALIAN_LEATHER                      : ColorRGB = lambda *_: ColorRGB(0x635a52, "italian-leather",                       register=True)
    ITALIAN_ROAST                        : ColorRGB = lambda *_: ColorRGB(0x906652, "italian-roast",                         register=True)
    IVORY                                : ColorRGB = lambda *_: ColorRGB(0xffffcb, "ivory",                                 register=True)
    IWAI_BROWN                           : ColorRGB = lambda *_: ColorRGB(0x5e5545, "iwai-brown",                            register=True)
    JACARTA                              : ColorRGB = lambda *_: ColorRGB(0x3d325d, "jacarta",                               register=True)
    JACKO_BEAN                           : ColorRGB = lambda *_: ColorRGB(0x413628, "jacko-bean",                            register=True)
    JADE                                 : ColorRGB = lambda *_: ColorRGB(0x1fa774, "jade",                                  register=True)
    JADE_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x2baf6a, "jade-green",                            register=True)
    JAMES_BLONDE                         : ColorRGB = lambda *_: ColorRGB(0xfffae6, "james-blonde",                          register=True)
    JAPANESE_CARMINE                     : ColorRGB = lambda *_: ColorRGB(0x9d2933, "japanese-carmine",                      register=True)
    JAPANESE_INDIGO                      : ColorRGB = lambda *_: ColorRGB(0x264348, "japanese-indigo",                       register=True)
    JAPANESE_IRIS_AND_SOOTY_BAMBOO       : ColorRGB = lambda *_: ColorRGB(0x7f5d3b, "japanese-iris-and-sooty-bamboo",        register=True)
    JAPANESE_LAUREL                      : ColorRGB = lambda *_: ColorRGB(0x2f7532, "japanese-laurel",                       register=True)
    JAPANESE_TRIANDRA_GRASS              : ColorRGB = lambda *_: ColorRGB(0xe2b13c, "japanese-triandra-grass",               register=True)
    JAPANESE_VIOLET                      : ColorRGB = lambda *_: ColorRGB(0x5b3256, "japanese-violet",                       register=True)
    JASMINE                              : ColorRGB = lambda *_: ColorRGB(0xf8de7e, "jasmine",                               register=True)
    JASPER                               : ColorRGB = lambda *_: ColorRGB(0xd73b3e, "jasper",                                register=True)
    JASPER_ORANGE                        : ColorRGB = lambda *_: ColorRGB(0xde8f4e, "jasper-orange",                         register=True)
    JAZZBERRY_JAM                        : ColorRGB = lambda *_: ColorRGB(0xa50b5e, "jazzberry-jam",                         register=True)
    JELLY_BEAN                           : ColorRGB = lambda *_: ColorRGB(0xda614e, "jelly-bean",                            register=True)
    JELLY_BEAN_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x44798e, "jelly-bean-blue",                       register=True)
    JET                                  : ColorRGB = lambda *_: ColorRGB(0x343434, "jet",                                   register=True)
    JET_STREAM                           : ColorRGB = lambda *_: ColorRGB(0xbbd0c9, "jet-stream",                            register=True)
    JOLLY_FUN_TIME                       : ColorRGB = lambda *_: ColorRGB(0xdfe1e6, "jolly-fun-time",                        register=True)
    JONQUIL                              : ColorRGB = lambda *_: ColorRGB(0xf4ca16, "jonquil",                               register=True)
    JORDY_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x8ab9f1, "jordy-blue",                            register=True)
    JUNE_BUD                             : ColorRGB = lambda *_: ColorRGB(0xbdda57, "june-bud",                              register=True)
    JUNGLE_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x048243, "jungle-green",                          register=True)
    KARL                                 : ColorRGB = lambda *_: ColorRGB(0xc1c7d0, "karl",                                  register=True)
    KEEN_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x006644, "keen-green",                            register=True)
    KELLEY_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x009337, "kelley-green",                          register=True)
    KELLY_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x02ab2e, "kelly-green",                           register=True)
    KENYAN_COPPER                        : ColorRGB = lambda *_: ColorRGB(0x7c1c05, "kenyan-copper",                         register=True)
    KEPPEL                               : ColorRGB = lambda *_: ColorRGB(0x3ab09e, "keppel",                                register=True)
    KERMIT_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x5cb200, "kermit-green",                          register=True)
    KEY_LIME                             : ColorRGB = lambda *_: ColorRGB(0xaeff6e, "key-lime",                              register=True)
    KHAKI                                : ColorRGB = lambda *_: ColorRGB(0xaaa662, "khaki",                                 register=True)
    KHAKI_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x728639, "khaki-green",                           register=True)
    KIMONO_STORAGE                       : ColorRGB = lambda *_: ColorRGB(0x3d4c51, "kimono-storage",                        register=True)
    KIWI                                 : ColorRGB = lambda *_: ColorRGB(0x9cef43, "kiwi",                                  register=True)
    KIWI_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x8ee53f, "kiwi-green",                            register=True)
    KOBE                                 : ColorRGB = lambda *_: ColorRGB(0x882d17, "kobe",                                  register=True)
    KOBI                                 : ColorRGB = lambda *_: ColorRGB(0xe79fc4, "kobi",                                  register=True)
    KOMBU_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x354230, "kombu-green",                           register=True)
    KSU_PURPLE                           : ColorRGB = lambda *_: ColorRGB(0x512888, "ksu-purple",                            register=True)
    KU_CRIMSON                           : ColorRGB = lambda *_: ColorRGB(0xe8000d, "ku-crimson",                            register=True)
    KUMQUAT                              : ColorRGB = lambda *_: ColorRGB(0xfb9912, "kumquat",                               register=True)
    LANGUID_LAVENDER                     : ColorRGB = lambda *_: ColorRGB(0xd6cadd, "languid-lavender",                      register=True)
    LANZONES                             : ColorRGB = lambda *_: ColorRGB(0xe0bc5b, "lanzones",                              register=True)
    LAPIS_LAZULI                         : ColorRGB = lambda *_: ColorRGB(0x26619c, "lapis-lazuli",                          register=True, variation_map={
             0x1f4788:   "ruri-iro" })
    LASER_LEMON                          : ColorRGB = lambda *_: ColorRGB(0xffff66, "laser-lemon",                           register=True)
    LAUREL_GREEN                         : ColorRGB = lambda *_: ColorRGB(0xa9ba9d, "laurel-green",                          register=True)
    LAVA                                 : ColorRGB = lambda *_: ColorRGB(0xcf1020, "lava",                                  register=True)
    LAVENDER                             : ColorRGB = lambda *_: ColorRGB(0xc79fef, "lavender",                              register=True, variation_map={
             0xb57edc:   "floral" })
    LAVENDER_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x8b88f8, "lavender-blue",                         register=True)
    LAVENDER_BLUSH                       : ColorRGB = lambda *_: ColorRGB(0xfff0f5, "lavender-blush",                        register=True)
    LAVENDER_GRAY                        : ColorRGB = lambda *_: ColorRGB(0xc4c3d0, "lavender-gray",                         register=True)
    LAVENDER_INDIGO                      : ColorRGB = lambda *_: ColorRGB(0x9457eb, "lavender-indigo",                       register=True)
    LAVENDER_MAGENTA                     : ColorRGB = lambda *_: ColorRGB(0xee82ee, "lavender-magenta",                      register=True)
    LAVENDER_MIST                        : ColorRGB = lambda *_: ColorRGB(0xe6e6fa, "lavender-mist",                         register=True)
    LAVENDER_PINK                        : ColorRGB = lambda *_: ColorRGB(0xdd85d7, "lavender-pink",                         register=True)
    LAVENDER_PURPLE                      : ColorRGB = lambda *_: ColorRGB(0x967bb6, "lavender-purple",                       register=True)
    LAVENDER_ROSE                        : ColorRGB = lambda *_: ColorRGB(0xfba0e3, "lavender-rose",                         register=True)
    LAVENDER_SECRET                      : ColorRGB = lambda *_: ColorRGB(0xeae6ff, "lavender-secret",                       register=True)
    LAWN_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x4da409, "lawn-green",                            register=True)
    LEAF                                 : ColorRGB = lambda *_: ColorRGB(0x71aa34, "leaf",                                  register=True)
    LEAF_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x5ca904, "leaf-green",                            register=True)
    LEAFY_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x51b73b, "leafy-green",                           register=True)
    LEATHER                              : ColorRGB = lambda *_: ColorRGB(0xac7434, "leather",                               register=True)
    LEGAL_DYE                            : ColorRGB = lambda *_: ColorRGB(0x2e211b, "legal-dye",                             register=True)
    LEMON                                : ColorRGB = lambda *_: ColorRGB(0xfdff52, "lemon",                                 register=True)
    LEMON_CHIFFON                        : ColorRGB = lambda *_: ColorRGB(0xfffacd, "lemon-chiffon",                         register=True)
    LEMON_CURRY                          : ColorRGB = lambda *_: ColorRGB(0xcca01d, "lemon-curry",                           register=True)
    LEMON_GLACIER                        : ColorRGB = lambda *_: ColorRGB(0xfdff00, "lemon-glacier",                         register=True)
    LEMON_GREEN                          : ColorRGB = lambda *_: ColorRGB(0xadf802, "lemon-green",                           register=True)
    LEMON_ICED_TEA                       : ColorRGB = lambda *_: ColorRGB(0xbd3000, "lemon-iced-tea",                        register=True)
    LEMON_LIME                           : ColorRGB = lambda *_: ColorRGB(0xbffe28, "lemon-lime",                            register=True, variation_map={
             0x5cff67:   "2" })
    LEMON_MERINGUE                       : ColorRGB = lambda *_: ColorRGB(0xf6eabe, "lemon-meringue",                        register=True)
    LEMON_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xfdff38, "lemon-yellow",                          register=True, variation_map={
             0xffff9f:   "crayola" })
    LENURPLE                             : ColorRGB = lambda *_: ColorRGB(0xba93d8, "lenurple",                              register=True)
    LIBERTY                              : ColorRGB = lambda *_: ColorRGB(0x545aa7, "liberty",                               register=True)
    LICHEN                               : ColorRGB = lambda *_: ColorRGB(0x8fb67b, "lichen",                                register=True)
    LICORICE                             : ColorRGB = lambda *_: ColorRGB(0x1a1110, "licorice",                              register=True)
    LIGHT_AQUA                           : ColorRGB = lambda *_: ColorRGB(0x8cffdb, "light-aqua",                            register=True)
    LIGHT_AQUAMARINE                     : ColorRGB = lambda *_: ColorRGB(0x7bfdc7, "light-aquamarine",                      register=True)
    LIGHT_BEIGE                          : ColorRGB = lambda *_: ColorRGB(0xfffeb6, "light-beige",                           register=True)
    LIGHT_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x95d0fc, "light-blue",                            register=True, variation_map={
             0x48929b:   "asagi-iro" })
    LIGHT_BLUE_FLOWER                    : ColorRGB = lambda *_: ColorRGB(0x1d697c, "light-blue-flower",                     register=True)
    LIGHT_BLUE_GREEN                     : ColorRGB = lambda *_: ColorRGB(0x7efbb3, "light-blue-green",                      register=True)
    LIGHT_BLUE_GREY                      : ColorRGB = lambda *_: ColorRGB(0xb7c9e2, "light-blue-grey",                       register=True)
    LIGHT_BLUE_SILK                      : ColorRGB = lambda *_: ColorRGB(0x044f67, "light-blue-silk",                       register=True)
    LIGHT_BLUISH_GREEN                   : ColorRGB = lambda *_: ColorRGB(0x76fda8, "light-bluish-green",                    register=True)
    LIGHT_BRIGHT_GREEN                   : ColorRGB = lambda *_: ColorRGB(0x53fe5c, "light-bright-green",                    register=True)
    LIGHT_BROWN                          : ColorRGB = lambda *_: ColorRGB(0xad8150, "light-brown",                           register=True)
    LIGHT_BURGUNDY                       : ColorRGB = lambda *_: ColorRGB(0xa8415b, "light-burgundy",                        register=True)
    LIGHT_CARMINE_PINK                   : ColorRGB = lambda *_: ColorRGB(0xe66771, "light-carmine-pink",                    register=True)
    LIGHT_CHOCOLATE_COSMOS               : ColorRGB = lambda *_: ColorRGB(0x551f2f, "light-chocolate-cosmos",                register=True)
    LIGHT_COBALT_BLUE                    : ColorRGB = lambda *_: ColorRGB(0x88ace0, "light-cobalt-blue",                     register=True)
    LIGHT_CORAL                          : ColorRGB = lambda *_: ColorRGB(0xf08080, "light-coral",                           register=True)
    LIGHT_CRIMSON                        : ColorRGB = lambda *_: ColorRGB(0xf56991, "light-crimson",                         register=True)
    LIGHT_CYAN                           : ColorRGB = lambda *_: ColorRGB(0xacfffc, "light-cyan",                            register=True)
    LIGHT_DEEP_PINK                      : ColorRGB = lambda *_: ColorRGB(0xff5ccd, "light-deep-pink",                       register=True)
    LIGHT_EGGPLANT                       : ColorRGB = lambda *_: ColorRGB(0x894585, "light-eggplant",                        register=True)
    LIGHT_FOREST_GREEN                   : ColorRGB = lambda *_: ColorRGB(0x4f9153, "light-forest-green",                    register=True)
    LIGHT_FRENCH_BEIGE                   : ColorRGB = lambda *_: ColorRGB(0xc8ad7f, "light-french-beige",                    register=True)
    LIGHT_FUCHSIA_PINK                   : ColorRGB = lambda *_: ColorRGB(0xf984ef, "light-fuchsia-pink",                    register=True)
    LIGHT_GOLD                           : ColorRGB = lambda *_: ColorRGB(0xfddc5c, "light-gold",                            register=True)
    LIGHT_GOLDENROD_YELLOW               : ColorRGB = lambda *_: ColorRGB(0xfafad2, "light-goldenrod-yellow",                register=True)
    LIGHT_GRASS_GREEN                    : ColorRGB = lambda *_: ColorRGB(0x9af764, "light-grass-green",                     register=True)
    LIGHT_GRAY                           : ColorRGB = lambda *_: ColorRGB(0xd3d3d3, "light-gray",                            register=True)
    LIGHT_GRAYISH_MAGENTA                : ColorRGB = lambda *_: ColorRGB(0xcc99cc, "light-grayish-magenta",                 register=True)
    LIGHT_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x96f97b, "light-green",                           register=True)
    LIGHT_GREEN_BLUE                     : ColorRGB = lambda *_: ColorRGB(0x56fca2, "light-green-blue",                      register=True)
    LIGHT_GREENISH_BLUE                  : ColorRGB = lambda *_: ColorRGB(0x63f7b4, "light-greenish-blue",                   register=True)
    LIGHT_GREY                           : ColorRGB = lambda *_: ColorRGB(0xd8dcd6, "light-grey",                            register=True)
    LIGHT_GREY_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x9dbcd4, "light-grey-blue",                       register=True)
    LIGHT_GREY_GREEN                     : ColorRGB = lambda *_: ColorRGB(0xb7e1a1, "light-grey-green",                      register=True)
    LIGHT_HOT_PINK                       : ColorRGB = lambda *_: ColorRGB(0xffb3de, "light-hot-pink",                        register=True)
    LIGHT_INDIGO                         : ColorRGB = lambda *_: ColorRGB(0x6d5acf, "light-indigo",                          register=True)
    LIGHT_KHAKI                          : ColorRGB = lambda *_: ColorRGB(0xe6f2a2, "light-khaki",                           register=True)
    LIGHT_LAVENDAR                       : ColorRGB = lambda *_: ColorRGB(0xefc0fe, "light-lavendar",                        register=True)
    LIGHT_LAVENDER                       : ColorRGB = lambda *_: ColorRGB(0xdfc5fe, "light-lavender",                        register=True)
    LIGHT_LIGHT_BLUE                     : ColorRGB = lambda *_: ColorRGB(0xcafffb, "light-light-blue",                      register=True)
    LIGHT_LIGHT_GREEN                    : ColorRGB = lambda *_: ColorRGB(0xc8ffb0, "light-light-green",                     register=True)
    LIGHT_LILAC                          : ColorRGB = lambda *_: ColorRGB(0xedc8ff, "light-lilac",                           register=True)
    LIGHT_LIME                           : ColorRGB = lambda *_: ColorRGB(0xaefd6c, "light-lime",                            register=True)
    LIGHT_LIME_GREEN                     : ColorRGB = lambda *_: ColorRGB(0xb9ff66, "light-lime-green",                      register=True)
    LIGHT_MAGENTA                        : ColorRGB = lambda *_: ColorRGB(0xfa5ff7, "light-magenta",                         register=True)
    LIGHT_MAROON                         : ColorRGB = lambda *_: ColorRGB(0xa24857, "light-maroon",                          register=True)
    LIGHT_MAUVE                          : ColorRGB = lambda *_: ColorRGB(0xc292a1, "light-mauve",                           register=True)
    LIGHT_MEDIUM_ORCHID                  : ColorRGB = lambda *_: ColorRGB(0xd39bcb, "light-medium-orchid",                   register=True)
    LIGHT_MINT                           : ColorRGB = lambda *_: ColorRGB(0xb6ffbb, "light-mint",                            register=True)
    LIGHT_MINT_GREEN                     : ColorRGB = lambda *_: ColorRGB(0xa6fbb2, "light-mint-green",                      register=True)
    LIGHT_MOSS_GREEN                     : ColorRGB = lambda *_: ColorRGB(0xa6c875, "light-moss-green",                      register=True)
    LIGHT_MUSTARD                        : ColorRGB = lambda *_: ColorRGB(0xf7d560, "light-mustard",                         register=True)
    LIGHT_NAVY                           : ColorRGB = lambda *_: ColorRGB(0x155084, "light-navy",                            register=True)
    LIGHT_NAVY_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x2e5a88, "light-navy-blue",                       register=True)
    LIGHT_NEON_GREEN                     : ColorRGB = lambda *_: ColorRGB(0x4efd54, "light-neon-green",                      register=True)
    LIGHT_OLIVE                          : ColorRGB = lambda *_: ColorRGB(0xacbf69, "light-olive",                           register=True)
    LIGHT_OLIVE_GREEN                    : ColorRGB = lambda *_: ColorRGB(0xa4be5c, "light-olive-green",                     register=True)
    LIGHT_ORANGE                         : ColorRGB = lambda *_: ColorRGB(0xfdaa48, "light-orange",                          register=True)
    LIGHT_ORCHID                         : ColorRGB = lambda *_: ColorRGB(0xe6a8d7, "light-orchid",                          register=True)
    LIGHT_PASTEL_GREEN                   : ColorRGB = lambda *_: ColorRGB(0xb2fba5, "light-pastel-green",                    register=True)
    LIGHT_PASTEL_PURPLE                  : ColorRGB = lambda *_: ColorRGB(0xb19cd9, "light-pastel-purple",                   register=True)
    LIGHT_PEA_GREEN                      : ColorRGB = lambda *_: ColorRGB(0xc4fe82, "light-pea-green",                       register=True)
    LIGHT_PEACH                          : ColorRGB = lambda *_: ColorRGB(0xffd8b1, "light-peach",                           register=True)
    LIGHT_PERIWINKLE                     : ColorRGB = lambda *_: ColorRGB(0xc1c6fc, "light-periwinkle",                      register=True)
    LIGHT_PINK                           : ColorRGB = lambda *_: ColorRGB(0xffd1df, "light-pink",                            register=True)
    LIGHT_PLUM                           : ColorRGB = lambda *_: ColorRGB(0x9d5783, "light-plum",                            register=True)
    LIGHT_PURPLE                         : ColorRGB = lambda *_: ColorRGB(0xbf77f6, "light-purple",                          register=True)
    LIGHT_RED                            : ColorRGB = lambda *_: ColorRGB(0xff474c, "light-red",                             register=True)
    LIGHT_ROSE                           : ColorRGB = lambda *_: ColorRGB(0xffc5cb, "light-rose",                            register=True)
    LIGHT_ROYAL_BLUE                     : ColorRGB = lambda *_: ColorRGB(0x3a2efe, "light-royal-blue",                      register=True)
    LIGHT_SAGE                           : ColorRGB = lambda *_: ColorRGB(0xbcecac, "light-sage",                            register=True)
    LIGHT_SALMON                         : ColorRGB = lambda *_: ColorRGB(0xfea993, "light-salmon",                          register=True)
    LIGHT_SALMON_PINK                    : ColorRGB = lambda *_: ColorRGB(0xff9999, "light-salmon-pink",                     register=True)
    LIGHT_SEA_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x98f6b0, "light-sea-green",                       register=True)
    LIGHT_SEAFOAM                        : ColorRGB = lambda *_: ColorRGB(0xa0febf, "light-seafoam",                         register=True)
    LIGHT_SEAFOAM_GREEN                  : ColorRGB = lambda *_: ColorRGB(0xa7ffb5, "light-seafoam-green",                   register=True)
    LIGHT_SILVER                         : ColorRGB = lambda *_: ColorRGB(0xd8d8d8, "light-silver",                          register=True)
    LIGHT_SKY_BLUE                       : ColorRGB = lambda *_: ColorRGB(0xc6fcff, "light-sky-blue",                        register=True)
    LIGHT_SLATE_GRAY                     : ColorRGB = lambda *_: ColorRGB(0x778899, "light-slate-gray",                      register=True)
    LIGHT_STEEL_BLUE                     : ColorRGB = lambda *_: ColorRGB(0xb0c4de, "light-steel-blue",                      register=True)
    LIGHT_TAN                            : ColorRGB = lambda *_: ColorRGB(0xfbeeac, "light-tan",                             register=True)
    LIGHT_TAUPE                          : ColorRGB = lambda *_: ColorRGB(0xb38b6d, "light-taupe",                           register=True)
    LIGHT_TEAL                           : ColorRGB = lambda *_: ColorRGB(0x90e4c1, "light-teal",                            register=True)
    LIGHT_TURQUOISE                      : ColorRGB = lambda *_: ColorRGB(0x7ef4cc, "light-turquoise",                       register=True)
    LIGHT_URPLE                          : ColorRGB = lambda *_: ColorRGB(0xb36ff6, "light-urple",                           register=True)
    LIGHT_VIOLET                         : ColorRGB = lambda *_: ColorRGB(0xd6b4fc, "light-violet",                          register=True)
    LIGHT_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xfffe7a, "light-yellow",                          register=True, variation_map={
             0xf7bb7d:   "asagi" })
    LIGHT_YELLOW_GREEN                   : ColorRGB = lambda *_: ColorRGB(0xccfd7f, "light-yellow-green",                    register=True)
    LIGHT_YELLOWISH_GREEN                : ColorRGB = lambda *_: ColorRGB(0xc2ff89, "light-yellowish-green",                 register=True)
    LIGHTBLUE                            : ColorRGB = lambda *_: ColorRGB(0x7bc8f6, "lightblue",                             register=True)
    LIGHTER_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x75fd63, "lighter-green",                         register=True)
    LIGHTER_PURPLE                       : ColorRGB = lambda *_: ColorRGB(0xa55af4, "lighter-purple",                        register=True)
    LIGHTGREEN                           : ColorRGB = lambda *_: ColorRGB(0x76ff7b, "lightgreen",                            register=True)
    LIGHTISH_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x3d7afd, "lightish-blue",                         register=True)
    LIGHTISH_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x61e160, "lightish-green",                        register=True)
    LIGHTISH_PURPLE                      : ColorRGB = lambda *_: ColorRGB(0xa552e6, "lightish-purple",                       register=True)
    LIGHTISH_RED                         : ColorRGB = lambda *_: ColorRGB(0xfe2f4a, "lightish-red",                          register=True)
    LILAC                                : ColorRGB = lambda *_: ColorRGB(0xcea2fd, "lilac",                                 register=True)
    LILAC_LUSTER                         : ColorRGB = lambda *_: ColorRGB(0xae98aa, "lilac-luster",                          register=True)
    LILIAC                               : ColorRGB = lambda *_: ColorRGB(0xc48efd, "liliac",                                register=True)
    LIME                                 : ColorRGB = lambda *_: ColorRGB(0xaaff32, "lime",                                  register=True)
    LIME_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x89fe05, "lime-green",                            register=True)
    LIME_PULP                            : ColorRGB = lambda *_: ColorRGB(0xd1e189, "lime-pulp",                             register=True)
    LIME_YELLOW                          : ColorRGB = lambda *_: ColorRGB(0xd0fe1d, "lime-yellow",                           register=True)
    LIMERICK                             : ColorRGB = lambda *_: ColorRGB(0x9dc209, "limerick",                              register=True)
    LINCOLN_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x195905, "lincoln-green",                         register=True)
    LINEN                                : ColorRGB = lambda *_: ColorRGB(0xfaf0e6, "linen",                                 register=True)
    LINKEDIN_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x0072b1, "linkedin-blue",                         register=True)
    LIPSTICK                             : ColorRGB = lambda *_: ColorRGB(0xd5174e, "lipstick",                              register=True)
    LIPSTICK_RED                         : ColorRGB = lambda *_: ColorRGB(0xc0022f, "lipstick-red",                          register=True)
    LITTLE_BOY_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x6ca0dc, "little-boy-blue",                       register=True)
    LITTLE_GIRL_PINK                     : ColorRGB = lambda *_: ColorRGB(0xf8b9d4, "little-girl-pink",                      register=True)
    LIVER                                : ColorRGB = lambda *_: ColorRGB(0x674c47, "liver",                                 register=True, variation_map={
             0xb86d29:   "dogs", 
             0x6c2e1f:   "organ" })
    LIVER_CHESTNUT                       : ColorRGB = lambda *_: ColorRGB(0x987456, "liver-chestnut",                        register=True)
    LONG_SPRING                          : ColorRGB = lambda *_: ColorRGB(0xb95754, "long-spring",                           register=True)
    LOQUAT                               : ColorRGB = lambda *_: ColorRGB(0xab6134, "loquat",                                register=True)
    LOTION                               : ColorRGB = lambda *_: ColorRGB(0xfefdfa, "lotion",                                register=True)
    LOTION_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x15f2fd, "lotion-blue",                           register=True)
    LOTION_PINK                          : ColorRGB = lambda *_: ColorRGB(0xeccfcf, "lotion-pink",                           register=True)
    LUMBER                               : ColorRGB = lambda *_: ColorRGB(0xffe4cd, "lumber",                                register=True)
    LUST                                 : ColorRGB = lambda *_: ColorRGB(0xe62020, "lust",                                  register=True)
    LYCHEE                               : ColorRGB = lambda *_: ColorRGB(0xdc5349, "lychee",                                register=True)
    LYE                                  : ColorRGB = lambda *_: ColorRGB(0x7f6b5d, "lye",                                   register=True)
    MACARONI_AND_CHEESE                  : ColorRGB = lambda *_: ColorRGB(0xefb435, "macaroni-and-cheese",                   register=True)
    MACEDONIAN_ORANGE                    : ColorRGB = lambda *_: ColorRGB(0xe97f28, "macedonian-orange",                     register=True)
    MADDER_LAKE                          : ColorRGB = lambda *_: ColorRGB(0xcc3336, "madder-lake",                           register=True)
    MAGENTA                              : ColorRGB = lambda *_: ColorRGB(0xc20078, "magenta",                               register=True, variation_map={
             0xca1f7b:   "dye", 
             0xd0417e:   "pantone", 
             0xff0090:   "process" })
    MAGENTA_HAZE                         : ColorRGB = lambda *_: ColorRGB(0x9f4576, "magenta-haze",                          register=True)
    MAGENTA_KALM                         : ColorRGB = lambda *_: ColorRGB(0x85678f, "magenta-kalm",                          register=True)
    MAGENTA_PANIK                        : ColorRGB = lambda *_: ColorRGB(0xff69c1, "magenta-panik",                         register=True)
    MAGENTA_PINK                         : ColorRGB = lambda *_: ColorRGB(0xcc338b, "magenta-pink",                          register=True)
    MAGIC_MINT                           : ColorRGB = lambda *_: ColorRGB(0xaaf0d1, "magic-mint",                            register=True)
    MAGIC_POTION                         : ColorRGB = lambda *_: ColorRGB(0xff4466, "magic-potion",                          register=True)
    MAGNOLIA                             : ColorRGB = lambda *_: ColorRGB(0xf8f4ff, "magnolia",                              register=True)
    MAHOGANY                             : ColorRGB = lambda *_: ColorRGB(0x4a0100, "mahogany",                              register=True)
    MAIZE                                : ColorRGB = lambda *_: ColorRGB(0xf4d054, "maize",                                 register=True, variation_map={
             0xf2c649:   "crayola" })
    MAJORELLE_BLUE                       : ColorRGB = lambda *_: ColorRGB(0x6050dc, "majorelle-blue",                        register=True)
    MALACHITE                            : ColorRGB = lambda *_: ColorRGB(0x0bda51, "malachite",                             register=True)
    MANATEE                              : ColorRGB = lambda *_: ColorRGB(0x979aaa, "manatee",                               register=True)
    MANDARIN                             : ColorRGB = lambda *_: ColorRGB(0xf37a48, "mandarin",                              register=True)
    MANGO                                : ColorRGB = lambda *_: ColorRGB(0xffa62b, "mango",                                 register=True)
    MANGO_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x96ff00, "mango-green",                           register=True)
    MANGO_TANGO                          : ColorRGB = lambda *_: ColorRGB(0xff8243, "mango-tango",                           register=True)
    MANILA                               : ColorRGB = lambda *_: ColorRGB(0xe7c9a9, "manila",                                register=True)
    MANILLA                              : ColorRGB = lambda *_: ColorRGB(0xfffa86, "manilla",                               register=True)
    MANTIS                               : ColorRGB = lambda *_: ColorRGB(0x74c365, "mantis",                                register=True)
    MAPLE_SYRUP                          : ColorRGB = lambda *_: ColorRGB(0xbb9351, "maple-syrup",                           register=True)
    MARDI_GRAS                           : ColorRGB = lambda *_: ColorRGB(0x880085, "mardi-gras",                            register=True)
    MARGARINE                            : ColorRGB = lambda *_: ColorRGB(0xf2d930, "margarine",                             register=True)
    MARGARITA                            : ColorRGB = lambda *_: ColorRGB(0xb0c24a, "margarita",                             register=True)
    MARIGOLD                             : ColorRGB = lambda *_: ColorRGB(0xfcc006, "marigold",                              register=True)
    MARINE                               : ColorRGB = lambda *_: ColorRGB(0x042e60, "marine",                                register=True)
    MARINE_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x01386a, "marine-blue",                           register=True)
    MAROON                               : ColorRGB = lambda *_: ColorRGB(0x650021, "maroon",                                register=True, variation_map={
             0x5e2824:   "shrimp-brown", 
             0xb03060:   "x11" })
    MAUVE                                : ColorRGB = lambda *_: ColorRGB(0xae7181, "mauve",                                 register=True)
    MAUVE_TAUPE                          : ColorRGB = lambda *_: ColorRGB(0x915f6d, "mauve-taupe",                           register=True)
    MAUVELOUS                            : ColorRGB = lambda *_: ColorRGB(0xef98aa, "mauvelous",                             register=True)
    MAXIMUM_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x47abcc, "maximum-blue",                          register=True)
    MAXIMUM_BLUE_GREEN                   : ColorRGB = lambda *_: ColorRGB(0x30bfbf, "maximum-blue-green",                    register=True)
    MAXIMUM_BLUE_PURPLE                  : ColorRGB = lambda *_: ColorRGB(0xacace6, "maximum-blue-purple",                   register=True)
    MAXIMUM_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x5e8c31, "maximum-green",                         register=True)
    MAXIMUM_GREEN_YELLOW                 : ColorRGB = lambda *_: ColorRGB(0xd9e650, "maximum-green-yellow",                  register=True)
    MAXIMUM_ORANGE                       : ColorRGB = lambda *_: ColorRGB(0xff5b00, "maximum-orange",                        register=True)
    MAXIMUM_PURPLE                       : ColorRGB = lambda *_: ColorRGB(0x733380, "maximum-purple",                        register=True)
    MAXIMUM_RED                          : ColorRGB = lambda *_: ColorRGB(0xd92121, "maximum-red",                           register=True)
    MAXIMUM_RED_PURPLE                   : ColorRGB = lambda *_: ColorRGB(0xa63a79, "maximum-red-purple",                    register=True)
    MAXIMUM_YELLOW                       : ColorRGB = lambda *_: ColorRGB(0xfafa37, "maximum-yellow",                        register=True)
    MAXIMUM_YELLOW_RED                   : ColorRGB = lambda *_: ColorRGB(0xf2ba49, "maximum-yellow-red",                    register=True)
    MAY_GREEN                            : ColorRGB = lambda *_: ColorRGB(0x4c9141, "may-green",                             register=True)
    MAYA_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x73c2fb, "maya-blue",                             register=True)
    MC_FANNING                           : ColorRGB = lambda *_: ColorRGB(0x42526e, "mc-fanning",                            register=True)
    MEAT                                 : ColorRGB = lambda *_: ColorRGB(0xf9906f, "meat",                                  register=True)
    MEAT_BROWN                           : ColorRGB = lambda *_: ColorRGB(0xe5b73b, "meat-brown",                            register=True)
    MEDIUM_AQUAMARINE                    : ColorRGB = lambda *_: ColorRGB(0x66ddaa, "medium-aquamarine",                     register=True)
    MEDIUM_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x2c6fbb, "medium-blue",                           register=True)
    MEDIUM_BROWN                         : ColorRGB = lambda *_: ColorRGB(0x7f5112, "medium-brown",                          register=True)
    MEDIUM_CANDY_APPLE_RED               : ColorRGB = lambda *_: ColorRGB(0xe2062c, "medium-candy-apple-red",                register=True)
    MEDIUM_CARMINE                       : ColorRGB = lambda *_: ColorRGB(0xaf4035, "medium-carmine",                        register=True)
    MEDIUM_CHAMPAGNE                     : ColorRGB = lambda *_: ColorRGB(0xf3e5ab, "medium-champagne",                      register=True)
    MEDIUM_CRIMSON                       : ColorRGB = lambda *_: ColorRGB(0xc93756, "medium-crimson",                        register=True)
    MEDIUM_ELECTRIC_BLUE                 : ColorRGB = lambda *_: ColorRGB(0x035096, "medium-electric-blue",                  register=True)
    MEDIUM_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x39ad48, "medium-green",                          register=True)
    MEDIUM_GREY                          : ColorRGB = lambda *_: ColorRGB(0x7d7f7c, "medium-grey",                           register=True)
    MEDIUM_JUNGLE_GREEN                  : ColorRGB = lambda *_: ColorRGB(0x1c352d, "medium-jungle-green",                   register=True)
    MEDIUM_LAVENDER_MAGENTA              : ColorRGB = lambda *_: ColorRGB(0xdda0dd, "medium-lavender-magenta",               register=True)
    MEDIUM_ORCHID                        : ColorRGB = lambda *_: ColorRGB(0xba55d3, "medium-orchid",                         register=True)
    MEDIUM_PERSIAN_BLUE                  : ColorRGB = lambda *_: ColorRGB(0x0067a5, "medium-persian-blue",                   register=True)
    MEDIUM_PINK                          : ColorRGB = lambda *_: ColorRGB(0xf36196, "medium-pink",                           register=True)
    MEDIUM_PURPLE                        : ColorRGB = lambda *_: ColorRGB(0x9e43a2, "medium-purple",                         register=True)
    MEDIUM_RED_VIOLET                    : ColorRGB = lambda *_: ColorRGB(0xbb3385, "medium-red-violet",                     register=True)
    MEDIUM_RUBY                          : ColorRGB = lambda *_: ColorRGB(0xaa4069, "medium-ruby",                           register=True)
    MEDIUM_SEA_GREEN                     : ColorRGB = lambda *_: ColorRGB(0x3cb371, "medium-sea-green",                      register=True)
    MEDIUM_SKY_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x80daeb, "medium-sky-blue",                       register=True)
    MEDIUM_SLATE_BLUE                    : ColorRGB = lambda *_: ColorRGB(0x7b68ee, "medium-slate-blue",                     register=True)
    MEDIUM_SPRING_BUD                    : ColorRGB = lambda *_: ColorRGB(0xc9dc87, "medium-spring-bud",                     register=True)
    MEDIUM_SPRING_GREEN                  : ColorRGB = lambda *_: ColorRGB(0x00fa9a, "medium-spring-green",                   register=True)
    MEDIUM_TURQUOISE                     : ColorRGB = lambda *_: ColorRGB(0x48d1cc, "medium-turquoise",                      register=True)
    MEDIUM_VERMILION                     : ColorRGB = lambda *_: ColorRGB(0xd9603b, "medium-vermilion",                      register=True)
    MEDIUM_VIOLET_RED                    : ColorRGB = lambda *_: ColorRGB(0xc71585, "medium-violet-red",                     register=True)
    MELANCHOLY                           : ColorRGB = lambda *_: ColorRGB(0xfdbcb4, "melancholy",                            register=True)
    MELLOW_APRICOT                       : ColorRGB = lambda *_: ColorRGB(0xf8b878, "mellow-apricot",                        register=True)
    MELON                                : ColorRGB = lambda *_: ColorRGB(0xff7855, "melon",                                 register=True)
    MENTHOL                              : ColorRGB = lambda *_: ColorRGB(0xc1f9a2, "menthol",                               register=True)
    MEREDITH                             : ColorRGB = lambda *_: ColorRGB(0x8993a4, "meredith",                              register=True)
    MERLOT                               : ColorRGB = lambda *_: ColorRGB(0x730039, "merlot",                                register=True)
    MERMAID_NET                          : ColorRGB = lambda *_: ColorRGB(0x00c7e6, "mermaid-net",                           register=True)
    METALLIC_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x4f738e, "metallic-blue",                         register=True)
    METALLIC_BRONZE                      : ColorRGB = lambda *_: ColorRGB(0xa97142, "metallic-bronze",                       register=True)
    METALLIC_BROWN                       : ColorRGB = lambda *_: ColorRGB(0xac4313, "metallic-brown",                        register=True)
    METALLIC_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x296e01, "metallic-green",                        register=True)
    METALLIC_ORANGE                      : ColorRGB = lambda *_: ColorRGB(0xda680f, "metallic-orange",                       register=True)
    METALLIC_PINK                        : ColorRGB = lambda *_: ColorRGB(0xeda6c4, "metallic-pink",                         register=True)
    METALLIC_RED                         : ColorRGB = lambda *_: ColorRGB(0xa62c2b, "metallic-red",                          register=True)
    METALLIC_SEAWEED                     : ColorRGB = lambda *_: ColorRGB(0x0a7e8c, "metallic-seaweed",                      register=True)
    METALLIC_SUNBURST                    : ColorRGB = lambda *_: ColorRGB(0x9c7c38, "metallic-sunburst",                     register=True)
    METALLIC_VIOLET                      : ColorRGB = lambda *_: ColorRGB(0x5b0a91, "metallic-violet",                       register=True)
    METALLIC_YELLOW                      : ColorRGB = lambda *_: ColorRGB(0xfdcc0d, "metallic-yellow",                       register=True)
    MEXICAN_PINK                         : ColorRGB = lambda *_: ColorRGB(0xe4007c, "mexican-pink",                          register=True)
    MICROSOFT_BLUE                       : ColorRGB = lambda *_: ColorRGB(0x00a2ed, "microsoft-blue",                        register=True)
    MICROSOFT_EDGE_BLUE                  : ColorRGB = lambda *_: ColorRGB(0x0078d7, "microsoft-edge-blue",                   register=True)
    MICROSOFT_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x7db700, "microsoft-green",                       register=True)
    MICROSOFT_RED                        : ColorRGB = lambda *_: ColorRGB(0xf04e1f, "microsoft-red",                         register=True)
    MICROSOFT_YELLOW                     : ColorRGB = lambda *_: ColorRGB(0xfdb900, "microsoft-yellow",                      register=True)
    MID_BLUE                             : ColorRGB = lambda *_: ColorRGB(0x276ab3, "mid-blue",                              register=True)
    MID_GREEN                            : ColorRGB = lambda *_: ColorRGB(0x50a747, "mid-green",                             register=True)
    MIDDLE_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x7ed4e6, "middle-blue",                           register=True)
    MIDDLE_BLUE_GREEN                    : ColorRGB = lambda *_: ColorRGB(0x8dd9cc, "middle-blue-green",                     register=True)
    MIDDLE_BLUE_PURPLE                   : ColorRGB = lambda *_: ColorRGB(0x8b72be, "middle-blue-purple",                    register=True)
    MIDDLE_GRAY                          : ColorRGB = lambda *_: ColorRGB(0x8b8680, "middle-gray",                           register=True)
    MIDDLE_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x4d8c57, "middle-green",                          register=True)
    MIDDLE_GREEN_YELLOW                  : ColorRGB = lambda *_: ColorRGB(0xacbf60, "middle-green-yellow",                   register=True)
    MIDDLE_PURPLE                        : ColorRGB = lambda *_: ColorRGB(0xd982b5, "middle-purple",                         register=True)
    MIDDLE_RED                           : ColorRGB = lambda *_: ColorRGB(0xe58e73, "middle-red",                            register=True)
    MIDDLE_RED_PURPLE                    : ColorRGB = lambda *_: ColorRGB(0xa55353, "middle-red-purple",                     register=True)
    MIDDLE_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xffeb00, "middle-yellow",                         register=True)
    MIDDLE_YELLOW_RED                    : ColorRGB = lambda *_: ColorRGB(0xecb176, "middle-yellow-red",                     register=True)
    MIDNIGHT                             : ColorRGB = lambda *_: ColorRGB(0x03012d, "midnight",                              register=True)
    MIDNIGHT_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x020035, "midnight-blue",                         register=True, variation_map={
             0x00468c:   "2" })
    MIDNIGHT_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x004953, "midnight-green",                        register=True)
    MIDNIGHT_PURPLE                      : ColorRGB = lambda *_: ColorRGB(0x280137, "midnight-purple",                       register=True)
    MIKADO_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xffc40c, "mikado-yellow",                         register=True)
    MILITARY_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x667c3e, "military-green",                        register=True)
    MILK                                 : ColorRGB = lambda *_: ColorRGB(0xfdfff5, "milk",                                  register=True)
    MILK_CHOCOLATE                       : ColorRGB = lambda *_: ColorRGB(0x7f4e1e, "milk-chocolate",                        register=True)
    MIMI_PINK                            : ColorRGB = lambda *_: ColorRGB(0xffdae9, "mimi-pink",                             register=True)
    MINDARO                              : ColorRGB = lambda *_: ColorRGB(0xe3f988, "mindaro",                               register=True)
    MING                                 : ColorRGB = lambda *_: ColorRGB(0x36747d, "ming",                                  register=True)
    MINION_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xf5e050, "minion-yellow",                         register=True)
    MINT                                 : ColorRGB = lambda *_: ColorRGB(0x9ffeb0, "mint",                                  register=True)
    MINT_CREAM                           : ColorRGB = lambda *_: ColorRGB(0xf5fffa, "mint-cream",                            register=True)
    MINT_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x8fff9f, "mint-green",                            register=True)
    MINTIE                               : ColorRGB = lambda *_: ColorRGB(0xabf5d1, "mintie",                                register=True)
    MINTY_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x0bf77d, "minty-green",                           register=True)
    MISTY_MOSS                           : ColorRGB = lambda *_: ColorRGB(0xbbb477, "misty-moss",                            register=True)
    MISTY_ROSE                           : ColorRGB = lambda *_: ColorRGB(0xffe4e1, "misty-rose",                            register=True)
    MOCHA                                : ColorRGB = lambda *_: ColorRGB(0x9d7651, "mocha",                                 register=True)
    MOONLIGHTD                           : ColorRGB = lambda *_: ColorRGB(0x7280e2, "moonlightd",                            register=True)
    MOONSTONE                            : ColorRGB = lambda *_: ColorRGB(0x3aa8c1, "moonstone",                             register=True)
    MOONSTONE_BLUE                       : ColorRGB = lambda *_: ColorRGB(0x73a9c2, "moonstone-blue",                        register=True)
    MORDANT_RED_19                       : ColorRGB = lambda *_: ColorRGB(0xae0c00, "mordant-red-19",                        register=True)
    MORNING_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x8da399, "morning-blue",                          register=True)
    MOSS                                 : ColorRGB = lambda *_: ColorRGB(0x769958, "moss",                                  register=True, variation_map={
             0x8b7d3a:   "koke-iro" })
    MOSS_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x658b38, "moss-green",                            register=True)
    MOSSY_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x638b27, "mossy-green",                           register=True)
    MOUNTAIN_MEADOW                      : ColorRGB = lambda *_: ColorRGB(0x30ba8f, "mountain-meadow",                       register=True)
    MOUNTBATTEN_PINK                     : ColorRGB = lambda *_: ColorRGB(0x997a8d, "mountbatten-pink",                      register=True)
    MOUSY_INDIGO                         : ColorRGB = lambda *_: ColorRGB(0x5c544e, "mousy-indigo",                          register=True)
    MOUSY_WISTERIA                       : ColorRGB = lambda *_: ColorRGB(0x766980, "mousy-wisteria",                        register=True)
    MSU_GREEN                            : ColorRGB = lambda *_: ColorRGB(0x18453b, "msu-green",                             register=True)
    MUD                                  : ColorRGB = lambda *_: ColorRGB(0x735c12, "mud",                                   register=True)
    MUD_BROWN                            : ColorRGB = lambda *_: ColorRGB(0x60460f, "mud-brown",                             register=True)
    MUD_GREEN                            : ColorRGB = lambda *_: ColorRGB(0x606602, "mud-green",                             register=True)
    MUDDY_BROWN                          : ColorRGB = lambda *_: ColorRGB(0x886806, "muddy-brown",                           register=True, variation_map={
             0xcb6649:   "lit-the-brown-of-a-distant-river" })
    MUDDY_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x657432, "muddy-green",                           register=True)
    MUDDY_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xbfac05, "muddy-yellow",                          register=True)
    MUGHAL_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x306030, "mughal-green",                          register=True)
    MULBERRY                             : ColorRGB = lambda *_: ColorRGB(0x920a4e, "mulberry",                              register=True, variation_map={
             0xc8509b:   "crayola" })
    MULBERRY_DYE                         : ColorRGB = lambda *_: ColorRGB(0x59292c, "mulberry-dye",                          register=True)
    MULBERRY_DYED                        : ColorRGB = lambda *_: ColorRGB(0xc57f2e, "mulberry-dyed",                         register=True)
    MURKY_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x6c7a0e, "murky-green",                           register=True)
    MUSHROOM                             : ColorRGB = lambda *_: ColorRGB(0xba9e88, "mushroom",                              register=True)
    MUSTARD                              : ColorRGB = lambda *_: ColorRGB(0xceb301, "mustard",                               register=True)
    MUSTARD_BROWN                        : ColorRGB = lambda *_: ColorRGB(0xac7e04, "mustard-brown",                         register=True)
    MUSTARD_GREEN                        : ColorRGB = lambda *_: ColorRGB(0xa8b504, "mustard-green",                         register=True)
    MUSTARD_YELLOW                       : ColorRGB = lambda *_: ColorRGB(0xd2bd0a, "mustard-yellow",                        register=True)
    MUTED_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x3b719f, "muted-blue",                            register=True)
    MUTED_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x5fa052, "muted-green",                           register=True)
    MUTED_PINK                           : ColorRGB = lambda *_: ColorRGB(0xd1768f, "muted-pink",                            register=True)
    MUTED_PURPLE                         : ColorRGB = lambda *_: ColorRGB(0x805b87, "muted-purple",                          register=True)
    MYRTLE_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x317873, "myrtle-green",                          register=True)
    MYSTIC                               : ColorRGB = lambda *_: ColorRGB(0xd65282, "mystic",                                register=True)
    MYSTIC_MAROON                        : ColorRGB = lambda *_: ColorRGB(0xad4379, "mystic-maroon",                         register=True)
    MYSTIC_RED                           : ColorRGB = lambda *_: ColorRGB(0xff5500, "mystic-red",                            register=True)
    NADESHIKO_PINK                       : ColorRGB = lambda *_: ColorRGB(0xf6adc6, "nadeshiko-pink",                        register=True)
    NAPIER_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x2a8000, "napier-green",                          register=True)
    NAPLES_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xfada5e, "naples-yellow",                         register=True)
    NASTY_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x70b23f, "nasty-green",                           register=True)
    NAVAJO_WHITE                         : ColorRGB = lambda *_: ColorRGB(0xffdead, "navajo-white",                          register=True)
    NAVY                                 : ColorRGB = lambda *_: ColorRGB(0x01153e, "navy",                                  register=True)
    NAVY_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x001146, "navy-blue",                             register=True)
    NAVY_BLUE_BELLFLOWER                 : ColorRGB = lambda *_: ColorRGB(0x191f45, "navy-blue-bellflower",                  register=True)
    NAVY_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x35530a, "navy-green",                            register=True)
    NEON_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x04d9ff, "neon-blue",                             register=True)
    NEON_CARROT                          : ColorRGB = lambda *_: ColorRGB(0xffa343, "neon-carrot",                           register=True)
    NEON_FUCHSIA                         : ColorRGB = lambda *_: ColorRGB(0xfe4164, "neon-fuchsia",                          register=True)
    NEON_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x0cff0c, "neon-green",                            register=True)
    NEON_PINK                            : ColorRGB = lambda *_: ColorRGB(0xfe019a, "neon-pink",                             register=True)
    NEON_PURPLE                          : ColorRGB = lambda *_: ColorRGB(0xbc13fe, "neon-purple",                           register=True)
    NEON_RED                             : ColorRGB = lambda *_: ColorRGB(0xff073a, "neon-red",                              register=True)
    NEON_YELLOW                          : ColorRGB = lambda *_: ColorRGB(0xcfff04, "neon-yellow",                           register=True)
    NEW_BRIDGE                           : ColorRGB = lambda *_: ColorRGB(0x006c7f, "new-bridge",                            register=True)
    NEW_CAR                              : ColorRGB = lambda *_: ColorRGB(0x214fc6, "new-car",                               register=True)
    NEW_YORK_PINK                        : ColorRGB = lambda *_: ColorRGB(0xd7837f, "new-york-pink",                         register=True)
    NICE_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x107ab0, "nice-blue",                             register=True)
    NICKEL                               : ColorRGB = lambda *_: ColorRGB(0x727472, "nickel",                                register=True)
    NIGHT_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x040348, "night-blue",                            register=True)
    NIGHTINGALE_BROWN                    : ColorRGB = lambda *_: ColorRGB(0x5c4827, "nightingale-brown",                     register=True)
    NINTENDO_RED                         : ColorRGB = lambda *_: ColorRGB(0xe4000f, "nintendo-red",                          register=True)
    NON_PHOTO_BLUE                       : ColorRGB = lambda *_: ColorRGB(0xa4dded, "non-photo-blue",                        register=True)
    NORTH_TEXAS_GREEN                    : ColorRGB = lambda *_: ColorRGB(0x059033, "north-texas-green",                     register=True)
    NORTHEAST_SNOW                       : ColorRGB = lambda *_: ColorRGB(0xebecf0, "northeast-snow",                        register=True)
    NYANZA                               : ColorRGB = lambda *_: ColorRGB(0xe9ffdb, "nyanza",                                register=True)
    OCEAN                                : ColorRGB = lambda *_: ColorRGB(0x017b92, "ocean",                                 register=True)
    OCEAN_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x03719c, "ocean-blue",                            register=True)
    OCEAN_BOAT_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x0077be, "ocean-boat-blue",                       register=True)
    OCEAN_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x3d9973, "ocean-green",                           register=True)
    OCHER                                : ColorRGB = lambda *_: ColorRGB(0xbf9b0c, "ocher",                                 register=True)
    OCHRE                                : ColorRGB = lambda *_: ColorRGB(0xbf9005, "ochre",                                 register=True, variation_map={
             0xff4e20:   "earthen-yellow-red-brown", 
             0xbe7f51:   "lit-earthen-yellow" })
    OCRE                                 : ColorRGB = lambda *_: ColorRGB(0xc69c04, "ocre",                                  register=True)
    OFF_BLUE                             : ColorRGB = lambda *_: ColorRGB(0x5684ae, "off-blue",                              register=True)
    OFF_GREEN                            : ColorRGB = lambda *_: ColorRGB(0x6ba353, "off-green",                             register=True)
    OFF_WHITE                            : ColorRGB = lambda *_: ColorRGB(0xffffe4, "off-white",                             register=True)
    OFF_YELLOW                           : ColorRGB = lambda *_: ColorRGB(0xf1f33f, "off-yellow",                            register=True)
    OGRE_ODOR                            : ColorRGB = lambda *_: ColorRGB(0xfd5240, "ogre-odor",                             register=True)
    OLD_BAMBOO                           : ColorRGB = lambda *_: ColorRGB(0x5e644f, "old-bamboo",                            register=True)
    OLD_BURGUNDY                         : ColorRGB = lambda *_: ColorRGB(0x43302e, "old-burgundy",                          register=True)
    OLD_GOLD                             : ColorRGB = lambda *_: ColorRGB(0xcfb53b, "old-gold",                              register=True)
    OLD_LACE                             : ColorRGB = lambda *_: ColorRGB(0xfdf5e6, "old-lace",                              register=True)
    OLD_LAVENDER                         : ColorRGB = lambda *_: ColorRGB(0x796878, "old-lavender",                          register=True)
    OLD_MAUVE                            : ColorRGB = lambda *_: ColorRGB(0x673147, "old-mauve",                             register=True)
    OLD_MOSS_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x867e36, "old-moss-green",                        register=True)
    OLD_PINK                             : ColorRGB = lambda *_: ColorRGB(0xc77986, "old-pink",                              register=True)
    OLD_ROSE                             : ColorRGB = lambda *_: ColorRGB(0xc87f89, "old-rose",                              register=True)
    OLIVE                                : ColorRGB = lambda *_: ColorRGB(0x6e750e, "olive",                                 register=True)
    OLIVE_BROWN                          : ColorRGB = lambda *_: ColorRGB(0x645403, "olive-brown",                           register=True)
    OLIVE_DRAB                           : ColorRGB = lambda *_: ColorRGB(0x6f7632, "olive-drab",                            register=True, variation_map={
             0x6b8e23:   "3", 
             0x3c341f:   "7" })
    OLIVE_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x677a04, "olive-green",                           register=True)
    OLIVE_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xc2b709, "olive-yellow",                          register=True)
    OLIVINE                              : ColorRGB = lambda *_: ColorRGB(0x9ab973, "olivine",                               register=True)
    OLYMPIC_BRONZE                       : ColorRGB = lambda *_: ColorRGB(0x4e3e37, "olympic-bronze",                        register=True)
    ONANDO                               : ColorRGB = lambda *_: ColorRGB(0x364141, "onando",                                register=True)
    ONE_KIN                              : ColorRGB = lambda *_: ColorRGB(0xf08f90, "one-kin",                               register=True)
    ONYX                                 : ColorRGB = lambda *_: ColorRGB(0x353839, "onyx",                                  register=True)
    OPAL                                 : ColorRGB = lambda *_: ColorRGB(0xa8c3bc, "opal",                                  register=True)
    OPERA_MAUVE                          : ColorRGB = lambda *_: ColorRGB(0xb784a7, "opera-mauve",                           register=True)
    OPERA_RED                            : ColorRGB = lambda *_: ColorRGB(0xff1b2d, "opera-red",                             register=True)
    OPPOSITE_FLOWER                      : ColorRGB = lambda *_: ColorRGB(0x4d646c, "opposite-flower",                       register=True)
    ORANGE                               : ColorRGB = lambda *_: ColorRGB(0xf97306, "orange",                                register=True, variation_map={
             0xff7f00:   "color-wheel", 
             0xff7538:   "crayola", 
             0xff5800:   "pantone", 
             0xfb9902:   "ryb" })
    ORANGE_BROWN                         : ColorRGB = lambda *_: ColorRGB(0xbe6400, "orange-brown",                          register=True)
    ORANGE_PEEL                          : ColorRGB = lambda *_: ColorRGB(0xff9f00, "orange-peel",                           register=True)
    ORANGE_PINK                          : ColorRGB = lambda *_: ColorRGB(0xff6f52, "orange-pink",                           register=True)
    ORANGE_RED                           : ColorRGB = lambda *_: ColorRGB(0xfd411e, "orange-red",                            register=True, variation_map={
             0xff5349:   "crayola" })
    ORANGE_SODA                          : ColorRGB = lambda *_: ColorRGB(0xfa5b3d, "orange-soda",                           register=True, variation_map={
             0xeb593d:   "2" })
    ORANGE_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xffad01, "orange-yellow",                         register=True, variation_map={
             0xf8d568:   "crayola" })
    ORANGEISH                            : ColorRGB = lambda *_: ColorRGB(0xfd8d49, "orangeish",                             register=True)
    ORANGERED                            : ColorRGB = lambda *_: ColorRGB(0xfe420f, "orangered",                             register=True)
    ORANGEY_BROWN                        : ColorRGB = lambda *_: ColorRGB(0xb16002, "orangey-brown",                         register=True)
    ORANGEY_RED                          : ColorRGB = lambda *_: ColorRGB(0xfa4224, "orangey-red",                           register=True)
    ORANGEY_YELLOW                       : ColorRGB = lambda *_: ColorRGB(0xfdb915, "orangey-yellow",                        register=True)
    ORANGISH                             : ColorRGB = lambda *_: ColorRGB(0xfc824a, "orangish",                              register=True)
    ORANGISH_BROWN                       : ColorRGB = lambda *_: ColorRGB(0xb25f03, "orangish-brown",                        register=True)
    ORANGISH_RED                         : ColorRGB = lambda *_: ColorRGB(0xf43605, "orangish-red",                          register=True)
    ORCHID                               : ColorRGB = lambda *_: ColorRGB(0xc875c4, "orchid",                                register=True, variation_map={
             0xe29cd2:   "crayola" })
    ORCHID_PINK                          : ColorRGB = lambda *_: ColorRGB(0xf2bdcd, "orchid-pink",                           register=True)
    ORIOLES_ORANGE                       : ColorRGB = lambda *_: ColorRGB(0xfb4f14, "orioles-orange",                        register=True)
    OU_CRIMSON_RED                       : ColorRGB = lambda *_: ColorRGB(0x841617, "ou-crimson-red",                        register=True)
    OUTER_SPACE                          : ColorRGB = lambda *_: ColorRGB(0x414a4c, "outer-space",                           register=True, variation_map={
             0x2d383a:   "crayola" })
    OUTRAGEOUS_ORANGE                    : ColorRGB = lambda *_: ColorRGB(0xff6e4a, "outrageous-orange",                     register=True)
    OVERDYED_REFRESHED_RED_BROWN         : ColorRGB = lambda *_: ColorRGB(0xe35c38, "overdyed-refreshed-red-brown",          register=True)
    OXFORD_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x002147, "oxford-blue",                           register=True)
    OXLEY                                : ColorRGB = lambda *_: ColorRGB(0x6d9a79, "oxley",                                 register=True)
    PACIFIC_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x1ca9c9, "pacific-blue",                          register=True)
    PACIFIC_BRIDGE                       : ColorRGB = lambda *_: ColorRGB(0x0052cc, "pacific-bridge",                        register=True, aliases=['jira-blue'])
    PAINITE                              : ColorRGB = lambda *_: ColorRGB(0x6b4947, "painite",                               register=True)
    PAKISTAN_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x006600, "pakistan-green",                        register=True)
    PALATINATE_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x273be2, "palatinate-blue",                       register=True)
    PALATINATE_PURPLE                    : ColorRGB = lambda *_: ColorRGB(0x682860, "palatinate-purple",                     register=True)
    PALE                                 : ColorRGB = lambda *_: ColorRGB(0xfff9d0, "pale",                                  register=True)
    PALE_AQUA                            : ColorRGB = lambda *_: ColorRGB(0xb8ffeb, "pale-aqua",                             register=True)
    PALE_BLUE                            : ColorRGB = lambda *_: ColorRGB(0xd0fefe, "pale-blue",                             register=True, variation_map={
             0x8c9c76:   "usuao" })
    PALE_BLUE_PALE_TURQUOISE             : ColorRGB = lambda *_: ColorRGB(0xafeeee, "pale-blue-pale-turquoise",              register=True)
    PALE_BROWN                           : ColorRGB = lambda *_: ColorRGB(0xb1916e, "pale-brown",                            register=True)
    PALE_CERULEAN                        : ColorRGB = lambda *_: ColorRGB(0x9bc4e2, "pale-cerulean",                         register=True)
    PALE_CHESTNUT                        : ColorRGB = lambda *_: ColorRGB(0xddadaf, "pale-chestnut",                         register=True)
    PALE_CORNFLOWER_BLUE                 : ColorRGB = lambda *_: ColorRGB(0xabcdef, "pale-cornflower-blue",                  register=True)
    PALE_CYAN                            : ColorRGB = lambda *_: ColorRGB(0xb7fffa, "pale-cyan",                             register=True)
    PALE_FALLEN_LEAVES                   : ColorRGB = lambda *_: ColorRGB(0xaa8736, "pale-fallen-leaves",                    register=True)
    PALE_GOLD                            : ColorRGB = lambda *_: ColorRGB(0xfdde6c, "pale-gold",                             register=True)
    PALE_GOLDENROD                       : ColorRGB = lambda *_: ColorRGB(0xeee8aa, "pale-goldenrod",                        register=True)
    PALE_GREEN                           : ColorRGB = lambda *_: ColorRGB(0xc7fdb5, "pale-green",                            register=True)
    PALE_GREEN_ONION                     : ColorRGB = lambda *_: ColorRGB(0x749f8d, "pale-green-onion",                      register=True)
    PALE_GREY                            : ColorRGB = lambda *_: ColorRGB(0xfdfdfe, "pale-grey",                             register=True)
    PALE_INCENSE                         : ColorRGB = lambda *_: ColorRGB(0xffa565, "pale-incense",                          register=True)
    PALE_LAVENDER                        : ColorRGB = lambda *_: ColorRGB(0xeecffe, "pale-lavender",                         register=True)
    PALE_LIGHT_GREEN                     : ColorRGB = lambda *_: ColorRGB(0xb1fc99, "pale-light-green",                      register=True)
    PALE_LILAC                           : ColorRGB = lambda *_: ColorRGB(0xe4cbff, "pale-lilac",                            register=True)
    PALE_LIME                            : ColorRGB = lambda *_: ColorRGB(0xbefd73, "pale-lime",                             register=True)
    PALE_LIME_GREEN                      : ColorRGB = lambda *_: ColorRGB(0xb1ff65, "pale-lime-green",                       register=True)
    PALE_MAGENTA                         : ColorRGB = lambda *_: ColorRGB(0xd767ad, "pale-magenta",                          register=True)
    PALE_MAGENTA_PINK                    : ColorRGB = lambda *_: ColorRGB(0xff99cc, "pale-magenta-pink",                     register=True)
    PALE_MAUVE                           : ColorRGB = lambda *_: ColorRGB(0xfed0fc, "pale-mauve",                            register=True)
    PALE_OAK                             : ColorRGB = lambda *_: ColorRGB(0xbba46d, "pale-oak",                              register=True)
    PALE_OLIVE                           : ColorRGB = lambda *_: ColorRGB(0xb9cc81, "pale-olive",                            register=True)
    PALE_OLIVE_GREEN                     : ColorRGB = lambda *_: ColorRGB(0xb1d27b, "pale-olive-green",                      register=True)
    PALE_ORANGE                          : ColorRGB = lambda *_: ColorRGB(0xffa756, "pale-orange",                           register=True)
    PALE_PEACH                           : ColorRGB = lambda *_: ColorRGB(0xffe5ad, "pale-peach",                            register=True)
    PALE_PERSIMMON                       : ColorRGB = lambda *_: ColorRGB(0xfca474, "pale-persimmon",                        register=True)
    PALE_PINK                            : ColorRGB = lambda *_: ColorRGB(0xffcfdc, "pale-pink",                             register=True)
    PALE_PURPLE                          : ColorRGB = lambda *_: ColorRGB(0xb790d4, "pale-purple",                           register=True, variation_map={
             0xfae6fa:   "pantone" })
    PALE_RED                             : ColorRGB = lambda *_: ColorRGB(0xd9544d, "pale-red",                              register=True)
    PALE_RED_VIOLET                      : ColorRGB = lambda *_: ColorRGB(0xdb7093, "pale-red-violet",                       register=True)
    PALE_ROBIN_EGG_BLUE                  : ColorRGB = lambda *_: ColorRGB(0x96ded1, "pale-robin-egg-blue",                   register=True)
    PALE_ROSE                            : ColorRGB = lambda *_: ColorRGB(0xfdc1c5, "pale-rose",                             register=True)
    PALE_SALMON                          : ColorRGB = lambda *_: ColorRGB(0xffb19a, "pale-salmon",                           register=True)
    PALE_SILVER                          : ColorRGB = lambda *_: ColorRGB(0xc9c0bb, "pale-silver",                           register=True)
    PALE_SKY_BLUE                        : ColorRGB = lambda *_: ColorRGB(0xbdf6fe, "pale-sky-blue",                         register=True)
    PALE_SPRING_BUD                      : ColorRGB = lambda *_: ColorRGB(0xecebbd, "pale-spring-bud",                       register=True)
    PALE_TAUPE                           : ColorRGB = lambda *_: ColorRGB(0xbc987e, "pale-taupe",                            register=True)
    PALE_TEAL                            : ColorRGB = lambda *_: ColorRGB(0x82cbb2, "pale-teal",                             register=True)
    PALE_TURQUOISE                       : ColorRGB = lambda *_: ColorRGB(0xa5fbd5, "pale-turquoise",                        register=True)
    PALE_VIOLET                          : ColorRGB = lambda *_: ColorRGB(0xceaefa, "pale-violet",                           register=True)
    PALE_YELLOW                          : ColorRGB = lambda *_: ColorRGB(0xffff84, "pale-yellow",                           register=True)
    PALE_YOUNG_GREEN_ONION               : ColorRGB = lambda *_: ColorRGB(0x8db255, "pale-young-green-onion",                register=True)
    PALM_LEAF                            : ColorRGB = lambda *_: ColorRGB(0x6f9940, "palm-leaf",                             register=True)
    PANASONIC_BLUE                       : ColorRGB = lambda *_: ColorRGB(0x0040be, "panasonic-blue",                        register=True)
    PANSY_PURPLE                         : ColorRGB = lambda *_: ColorRGB(0x78184a, "pansy-purple",                          register=True)
    PAOLO_VERONESE_GREEN                 : ColorRGB = lambda *_: ColorRGB(0x009b7d, "paolo-veronese-green",                  register=True)
    PAPAYA_WHIP                          : ColorRGB = lambda *_: ColorRGB(0xffefd5, "papaya-whip",                           register=True)
    PARADISE_PINK                        : ColorRGB = lambda *_: ColorRGB(0xe63e62, "paradise-pink",                         register=True)
    PARAKEET_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x7eb6ff, "parakeet-blue",                         register=True)
    PARCHMENT                            : ColorRGB = lambda *_: ColorRGB(0xfefcaf, "parchment",                             register=True)
    PARROT_PINK                          : ColorRGB = lambda *_: ColorRGB(0xd998a0, "parrot-pink",                           register=True)
    PASTEL_BLUE                          : ColorRGB = lambda *_: ColorRGB(0xa2bffe, "pastel-blue",                           register=True)
    PASTEL_BROWN                         : ColorRGB = lambda *_: ColorRGB(0x836953, "pastel-brown",                          register=True)
    PASTEL_GRAY                          : ColorRGB = lambda *_: ColorRGB(0xcfcfc4, "pastel-gray",                           register=True)
    PASTEL_GREEN                         : ColorRGB = lambda *_: ColorRGB(0xb0ff9d, "pastel-green",                          register=True)
    PASTEL_MAGENTA                       : ColorRGB = lambda *_: ColorRGB(0xf49ac2, "pastel-magenta",                        register=True)
    PASTEL_ORANGE                        : ColorRGB = lambda *_: ColorRGB(0xff964f, "pastel-orange",                         register=True)
    PASTEL_PINK                          : ColorRGB = lambda *_: ColorRGB(0xffbacd, "pastel-pink",                           register=True)
    PASTEL_PURPLE                        : ColorRGB = lambda *_: ColorRGB(0xcaa0ff, "pastel-purple",                         register=True)
    PASTEL_RED                           : ColorRGB = lambda *_: ColorRGB(0xdb5856, "pastel-red",                            register=True)
    PASTEL_VIOLET                        : ColorRGB = lambda *_: ColorRGB(0xcb99c9, "pastel-violet",                         register=True)
    PASTEL_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xfffe71, "pastel-yellow",                         register=True)
    PASTELLI                             : ColorRGB = lambda *_: ColorRGB(0x8777d9, "pastelli",                              register=True)
    PATINA                               : ColorRGB = lambda *_: ColorRGB(0x407a52, "patina",                                register=True)
    PATRIARCH                            : ColorRGB = lambda *_: ColorRGB(0x800080, "patriarch",                             register=True)
    PATRINIA_FLOWERS                     : ColorRGB = lambda *_: ColorRGB(0xd9b611, "patrinia-flowers",                      register=True)
    PEA                                  : ColorRGB = lambda *_: ColorRGB(0xa4bf20, "pea",                                   register=True)
    PEA_GREEN                            : ColorRGB = lambda *_: ColorRGB(0x8eab12, "pea-green",                             register=True)
    PEA_SOUP                             : ColorRGB = lambda *_: ColorRGB(0x929901, "pea-soup",                              register=True)
    PEA_SOUP_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x94a617, "pea-soup-green",                        register=True)
    PEACH                                : ColorRGB = lambda *_: ColorRGB(0xffb07c, "peach",                                 register=True, variation_map={
             0xf47983:   "momo-iro" })
    PEACH_BURST                          : ColorRGB = lambda *_: ColorRGB(0xf39998, "peach-burst",                           register=True)
    PEACH_ORANGE                         : ColorRGB = lambda *_: ColorRGB(0xffcc99, "peach-orange",                          register=True)
    PEACH_PUFF                           : ColorRGB = lambda *_: ColorRGB(0xffdab9, "peach-puff",                            register=True)
    PEACH_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xfadfad, "peach-yellow",                          register=True)
    PEACHY_PINK                          : ColorRGB = lambda *_: ColorRGB(0xff9a8a, "peachy-pink",                           register=True)
    PEACOCK_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x016795, "peacock-blue",                          register=True)
    PEAR                                 : ColorRGB = lambda *_: ColorRGB(0xcbf85f, "pear",                                  register=True)
    PEARL                                : ColorRGB = lambda *_: ColorRGB(0xeae0c8, "pearl",                                 register=True)
    PEARL_AQUA                           : ColorRGB = lambda *_: ColorRGB(0x88d8c0, "pearl-aqua",                            register=True)
    PEARLY_PURPLE                        : ColorRGB = lambda *_: ColorRGB(0xb768a2, "pearly-purple",                         register=True)
    PENGUIN_WHITE                        : ColorRGB = lambda *_: ColorRGB(0xf5f3ef, "penguin-white",                         register=True)
    PERIDOT                              : ColorRGB = lambda *_: ColorRGB(0xe6e200, "peridot",                               register=True)
    PERIWINKLE                           : ColorRGB = lambda *_: ColorRGB(0x8e82fe, "periwinkle",                            register=True, variation_map={
             0xc3cde6:   "crayola" })
    PERIWINKLE_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x8f99fb, "periwinkle-blue",                       register=True)
    PERMANENT_GERANIUM_LAKE              : ColorRGB = lambda *_: ColorRGB(0xe12c2c, "permanent-geranium-lake",               register=True)
    PERRYWINKLE                          : ColorRGB = lambda *_: ColorRGB(0x8f8ce7, "perrywinkle",                           register=True)
    PERSIAN_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x1c39bb, "persian-blue",                          register=True)
    PERSIAN_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x00a693, "persian-green",                         register=True)
    PERSIAN_INDIGO                       : ColorRGB = lambda *_: ColorRGB(0x32127a, "persian-indigo",                        register=True)
    PERSIAN_ORANGE                       : ColorRGB = lambda *_: ColorRGB(0xd99058, "persian-orange",                        register=True)
    PERSIAN_PINK                         : ColorRGB = lambda *_: ColorRGB(0xf77fbe, "persian-pink",                          register=True)
    PERSIAN_PLUM                         : ColorRGB = lambda *_: ColorRGB(0x701c1c, "persian-plum",                          register=True)
    PERSIAN_RED                          : ColorRGB = lambda *_: ColorRGB(0xcc3333, "persian-red",                           register=True)
    PERSIAN_ROSE                         : ColorRGB = lambda *_: ColorRGB(0xfe28a2, "persian-rose",                          register=True)
    PERSIMMON                            : ColorRGB = lambda *_: ColorRGB(0xec5800, "persimmon",                             register=True)
    PERSIMMON_JUICE                      : ColorRGB = lambda *_: ColorRGB(0x934337, "persimmon-juice",                       register=True)
    PERU                                 : ColorRGB = lambda *_: ColorRGB(0xcd853f, "peru",                                  register=True)
    PET_ROCK                             : ColorRGB = lambda *_: ColorRGB(0x344563, "pet-rock",                              register=True)
    PETROL                               : ColorRGB = lambda *_: ColorRGB(0x005f6a, "petrol",                                register=True)
    PEWTER_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x8ba8b7, "pewter-blue",                           register=True)
    PHANTOM_MIST                         : ColorRGB = lambda *_: ColorRGB(0xc0b6f2, "phantom-mist",                          register=True)
    PHILIPPINE_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x0038a7, "philippine-blue",                       register=True)
    PHILIPPINE_BRONZE                    : ColorRGB = lambda *_: ColorRGB(0x6e3a07, "philippine-bronze",                     register=True)
    PHILIPPINE_BROWN                     : ColorRGB = lambda *_: ColorRGB(0x5d1916, "philippine-brown",                      register=True)
    PHILIPPINE_GOLD                      : ColorRGB = lambda *_: ColorRGB(0xb17304, "philippine-gold",                       register=True)
    PHILIPPINE_GRAY                      : ColorRGB = lambda *_: ColorRGB(0x8c8c8c, "philippine-gray",                       register=True)
    PHILIPPINE_GREEN                     : ColorRGB = lambda *_: ColorRGB(0x008543, "philippine-green",                      register=True)
    PHILIPPINE_ORANGE                    : ColorRGB = lambda *_: ColorRGB(0xff7300, "philippine-orange",                     register=True)
    PHILIPPINE_PINK                      : ColorRGB = lambda *_: ColorRGB(0xfa1a8e, "philippine-pink",                       register=True)
    PHILIPPINE_RED                       : ColorRGB = lambda *_: ColorRGB(0xce1127, "philippine-red",                        register=True)
    PHILIPPINE_SILVER                    : ColorRGB = lambda *_: ColorRGB(0xb3b3b3, "philippine-silver",                     register=True)
    PHILIPPINE_VIOLET                    : ColorRGB = lambda *_: ColorRGB(0x81007f, "philippine-violet",                     register=True)
    PHILIPPINE_YELLOW                    : ColorRGB = lambda *_: ColorRGB(0xfecb00, "philippine-yellow",                     register=True)
    PHLOX                                : ColorRGB = lambda *_: ColorRGB(0xdf00ff, "phlox",                                 register=True)
    PHTHALO_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x000f89, "phthalo-blue",                          register=True)
    PHTHALO_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x123524, "phthalo-green",                         register=True)
    PICTON_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x45b1e8, "picton-blue",                           register=True)
    PICTORIAL_CARMINE                    : ColorRGB = lambda *_: ColorRGB(0xc30b4e, "pictorial-carmine",                     register=True)
    PIG_PINK                             : ColorRGB = lambda *_: ColorRGB(0xe78ea5, "pig-pink",                              register=True)
    PIGGY_PINK                           : ColorRGB = lambda *_: ColorRGB(0xfddde6, "piggy-pink",                            register=True)
    PINE                                 : ColorRGB = lambda *_: ColorRGB(0x2b5d34, "pine",                                  register=True)
    PINE_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x0a481e, "pine-green",                            register=True)
    PINE_NEEDLE                          : ColorRGB = lambda *_: ColorRGB(0x454d32, "pine-needle",                           register=True)
    PINE_TREE                            : ColorRGB = lambda *_: ColorRGB(0x2a2f23, "pine-tree",                             register=True)
    PINEAPPLE                            : ColorRGB = lambda *_: ColorRGB(0x563c0d, "pineapple",                             register=True)
    PINK                                 : ColorRGB = lambda *_: ColorRGB(0xff81c0, "pink",                                  register=True, variation_map={
             0xd74894:   "pantone" })
    PINK_FLAMINGO                        : ColorRGB = lambda *_: ColorRGB(0xfc74fd, "pink-flamingo",                         register=True)
    PINK_LACE                            : ColorRGB = lambda *_: ColorRGB(0xffddf4, "pink-lace",                             register=True)
    PINK_LAVENDER                        : ColorRGB = lambda *_: ColorRGB(0xd8b2d1, "pink-lavender",                         register=True)
    PINK_PEARL                           : ColorRGB = lambda *_: ColorRGB(0xe7accf, "pink-pearl",                            register=True)
    PINK_PURPLE                          : ColorRGB = lambda *_: ColorRGB(0xdb4bda, "pink-purple",                           register=True)
    PINK_RASPBERRY                       : ColorRGB = lambda *_: ColorRGB(0x980036, "pink-raspberry",                        register=True)
    PINK_RED                             : ColorRGB = lambda *_: ColorRGB(0xf5054f, "pink-red",                              register=True)
    PINK_SHERBET                         : ColorRGB = lambda *_: ColorRGB(0xf78fa7, "pink-sherbet",                          register=True)
    PINKISH                              : ColorRGB = lambda *_: ColorRGB(0xd46a7e, "pinkish",                               register=True)
    PINKISH_BROWN                        : ColorRGB = lambda *_: ColorRGB(0xb17261, "pinkish-brown",                         register=True)
    PINKISH_GREY                         : ColorRGB = lambda *_: ColorRGB(0xc8aca9, "pinkish-grey",                          register=True)
    PINKISH_ORANGE                       : ColorRGB = lambda *_: ColorRGB(0xff724c, "pinkish-orange",                        register=True)
    PINKISH_PURPLE                       : ColorRGB = lambda *_: ColorRGB(0xd648d7, "pinkish-purple",                        register=True)
    PINKISH_RED                          : ColorRGB = lambda *_: ColorRGB(0xf10c45, "pinkish-red",                           register=True)
    PINKISH_TAN                          : ColorRGB = lambda *_: ColorRGB(0xd99b82, "pinkish-tan",                           register=True)
    PINKY                                : ColorRGB = lambda *_: ColorRGB(0xfc86aa, "pinky",                                 register=True)
    PINKY_PURPLE                         : ColorRGB = lambda *_: ColorRGB(0xc94cbe, "pinky-purple",                          register=True)
    PINKY_RED                            : ColorRGB = lambda *_: ColorRGB(0xfc2647, "pinky-red",                             register=True)
    PISS_YELLOW                          : ColorRGB = lambda *_: ColorRGB(0xddd618, "piss-yellow",                           register=True)
    PISTACHIO                            : ColorRGB = lambda *_: ColorRGB(0xc0fa8b, "pistachio",                             register=True)
    PIXIE_DUST                           : ColorRGB = lambda *_: ColorRGB(0xdeebff, "pixie-dust",                            register=True)
    PIXIE_POWDER                         : ColorRGB = lambda *_: ColorRGB(0x391285, "pixie-powder",                          register=True)
    PLAIN_MOUSE                          : ColorRGB = lambda *_: ColorRGB(0x6e5f57, "plain-mouse",                           register=True)
    PLATINUM                             : ColorRGB = lambda *_: ColorRGB(0xe5e4e2, "platinum",                              register=True)
    PLUM                                 : ColorRGB = lambda *_: ColorRGB(0x580f41, "plum",                                  register=True)
    PLUM_BLOSSOM_MOUSE                   : ColorRGB = lambda *_: ColorRGB(0x97645a, "plum-blossom-mouse",                    register=True)
    PLUM_DYED                            : ColorRGB = lambda *_: ColorRGB(0xfa9258, "plum-dyed",                             register=True)
    PLUM_PURPLE                          : ColorRGB = lambda *_: ColorRGB(0x4e0550, "plum-purple",                           register=True, variation_map={
             0x8f4155:   "umemurasaki" })
    PLUMP_PURPLE                         : ColorRGB = lambda *_: ColorRGB(0x5946b2, "plump-purple",                          register=True)
    POISON_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x40fd14, "poison-green",                          register=True)
    POKER_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x35654d, "poker-green",                           register=True)
    POLISHED_BROWN                       : ColorRGB = lambda *_: ColorRGB(0x985538, "polished-brown",                        register=True)
    POLISHED_PINE                        : ColorRGB = lambda *_: ColorRGB(0x5da493, "polished-pine",                         register=True)
    POMEGRANATE                          : ColorRGB = lambda *_: ColorRGB(0x660c21, "pomegranate",                           register=True)
    POMELO                               : ColorRGB = lambda *_: ColorRGB(0x96a53c, "pomelo",                                register=True)
    POMELO_OLIVE                         : ColorRGB = lambda *_: ColorRGB(0xbfbd70, "pomelo-olive",                          register=True)
    POMELO_WHITE                         : ColorRGB = lambda *_: ColorRGB(0xf9ffe3, "pomelo-white",                          register=True)
    POO                                  : ColorRGB = lambda *_: ColorRGB(0x8f7303, "poo",                                   register=True)
    POO_BROWN                            : ColorRGB = lambda *_: ColorRGB(0x885f01, "poo-brown",                             register=True)
    POOP                                 : ColorRGB = lambda *_: ColorRGB(0x7f5e00, "poop",                                  register=True)
    POOP_BROWN                           : ColorRGB = lambda *_: ColorRGB(0x7a5901, "poop-brown",                            register=True)
    POOP_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x6f7c00, "poop-green",                            register=True)
    POPPY_PETAL                          : ColorRGB = lambda *_: ColorRGB(0xf6a08c, "poppy-petal",                           register=True)
    POPPY_SURPRISE                       : ColorRGB = lambda *_: ColorRGB(0xff5630, "poppy-surprise",                        register=True, aliases=['jira-red'])
    POPSTAR                              : ColorRGB = lambda *_: ColorRGB(0xbe4f62, "popstar",                               register=True)
    PORCUPINE                            : ColorRGB = lambda *_: ColorRGB(0x936900, "porcupine",                             register=True)
    PORTLAND_ORANGE                      : ColorRGB = lambda *_: ColorRGB(0xff5a36, "portland-orange",                       register=True)
    POWDER_BLUE                          : ColorRGB = lambda *_: ColorRGB(0xb1d1fc, "powder-blue",                           register=True)
    POWDER_PINK                          : ColorRGB = lambda *_: ColorRGB(0xffb2d0, "powder-pink",                           register=True)
    PRIMARY_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x0804f9, "primary-blue",                          register=True)
    PRINCE                               : ColorRGB = lambda *_: ColorRGB(0x403294, "prince",                                register=True)
    PRINCESS_PERFUME                     : ColorRGB = lambda *_: ColorRGB(0xff85cf, "princess-perfume",                      register=True)
    PRINCETON_ORANGE                     : ColorRGB = lambda *_: ColorRGB(0xf58025, "princeton-orange",                      register=True)
    PROM_DRESS                           : ColorRGB = lambda *_: ColorRGB(0x00a3bf, "prom-dress",                            register=True)
    PRUSSIAN_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x004577, "prussian-blue",                         register=True, variation_map={
             0x003171:   "konjo-iro" })
    PUB_MIX                              : ColorRGB = lambda *_: ColorRGB(0xffc400, "pub-mix",                               register=True)
    PUCE                                 : ColorRGB = lambda *_: ColorRGB(0xa57e52, "puce",                                  register=True)
    PUCE_RED                             : ColorRGB = lambda *_: ColorRGB(0x722f37, "puce-red",                              register=True)
    PUKE                                 : ColorRGB = lambda *_: ColorRGB(0xa5a502, "puke",                                  register=True)
    PUKE_BROWN                           : ColorRGB = lambda *_: ColorRGB(0x947706, "puke-brown",                            register=True)
    PUKE_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x9aae07, "puke-green",                            register=True)
    PUKE_YELLOW                          : ColorRGB = lambda *_: ColorRGB(0xc2be0e, "puke-yellow",                           register=True)
    PULLMAN_BROWN                        : ColorRGB = lambda *_: ColorRGB(0x644117, "pullman-brown",                         register=True)
    PULLMAN_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x3b331c, "pullman-green",                         register=True)
    PUMPKIN                              : ColorRGB = lambda *_: ColorRGB(0xe17701, "pumpkin",                               register=True)
    PUMPKIN_ORANGE                       : ColorRGB = lambda *_: ColorRGB(0xfb7d07, "pumpkin-orange",                        register=True)
    PURE_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x0203e2, "pure-blue",                             register=True)
    PURE_CRIMSON                         : ColorRGB = lambda *_: ColorRGB(0xc3272b, "pure-crimson",                          register=True)
    PURPLE                               : ColorRGB = lambda *_: ColorRGB(0x7e1e9c, "purple",                                register=True, variation_map={
             0x9f00c5:   "munsell", 
             0x4f284b:   "murasaki", 
             0xa020f0:   "x11" })
    PURPLE_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x632de9, "purple-blue",                           register=True)
    PURPLE_BROWN                         : ColorRGB = lambda *_: ColorRGB(0x673a3f, "purple-brown",                          register=True)
    PURPLE_GREY                          : ColorRGB = lambda *_: ColorRGB(0x866f85, "purple-grey",                           register=True)
    PURPLE_HEART                         : ColorRGB = lambda *_: ColorRGB(0x69359c, "purple-heart",                          register=True)
    PURPLE_KITE                          : ColorRGB = lambda *_: ColorRGB(0x512c31, "purple-kite",                           register=True)
    PURPLE_MOUNTAIN_MAJESTY              : ColorRGB = lambda *_: ColorRGB(0x9678b6, "purple-mountain-majesty",               register=True)
    PURPLE_NAVY                          : ColorRGB = lambda *_: ColorRGB(0x4e5180, "purple-navy",                           register=True)
    PURPLE_PINK                          : ColorRGB = lambda *_: ColorRGB(0xe03fd8, "purple-pink",                           register=True)
    PURPLE_PIZZAZZ                       : ColorRGB = lambda *_: ColorRGB(0xfe4eda, "purple-pizzazz",                        register=True)
    PURPLE_PLUM                          : ColorRGB = lambda *_: ColorRGB(0x9c51b6, "purple-plum",                           register=True)
    PURPLE_RED                           : ColorRGB = lambda *_: ColorRGB(0x990147, "purple-red",                            register=True)
    PURPLE_TAUPE                         : ColorRGB = lambda *_: ColorRGB(0x50404d, "purple-taupe",                          register=True)
    PURPLEISH                            : ColorRGB = lambda *_: ColorRGB(0x98568d, "purpleish",                             register=True)
    PURPLEISH_BLUE                       : ColorRGB = lambda *_: ColorRGB(0x6140ef, "purpleish-blue",                        register=True)
    PURPLEISH_PINK                       : ColorRGB = lambda *_: ColorRGB(0xdf4ec8, "purpleish-pink",                        register=True)
    PURPLEY                              : ColorRGB = lambda *_: ColorRGB(0x8756e4, "purpley",                               register=True)
    PURPLEY_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x5f34e7, "purpley-blue",                          register=True)
    PURPLEY_GREY                         : ColorRGB = lambda *_: ColorRGB(0x947e94, "purpley-grey",                          register=True)
    PURPLEY_PINK                         : ColorRGB = lambda *_: ColorRGB(0xc83cb9, "purpley-pink",                          register=True)
    PURPLISH                             : ColorRGB = lambda *_: ColorRGB(0x94568c, "purplish",                              register=True)
    PURPLISH_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x601ef9, "purplish-blue",                         register=True)
    PURPLISH_BROWN                       : ColorRGB = lambda *_: ColorRGB(0x6b4247, "purplish-brown",                        register=True)
    PURPLISH_GREY                        : ColorRGB = lambda *_: ColorRGB(0x7a687f, "purplish-grey",                         register=True)
    PURPLISH_PINK                        : ColorRGB = lambda *_: ColorRGB(0xce5dae, "purplish-pink",                         register=True)
    PURPLISH_RED                         : ColorRGB = lambda *_: ColorRGB(0xb0054b, "purplish-red",                          register=True)
    PURPLY                               : ColorRGB = lambda *_: ColorRGB(0x983fb2, "purply",                                register=True)
    PURPLY_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x661aee, "purply-blue",                           register=True)
    PURPLY_PINK                          : ColorRGB = lambda *_: ColorRGB(0xf075e6, "purply-pink",                           register=True)
    PURPUREUS                            : ColorRGB = lambda *_: ColorRGB(0x9a4eae, "purpureus",                             register=True)
    PUTTY                                : ColorRGB = lambda *_: ColorRGB(0xbeae8a, "putty",                                 register=True)
    QUARTZ                               : ColorRGB = lambda *_: ColorRGB(0x51484f, "quartz",                                register=True)
    QUEEN_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x436b95, "queen-blue",                            register=True)
    QUEEN_PINK                           : ColorRGB = lambda *_: ColorRGB(0xe8ccd7, "queen-pink",                            register=True)
    QUICK_SILVER                         : ColorRGB = lambda *_: ColorRGB(0xa6a6a6, "quick-silver",                          register=True, aliases=['quicksilver'])
    QUINACRIDONE_MAGENTA                 : ColorRGB = lambda *_: ColorRGB(0x8e3a59, "quinacridone-magenta",                  register=True)
    QUINCY                               : ColorRGB = lambda *_: ColorRGB(0x6a5445, "quincy",                                register=True)
    RABBIT_EAR_IRIS                      : ColorRGB = lambda *_: ColorRGB(0x491e3c, "rabbit-ear-iris",                       register=True)
    RACING_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x014600, "racing-green",                          register=True)
    RADICAL_RED                          : ColorRGB = lambda *_: ColorRGB(0xff355e, "radical-red",                           register=True)
    RADIOACTIVE_GREEN                    : ColorRGB = lambda *_: ColorRGB(0x2cfa1f, "radioactive-green",                     register=True)
    RAISIN_BLACK                         : ColorRGB = lambda *_: ColorRGB(0x242124, "raisin-black",                          register=True)
    RAJAH                                : ColorRGB = lambda *_: ColorRGB(0xfbab60, "rajah",                                 register=True)
    RAMBUTAN                             : ColorRGB = lambda *_: ColorRGB(0xc71232, "rambutan",                              register=True)
    RAMBUTAN_RED                         : ColorRGB = lambda *_: ColorRGB(0xa72127, "rambutan-red",                          register=True)
    RAPE_BLOSSOM_BROWN                   : ColorRGB = lambda *_: ColorRGB(0xe3b130, "rape-blossom-brown",                    register=True)
    RAPESEED_OIL                         : ColorRGB = lambda *_: ColorRGB(0xa17917, "rapeseed-oil",                          register=True)
    RASPBERRY                            : ColorRGB = lambda *_: ColorRGB(0xb00149, "raspberry",                             register=True)
    RASPBERRY_PINK                       : ColorRGB = lambda *_: ColorRGB(0xe25098, "raspberry-pink",                        register=True)
    RAW_SIENNA                           : ColorRGB = lambda *_: ColorRGB(0x9a6200, "raw-sienna",                            register=True)
    RAW_UMBER                            : ColorRGB = lambda *_: ColorRGB(0xa75e09, "raw-umber",                             register=True)
    RAZZLE_DAZZLE_ROSE                   : ColorRGB = lambda *_: ColorRGB(0xff33cc, "razzle-dazzle-rose",                    register=True)
    RAZZMATAZZ                           : ColorRGB = lambda *_: ColorRGB(0xe3256b, "razzmatazz",                            register=True)
    RAZZMIC_BERRY                        : ColorRGB = lambda *_: ColorRGB(0x8d4e85, "razzmic-berry",                         register=True)
    REALLY_LIGHT_BLUE                    : ColorRGB = lambda *_: ColorRGB(0xd4ffff, "really-light-blue",                     register=True)
    REBECCA_PURPLE                       : ColorRGB = lambda *_: ColorRGB(0x663399, "rebecca-purple",                        register=True)
    RED                                  : ColorRGB = lambda *_: ColorRGB(0xe50000, "red",                                   register=True, variation_map={
             0xee204d:   "crayola", 
             0xf2003c:   "munsell", 
             0xc40233:   "ncs", 
             0xfe2712:   "ryb" })
    RED_BEAN                             : ColorRGB = lambda *_: ColorRGB(0x672422, "red-bean",                              register=True)
    RED_BIRCH                            : ColorRGB = lambda *_: ColorRGB(0x9d2b22, "red-birch",                             register=True)
    RED_BRONZE                           : ColorRGB = lambda *_: ColorRGB(0xfb8136, "red-bronze",                            register=True)
    RED_BROWN                            : ColorRGB = lambda *_: ColorRGB(0x8b2e16, "red-brown",                             register=True, variation_map={
             0xa13d2d:   "edo-brown", 
             0x8b352d:   "lit-chestnut-plum" })
    RED_COLA                             : ColorRGB = lambda *_: ColorRGB(0xb62020, "red-cola",                              register=True)
    RED_DEVIL                            : ColorRGB = lambda *_: ColorRGB(0x860111, "red-devil",                             register=True)
    RED_DIRT                             : ColorRGB = lambda *_: ColorRGB(0xde350b, "red-dirt",                              register=True)
    RED_INCENSE                          : ColorRGB = lambda *_: ColorRGB(0xf07f5e, "red-incense",                           register=True)
    RED_KALM                             : ColorRGB = lambda *_: ColorRGB(0xa54242, "red-kalm",                              register=True)
    RED_KITE                             : ColorRGB = lambda *_: ColorRGB(0x913228, "red-kite",                              register=True)
    RED_OCHRE                            : ColorRGB = lambda *_: ColorRGB(0x9f5233, "red-ochre",                             register=True)
    RED_ORANGE                           : ColorRGB = lambda *_: ColorRGB(0xfd3c06, "red-orange",                            register=True, variation_map={
             0xff4500:   "color-wheel", 
             0xdc3023:   "lit-orangutan" })
    RED_PANIK                            : ColorRGB = lambda *_: ColorRGB(0xff5b56, "red-panik",                             register=True)
    RED_PINK                             : ColorRGB = lambda *_: ColorRGB(0xfa2a55, "red-pink",                              register=True)
    RED_PLUM                             : ColorRGB = lambda *_: ColorRGB(0xdb5a6b, "red-plum",                              register=True)
    RED_PURPLE                           : ColorRGB = lambda *_: ColorRGB(0x820747, "red-purple",                            register=True)
    RED_SALSA                            : ColorRGB = lambda *_: ColorRGB(0xfd3a4a, "red-salsa",                             register=True)
    RED_VIOLET                           : ColorRGB = lambda *_: ColorRGB(0x9e0168, "red-violet",                            register=True, variation_map={
             0x922b3e:   "color-wheel", 
             0xc0448f:   "crayola" })
    RED_WINE                             : ColorRGB = lambda *_: ColorRGB(0x8c0034, "red-wine",                              register=True)
    RED_WISTERIA                         : ColorRGB = lambda *_: ColorRGB(0xbb7796, "red-wisteria",                          register=True)
    REDDISH                              : ColorRGB = lambda *_: ColorRGB(0xc44240, "reddish",                               register=True)
    REDDISH_BROWN                        : ColorRGB = lambda *_: ColorRGB(0x7f2b0a, "reddish-brown",                         register=True)
    REDDISH_GREY                         : ColorRGB = lambda *_: ColorRGB(0x997570, "reddish-grey",                          register=True)
    REDDISH_ORANGE                       : ColorRGB = lambda *_: ColorRGB(0xf8481c, "reddish-orange",                        register=True)
    REDDISH_PINK                         : ColorRGB = lambda *_: ColorRGB(0xfe2c54, "reddish-pink",                          register=True)
    REDDISH_PURPLE                       : ColorRGB = lambda *_: ColorRGB(0x910951, "reddish-purple",                        register=True)
    REDDY_BROWN                          : ColorRGB = lambda *_: ColorRGB(0x6e1005, "reddy-brown",                           register=True)
    REDWOOD                              : ColorRGB = lambda *_: ColorRGB(0xa45a52, "redwood",                               register=True)
    REGALIA                              : ColorRGB = lambda *_: ColorRGB(0x522d80, "regalia",                               register=True)
    RESOLUTION_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x002387, "resolution-blue",                       register=True)
    RHYTHM                               : ColorRGB = lambda *_: ColorRGB(0x777696, "rhythm",                                register=True)
    RICH_BLACK                           : ColorRGB = lambda *_: ColorRGB(0x004040, "rich-black",                            register=True, variation_map={
             0x010b13:   "fogra29", 
             0x010203:   "fogra39" })
    RICH_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x021bf9, "rich-blue",                             register=True)
    RICH_BRILLIANT_LAVENDER              : ColorRGB = lambda *_: ColorRGB(0xf1a7fe, "rich-brilliant-lavender",               register=True)
    RICH_ELECTRIC_BLUE                   : ColorRGB = lambda *_: ColorRGB(0x0892d0, "rich-electric-blue",                    register=True)
    RICH_GARDENIA                        : ColorRGB = lambda *_: ColorRGB(0xf57f4f, "rich-gardenia",                         register=True)
    RICH_LAVENDER                        : ColorRGB = lambda *_: ColorRGB(0xa76bcf, "rich-lavender",                         register=True)
    RICH_LILAC                           : ColorRGB = lambda *_: ColorRGB(0xb666d2, "rich-lilac",                            register=True)
    RICH_PURPLE                          : ColorRGB = lambda *_: ColorRGB(0x720058, "rich-purple",                           register=True)
    RIFLE_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x444c38, "rifle-green",                           register=True)
    RIKAN_BROWN                          : ColorRGB = lambda *_: ColorRGB(0x534a32, "rikan-brown",                           register=True)
    RINSED_OUT_RED                       : ColorRGB = lambda *_: ColorRGB(0xff7952, "rinsed-out-red",                        register=True)
    RIPE_MANGO                           : ColorRGB = lambda *_: ColorRGB(0xffc324, "ripe-mango",                            register=True)
    ROBIN_EGG_BLUE                       : ColorRGB = lambda *_: ColorRGB(0x8af1fe, "robin-egg-blue",                        register=True)
    ROBINS_EGG                           : ColorRGB = lambda *_: ColorRGB(0x6dedfd, "robins-egg",                            register=True)
    ROBINS_EGG_BLUE                      : ColorRGB = lambda *_: ColorRGB(0x98eff9, "robins-egg-blue",                       register=True)
    ROCKET_METALLIC                      : ColorRGB = lambda *_: ColorRGB(0x8a7f80, "rocket-metallic",                       register=True)
    ROMAN_SILVER                         : ColorRGB = lambda *_: ColorRGB(0x838996, "roman-silver",                          register=True)
    ROOT_BEER                            : ColorRGB = lambda *_: ColorRGB(0x290e05, "root-beer",                             register=True)
    ROSA                                 : ColorRGB = lambda *_: ColorRGB(0xfe86a4, "rosa",                                  register=True)
    ROSE                                 : ColorRGB = lambda *_: ColorRGB(0xcf6275, "rose",                                  register=True)
    ROSE_BONBON                          : ColorRGB = lambda *_: ColorRGB(0xf9429e, "rose-bonbon",                           register=True)
    ROSE_DUST                            : ColorRGB = lambda *_: ColorRGB(0x9e5e6f, "rose-dust",                             register=True)
    ROSE_EBONY                           : ColorRGB = lambda *_: ColorRGB(0x674846, "rose-ebony",                            register=True)
    ROSE_GARNET                          : ColorRGB = lambda *_: ColorRGB(0x960145, "rose-garnet",                           register=True)
    ROSE_GOLD                            : ColorRGB = lambda *_: ColorRGB(0xb76e79, "rose-gold",                             register=True)
    ROSE_PINK                            : ColorRGB = lambda *_: ColorRGB(0xf7879a, "rose-pink",                             register=True)
    ROSE_QUARTZ_PINK                     : ColorRGB = lambda *_: ColorRGB(0xbd559c, "rose-quartz-pink",                      register=True)
    ROSE_RED                             : ColorRGB = lambda *_: ColorRGB(0xbe013c, "rose-red",                              register=True)
    ROSE_TAUPE                           : ColorRGB = lambda *_: ColorRGB(0x905d5d, "rose-taupe",                            register=True)
    ROSE_VALE                            : ColorRGB = lambda *_: ColorRGB(0xab4e52, "rose-vale",                             register=True)
    ROSEWOOD                             : ColorRGB = lambda *_: ColorRGB(0x65000b, "rosewood",                              register=True)
    ROSIE                                : ColorRGB = lambda *_: ColorRGB(0xffebe6, "rosie",                                 register=True)
    ROSSO_CORSA                          : ColorRGB = lambda *_: ColorRGB(0xd40000, "rosso-corsa",                           register=True)
    ROSY_BROWN                           : ColorRGB = lambda *_: ColorRGB(0xbc8f8f, "rosy-brown",                            register=True)
    ROSY_PINK                            : ColorRGB = lambda *_: ColorRGB(0xf6688e, "rosy-pink",                             register=True)
    ROUGE                                : ColorRGB = lambda *_: ColorRGB(0xab1239, "rouge",                                 register=True)
    ROYAL                                : ColorRGB = lambda *_: ColorRGB(0x0c1793, "royal",                                 register=True)
    ROYAL_AZURE                          : ColorRGB = lambda *_: ColorRGB(0x0038a8, "royal-azure",                           register=True)
    ROYAL_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x0504aa, "royal-blue",                            register=True, variation_map={
             0x002366:   "dark", 
             0x4169e1:   "light" })
    ROYAL_BROWN                          : ColorRGB = lambda *_: ColorRGB(0x523b35, "royal-brown",                           register=True)
    ROYAL_FUCHSIA                        : ColorRGB = lambda *_: ColorRGB(0xca2c92, "royal-fuchsia",                         register=True)
    ROYAL_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x136207, "royal-green",                           register=True)
    ROYAL_ORANGE                         : ColorRGB = lambda *_: ColorRGB(0xf99245, "royal-orange",                          register=True)
    ROYAL_PINK                           : ColorRGB = lambda *_: ColorRGB(0xe73895, "royal-pink",                            register=True)
    ROYAL_PURPLE                         : ColorRGB = lambda *_: ColorRGB(0x4b006e, "royal-purple",                          register=True)
    ROYAL_RED                            : ColorRGB = lambda *_: ColorRGB(0xd00060, "royal-red",                             register=True, variation_map={
             0x9b1c31:   "2" })
    RUBER                                : ColorRGB = lambda *_: ColorRGB(0xce4676, "ruber",                                 register=True)
    RUBINE_RED                           : ColorRGB = lambda *_: ColorRGB(0xd10056, "rubine-red",                            register=True)
    RUBY                                 : ColorRGB = lambda *_: ColorRGB(0xca0147, "ruby",                                  register=True)
    RUBY_RED                             : ColorRGB = lambda *_: ColorRGB(0x9b111e, "ruby-red",                              register=True)
    RUDDY                                : ColorRGB = lambda *_: ColorRGB(0xff0028, "ruddy",                                 register=True)
    RUDDY_BROWN                          : ColorRGB = lambda *_: ColorRGB(0xbb6528, "ruddy-brown",                           register=True)
    RUDDY_PINK                           : ColorRGB = lambda *_: ColorRGB(0xe18e96, "ruddy-pink",                            register=True)
    RUFOUS                               : ColorRGB = lambda *_: ColorRGB(0xa81c07, "rufous",                                register=True)
    RUM                                  : ColorRGB = lambda *_: ColorRGB(0x716675, "rum",                                   register=True)
    RUSSET                               : ColorRGB = lambda *_: ColorRGB(0xa13905, "russet",                                register=True)
    RUSSIAN_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x679267, "russian-green",                         register=True)
    RUSSIAN_VIOLET                       : ColorRGB = lambda *_: ColorRGB(0x32174d, "russian-violet",                        register=True)
    RUST                                 : ColorRGB = lambda *_: ColorRGB(0xa83c09, "rust",                                  register=True)
    RUST_BROWN                           : ColorRGB = lambda *_: ColorRGB(0x8b3103, "rust-brown",                            register=True)
    RUST_ORANGE                          : ColorRGB = lambda *_: ColorRGB(0xc45508, "rust-orange",                           register=True)
    RUST_RED                             : ColorRGB = lambda *_: ColorRGB(0xaa2704, "rust-red",                              register=True)
    RUSTED_LIGHT_BLUE                    : ColorRGB = lambda *_: ColorRGB(0x6a7f7a, "rusted-light-blue",                     register=True)
    RUSTY_CELADON                        : ColorRGB = lambda *_: ColorRGB(0x898a74, "rusty-celadon",                         register=True)
    RUSTY_ORANGE                         : ColorRGB = lambda *_: ColorRGB(0xcd5909, "rusty-orange",                          register=True)
    RUSTY_RED                            : ColorRGB = lambda *_: ColorRGB(0xaf2f0d, "rusty-red",                             register=True)
    RUSTY_STORAGE                        : ColorRGB = lambda *_: ColorRGB(0x455859, "rusty-storage",                         register=True)
    RUSTY_STOREROOM                      : ColorRGB = lambda *_: ColorRGB(0x3a403b, "rusty-storeroom",                       register=True)
    SACRAMENTO_STATE_GREEN               : ColorRGB = lambda *_: ColorRGB(0x043927, "sacramento-state-green",                register=True)
    SADDLE_BROWN                         : ColorRGB = lambda *_: ColorRGB(0x8b4513, "saddle-brown",                          register=True)
    SAFETY_ORANGE                        : ColorRGB = lambda *_: ColorRGB(0xff7800, "safety-orange",                         register=True, variation_map={
             0xff6700:   "blaze-orange" })
    SAFETY_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xeed202, "safety-yellow",                         register=True)
    SAFFLOWER                            : ColorRGB = lambda *_: ColorRGB(0x5a4f74, "safflower",                             register=True)
    SAFFRON                              : ColorRGB = lambda *_: ColorRGB(0xfeb209, "saffron",                               register=True)
    SAGE                                 : ColorRGB = lambda *_: ColorRGB(0x87ae73, "sage",                                  register=True)
    SAGE_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x88b378, "sage-green",                            register=True)
    SALMON                               : ColorRGB = lambda *_: ColorRGB(0xff796c, "salmon",                                register=True)
    SALMON_PINK                          : ColorRGB = lambda *_: ColorRGB(0xfe7b7c, "salmon-pink",                           register=True)
    SALMON_ROSE                          : ColorRGB = lambda *_: ColorRGB(0xe7968b, "salmon-rose",                           register=True)
    SALMON_SASHIMI                       : ColorRGB = lambda *_: ColorRGB(0xff7452, "salmon-sashimi",                        register=True)
    SAMSUNG_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x12279e, "samsung-blue",                          register=True)
    SAND                                 : ColorRGB = lambda *_: ColorRGB(0xe2ca76, "sand",                                  register=True)
    SAND_BROWN                           : ColorRGB = lambda *_: ColorRGB(0xcba560, "sand-brown",                            register=True)
    SAND_YELLOW                          : ColorRGB = lambda *_: ColorRGB(0xfce166, "sand-yellow",                           register=True)
    SANDSTONE                            : ColorRGB = lambda *_: ColorRGB(0xc9ae74, "sandstone",                             register=True)
    SANDSTORM                            : ColorRGB = lambda *_: ColorRGB(0xecd540, "sandstorm",                             register=True)
    SANDY                                : ColorRGB = lambda *_: ColorRGB(0xf1da7a, "sandy",                                 register=True)
    SANDY_BROWN                          : ColorRGB = lambda *_: ColorRGB(0xc4a661, "sandy-brown",                           register=True)
    SANDY_TAN                            : ColorRGB = lambda *_: ColorRGB(0xfdd9b5, "sandy-tan",                             register=True)
    SANDY_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xfdee73, "sandy-yellow",                          register=True)
    SANGRIA                              : ColorRGB = lambda *_: ColorRGB(0x92000a, "sangria",                               register=True)
    SAP_GREEN                            : ColorRGB = lambda *_: ColorRGB(0x5c8b15, "sap-green",                             register=True)
    SAPPANWOOD                           : ColorRGB = lambda *_: ColorRGB(0x7e2639, "sappanwood",                            register=True)
    SAPPANWOOD_INCENSE                   : ColorRGB = lambda *_: ColorRGB(0xa24f46, "sappanwood-incense",                    register=True)
    SAPPHIRE                             : ColorRGB = lambda *_: ColorRGB(0x2138ab, "sapphire",                              register=True)
    SASQUATCH_SOCKS                      : ColorRGB = lambda *_: ColorRGB(0xff4681, "sasquatch-socks",                       register=True)
    SATIN_SHEEN_GOLD                     : ColorRGB = lambda *_: ColorRGB(0xcba135, "satin-sheen-gold",                      register=True)
    SAWTOOTH_OAK                         : ColorRGB = lambda *_: ColorRGB(0xec956c, "sawtooth-oak",                          register=True)
    SCARLET                              : ColorRGB = lambda *_: ColorRGB(0xbe0119, "scarlet",                               register=True, variation_map={
             0xcf3a24:   "blood" })
    SCHOOL_BUS_YELLOW                    : ColorRGB = lambda *_: ColorRGB(0xffd800, "school-bus-yellow",                     register=True)
    SCHWAG                               : ColorRGB = lambda *_: ColorRGB(0xb3d4ff, "schwag",                                register=True)
    SCORCHED_BROWN                       : ColorRGB = lambda *_: ColorRGB(0x351f19, "scorched-brown",                        register=True)
    SCREAMIN_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x66ff66, "screamin-green",                        register=True)
    SEA                                  : ColorRGB = lambda *_: ColorRGB(0x3c9992, "sea",                                   register=True)
    SEA_BLUE                             : ColorRGB = lambda *_: ColorRGB(0x047495, "sea-blue",                              register=True)
    SEA_FOAM_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x9fe2bf, "sea-foam-green",                        register=True)
    SEA_GREEN                            : ColorRGB = lambda *_: ColorRGB(0x53fca1, "sea-green",                             register=True, variation_map={
             0x00ffcd:   "crayola" })
    SEA_SERPENT                          : ColorRGB = lambda *_: ColorRGB(0x4bc7cf, "sea-serpent",                           register=True)
    SEAFOAM                              : ColorRGB = lambda *_: ColorRGB(0x80f9ad, "seafoam",                               register=True)
    SEAFOAM_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x78d1b6, "seafoam-blue",                          register=True)
    SEAFOAM_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x7af9ab, "seafoam-green",                         register=True)
    SEAL_BROWN                           : ColorRGB = lambda *_: ColorRGB(0x59260b, "seal-brown",                            register=True)
    SEASHELL                             : ColorRGB = lambda *_: ColorRGB(0xfff5ee, "seashell",                              register=True)
    SEAWEED                              : ColorRGB = lambda *_: ColorRGB(0x18d17b, "seaweed",                               register=True)
    SEAWEED_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x35ad6b, "seaweed-green",                         register=True)
    SELECTIVE_YELLOW                     : ColorRGB = lambda *_: ColorRGB(0xffba00, "selective-yellow",                      register=True)
    SEN_NO_RIKYUS_TEA                    : ColorRGB = lambda *_: ColorRGB(0x826b58, "sen-no-rikyus-tea",                     register=True)
    SENTINEL                             : ColorRGB = lambda *_: ColorRGB(0xb3bac5, "sentinel",                              register=True)
    SEPIA                                : ColorRGB = lambda *_: ColorRGB(0x985e2b, "sepia",                                 register=True)
    SEQUOIA_BEIGE                        : ColorRGB = lambda *_: ColorRGB(0xb1b7af, "sequoia-beige",                         register=True)
    SESAME_STREET_GREEN                  : ColorRGB = lambda *_: ColorRGB(0x00a870, "sesame-street-green",                   register=True)
    SHABBY_CHIC                          : ColorRGB = lambda *_: ColorRGB(0x008da6, "shabby-chic",                           register=True)
    SHADOW                               : ColorRGB = lambda *_: ColorRGB(0x8a795d, "shadow",                                register=True)
    SHADOW_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x778ba5, "shadow-blue",                           register=True)
    SHAMPOO                              : ColorRGB = lambda *_: ColorRGB(0xffcff1, "shampoo",                               register=True)
    SHAMROCK                             : ColorRGB = lambda *_: ColorRGB(0x01b44c, "shamrock",                              register=True)
    SHAMROCK_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x02c14d, "shamrock-green",                        register=True)
    SHANDY                               : ColorRGB = lambda *_: ColorRGB(0xffe670, "shandy",                                register=True)
    SHEEN_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x8fd400, "sheen-green",                           register=True)
    SHIMMERING_BLUSH                     : ColorRGB = lambda *_: ColorRGB(0xd98695, "shimmering-blush",                      register=True)
    SHINY_SHAMROCK                       : ColorRGB = lambda *_: ColorRGB(0x5fa778, "shiny-shamrock",                        register=True)
    SHIT                                 : ColorRGB = lambda *_: ColorRGB(0x7f5f00, "shit",                                  register=True)
    SHIT_BROWN                           : ColorRGB = lambda *_: ColorRGB(0x7b5804, "shit-brown",                            register=True)
    SHIT_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x758000, "shit-green",                            register=True)
    SHOCKING_PINK                        : ColorRGB = lambda *_: ColorRGB(0xfe02a2, "shocking-pink",                         register=True, variation_map={
             0xff6fff:   "crayola" })
    SICK_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x9db92c, "sick-green",                            register=True)
    SICKLY_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x94b21c, "sickly-green",                          register=True)
    SICKLY_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xd0e429, "sickly-yellow",                         register=True)
    SIENNA                               : ColorRGB = lambda *_: ColorRGB(0xa9561e, "sienna",                                register=True)
    SILK_CREPE_BROWN                     : ColorRGB = lambda *_: ColorRGB(0x354e4b, "silk-crepe-brown",                      register=True)
    SILVER                               : ColorRGB = lambda *_: ColorRGB(0xc5c9c7, "silver",                                register=True)
    SILVER_CHALICE                       : ColorRGB = lambda *_: ColorRGB(0xacacac, "silver-chalice",                        register=True)
    SILVER_FOIL                          : ColorRGB = lambda *_: ColorRGB(0xafb1ae, "silver-foil",                           register=True)
    SILVER_GRAY                          : ColorRGB = lambda *_: ColorRGB(0x97867c, "silver-gray",                           register=True)
    SILVER_LAKE_BLUE                     : ColorRGB = lambda *_: ColorRGB(0x5d89ba, "silver-lake-blue",                      register=True)
    SILVER_METALLIC                      : ColorRGB = lambda *_: ColorRGB(0xaaa9ad, "silver-metallic",                       register=True)
    SILVER_PINK                          : ColorRGB = lambda *_: ColorRGB(0xc4aead, "silver-pink",                           register=True)
    SILVER_SAND                          : ColorRGB = lambda *_: ColorRGB(0xbfc1c2, "silver-sand",                           register=True)
    SIMMERED_SEAWEED                     : ColorRGB = lambda *_: ColorRGB(0x4c3d30, "simmered-seaweed",                      register=True)
    SINOPIA                              : ColorRGB = lambda *_: ColorRGB(0xcb410b, "sinopia",                               register=True)
    SISKIN_SPROUT_YELLOW                 : ColorRGB = lambda *_: ColorRGB(0x7a942e, "siskin-sprout-yellow",                  register=True)
    SIZZLING_RED                         : ColorRGB = lambda *_: ColorRGB(0xff3855, "sizzling-red",                          register=True)
    SIZZLING_SUNRISE                     : ColorRGB = lambda *_: ColorRGB(0xffdb00, "sizzling-sunrise",                      register=True)
    SKOBELOFF                            : ColorRGB = lambda *_: ColorRGB(0x007474, "skobeloff",                             register=True)
    SKY                                  : ColorRGB = lambda *_: ColorRGB(0x82cafc, "sky",                                   register=True)
    SKY_BLUE                             : ColorRGB = lambda *_: ColorRGB(0x75bbfd, "sky-blue",                              register=True, variation_map={
             0x76d7ea:   "crayola", 
             0x4d8fac:   "sora-iro" })
    SKY_MAGENTA                          : ColorRGB = lambda *_: ColorRGB(0xcf71af, "sky-magenta",                           register=True)
    SKYLIGHT_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x0395f3, "skylight-blue",                         register=True)
    SLATE                                : ColorRGB = lambda *_: ColorRGB(0x516572, "slate",                                 register=True, variation_map={
             0x091e42:   "atlassian" })
    SLATE_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x5b7c99, "slate-blue",                            register=True)
    SLATE_GRAY                           : ColorRGB = lambda *_: ColorRGB(0x708090, "slate-gray",                            register=True)
    SLATE_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x658d6d, "slate-green",                           register=True)
    SLATE_GREY                           : ColorRGB = lambda *_: ColorRGB(0x59656d, "slate-grey",                            register=True)
    SLIME                                : ColorRGB = lambda *_: ColorRGB(0x00875a, "slime",                                 register=True)
    SLIME_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x99cc04, "slime-green",                           register=True)
    SLIMY_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x299617, "slimy-green",                           register=True)
    SMASHED_PUMPKIN                      : ColorRGB = lambda *_: ColorRGB(0xff6d3a, "smashed-pumpkin",                       register=True)
    SMITTEN                              : ColorRGB = lambda *_: ColorRGB(0xc84186, "smitten",                               register=True)
    SMOKE                                : ColorRGB = lambda *_: ColorRGB(0x738276, "smoke",                                 register=True)
    SMOKEY_TOPAZ                         : ColorRGB = lambda *_: ColorRGB(0x832a0d, "smokey-topaz",                          register=True)
    SMOKY_BLACK                          : ColorRGB = lambda *_: ColorRGB(0x100c08, "smoky-black",                           register=True)
    SMOKY_TOPAZ                          : ColorRGB = lambda *_: ColorRGB(0x933d41, "smoky-topaz",                           register=True)
    SNORLAX                              : ColorRGB = lambda *_: ColorRGB(0x253858, "snorlax",                               register=True)
    SNOT                                 : ColorRGB = lambda *_: ColorRGB(0xacbb0d, "snot",                                  register=True)
    SNOT_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x9dc100, "snot-green",                            register=True)
    SNOW                                 : ColorRGB = lambda *_: ColorRGB(0xfffafa, "snow",                                  register=True)
    SNOZZBERRY                           : ColorRGB = lambda *_: ColorRGB(0x5243aa, "snozzberry",                            register=True)
    SOAP                                 : ColorRGB = lambda *_: ColorRGB(0xcec8ef, "soap",                                  register=True)
    SODIUM_EXPLOSION                     : ColorRGB = lambda *_: ColorRGB(0x0065ff, "sodium-explosion",                      register=True)
    SOFT_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x6488ea, "soft-blue",                             register=True)
    SOFT_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x6fc276, "soft-green",                            register=True)
    SOFT_PINK                            : ColorRGB = lambda *_: ColorRGB(0xfdb0c0, "soft-pink",                             register=True)
    SOFT_PURPLE                          : ColorRGB = lambda *_: ColorRGB(0xa66fb5, "soft-purple",                           register=True)
    SOLDIER_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x545a2c, "soldier-green",                         register=True)
    SOLID_PINK                           : ColorRGB = lambda *_: ColorRGB(0x893843, "solid-pink",                            register=True)
    SONIC_SILVER                         : ColorRGB = lambda *_: ColorRGB(0x757575, "sonic-silver",                          register=True)
    SOOTY_WILLOW_BAMBOO                  : ColorRGB = lambda *_: ColorRGB(0x4d4b3a, "sooty-willow-bamboo",                   register=True)
    SPACE_CADET                          : ColorRGB = lambda *_: ColorRGB(0x1d2951, "space-cadet",                           register=True)
    SPANISH_BISTRE                       : ColorRGB = lambda *_: ColorRGB(0x807532, "spanish-bistre",                        register=True)
    SPANISH_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x0070b8, "spanish-blue",                          register=True)
    SPANISH_CARMINE                      : ColorRGB = lambda *_: ColorRGB(0xd10047, "spanish-carmine",                       register=True)
    SPANISH_CRIMSON                      : ColorRGB = lambda *_: ColorRGB(0xe51a4c, "spanish-crimson",                       register=True)
    SPANISH_GRAY                         : ColorRGB = lambda *_: ColorRGB(0x989898, "spanish-gray",                          register=True)
    SPANISH_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x009150, "spanish-green",                         register=True)
    SPANISH_ORANGE                       : ColorRGB = lambda *_: ColorRGB(0xe86100, "spanish-orange",                        register=True)
    SPANISH_PINK                         : ColorRGB = lambda *_: ColorRGB(0xf7bfbe, "spanish-pink",                          register=True)
    SPANISH_PURPLE                       : ColorRGB = lambda *_: ColorRGB(0x66033c, "spanish-purple",                        register=True)
    SPANISH_RED                          : ColorRGB = lambda *_: ColorRGB(0xe60026, "spanish-red",                           register=True)
    SPANISH_VIOLET                       : ColorRGB = lambda *_: ColorRGB(0x4c2882, "spanish-violet",                        register=True)
    SPANISH_VIRIDIAN                     : ColorRGB = lambda *_: ColorRGB(0x007f5c, "spanish-viridian",                      register=True)
    SPANISH_YELLOW                       : ColorRGB = lambda *_: ColorRGB(0xf6b511, "spanish-yellow",                        register=True)
    SPARROW_BROWN                        : ColorRGB = lambda *_: ColorRGB(0x8c4736, "sparrow-brown",                         register=True)
    SPARTAN_CRIMSON                      : ColorRGB = lambda *_: ColorRGB(0x9e1316, "spartan-crimson",                       register=True)
    SPEARMINT                            : ColorRGB = lambda *_: ColorRGB(0x1ef876, "spearmint",                             register=True)
    SPICY_MIX                            : ColorRGB = lambda *_: ColorRGB(0x8b5f4d, "spicy-mix",                             register=True)
    SPICY_RED_BROWN                      : ColorRGB = lambda *_: ColorRGB(0xb35c44, "spicy-red-brown",                       register=True)
    SPIRO_DISCO_BALL                     : ColorRGB = lambda *_: ColorRGB(0x0fc0fc, "spiro-disco-ball",                      register=True)
    SPONGE                               : ColorRGB = lambda *_: ColorRGB(0xfdfe03, "sponge",                                register=True)
    SPOOKY_GHOST                         : ColorRGB = lambda *_: ColorRGB(0x97a0af, "spooky-ghost",                          register=True)
    SPRING_BUD                           : ColorRGB = lambda *_: ColorRGB(0xa7fc00, "spring-bud",                            register=True)
    SPRING_FROST                         : ColorRGB = lambda *_: ColorRGB(0x87ff2a, "spring-frost",                          register=True)
    SPRING_GREEN                         : ColorRGB = lambda *_: ColorRGB(0xa9f971, "spring-green",                          register=True)
    SPRINT_RED                           : ColorRGB = lambda *_: ColorRGB(0xec2131, "sprint-red",                            register=True)
    SPRINT_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xffdd05, "sprint-yellow",                         register=True)
    SPRUCE                               : ColorRGB = lambda *_: ColorRGB(0x0a5f38, "spruce",                                register=True)
    SQUASH                               : ColorRGB = lambda *_: ColorRGB(0xf2ab15, "squash",                                register=True)
    SQUID_INK                            : ColorRGB = lambda *_: ColorRGB(0x172b4d, "squid-ink",                             register=True, aliases=['jira-black'])
    ST_PATRICKS_BLUE                     : ColorRGB = lambda *_: ColorRGB(0x23297a, "st-patricks-blue",                      register=True)
    STAINED_RED                          : ColorRGB = lambda *_: ColorRGB(0x78779b, "stained-red",                           register=True)
    STAR_COMMAND_BLUE                    : ColorRGB = lambda *_: ColorRGB(0x007bb8, "star-command-blue",                     register=True)
    STEAMED_CHESTNUT                     : ColorRGB = lambda *_: ColorRGB(0xd3b17d, "steamed-chestnut",                      register=True)
    STEEL                                : ColorRGB = lambda *_: ColorRGB(0x738595, "steel",                                 register=True)
    STEEL_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x5a7d9a, "steel-blue",                            register=True)
    STEEL_GREY                           : ColorRGB = lambda *_: ColorRGB(0x6f828a, "steel-grey",                            register=True)
    STEEL_PINK                           : ColorRGB = lambda *_: ColorRGB(0xcc33cc, "steel-pink",                            register=True)
    STEEL_TEAL                           : ColorRGB = lambda *_: ColorRGB(0x5f8a8b, "steel-teal",                            register=True)
    STONE                                : ColorRGB = lambda *_: ColorRGB(0xada587, "stone",                                 register=True)
    STONE_TERRACE                        : ColorRGB = lambda *_: ColorRGB(0xa09484, "stone-terrace",                         register=True)
    STOP_RED                             : ColorRGB = lambda *_: ColorRGB(0xcf142b, "stop-red",                              register=True)
    STOREROOM_BROWN                      : ColorRGB = lambda *_: ColorRGB(0x3d4035, "storeroom-brown",                       register=True)
    STORMCLOUD                           : ColorRGB = lambda *_: ColorRGB(0x4f666a, "stormcloud",                            register=True)
    STORMY_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x507b9c, "stormy-blue",                           register=True)
    STRAW                                : ColorRGB = lambda *_: ColorRGB(0xfcf679, "straw",                                 register=True)
    STRAWBERRY                           : ColorRGB = lambda *_: ColorRGB(0xfb2943, "strawberry",                            register=True)
    STRAWBERRY_DAIQUIRI                  : ColorRGB = lambda *_: ColorRGB(0xd9463e, "strawberry-daiquiri",                   register=True)
    STRAWBERRY_JAM                       : ColorRGB = lambda *_: ColorRGB(0x8b171a, "strawberry-jam",                        register=True)
    STRAWBERRY_RED                       : ColorRGB = lambda *_: ColorRGB(0xc83f49, "strawberry-red",                        register=True)
    STRONG_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x0c06f7, "strong-blue",                           register=True)
    STRONG_BOY_PINK                      : ColorRGB = lambda *_: ColorRGB(0xe9399e, "strong-boy-pink",                       register=True)
    STRONG_PINK                          : ColorRGB = lambda *_: ColorRGB(0xff0789, "strong-pink",                           register=True)
    STYLISH_PERSIMMON                    : ColorRGB = lambda *_: ColorRGB(0xffa26b, "stylish-persimmon",                     register=True)
    SUGAR_PLUM                           : ColorRGB = lambda *_: ColorRGB(0x914e75, "sugar-plum",                            register=True)
    SUMAC_DYED                           : ColorRGB = lambda *_: ColorRGB(0xe08a1e, "sumac-dyed",                            register=True, variation_map={
             0x592b1f:   "korozen" })
    SUN_YELLOW                           : ColorRGB = lambda *_: ColorRGB(0xffdf22, "sun-yellow",                            register=True)
    SUNBURNT_CYCLOPS                     : ColorRGB = lambda *_: ColorRGB(0xff404c, "sunburnt-cyclops",                      register=True)
    SUNFLOWER                            : ColorRGB = lambda *_: ColorRGB(0xffc512, "sunflower",                             register=True)
    SUNFLOWER_YELLOW                     : ColorRGB = lambda *_: ColorRGB(0xffda03, "sunflower-yellow",                      register=True)
    SUNGLOW                              : ColorRGB = lambda *_: ColorRGB(0xffcc33, "sunglow",                               register=True)
    SUNNY                                : ColorRGB = lambda *_: ColorRGB(0xf2f27a, "sunny",                                 register=True)
    SUNNY_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xfff917, "sunny-yellow",                          register=True)
    SUNRAY                               : ColorRGB = lambda *_: ColorRGB(0xe3ab57, "sunray",                                register=True)
    SUNRISE_CORAL                        : ColorRGB = lambda *_: ColorRGB(0xc27863, "sunrise-coral",                         register=True)
    SUNSET_ORANGE                        : ColorRGB = lambda *_: ColorRGB(0xfd5e53, "sunset-orange",                         register=True)
    SUNSET_YELLOW                        : ColorRGB = lambda *_: ColorRGB(0xffc922, "sunset-yellow",                         register=True)
    SUNSHINE_YELLOW                      : ColorRGB = lambda *_: ColorRGB(0xfffd37, "sunshine-yellow",                       register=True)
    SUPER_PINK                           : ColorRGB = lambda *_: ColorRGB(0xcf6ba9, "super-pink",                            register=True)
    SUPERUSER                            : ColorRGB = lambda *_: ColorRGB(0xf8b889, "superuser",                             register=True)
    SWAMP                                : ColorRGB = lambda *_: ColorRGB(0x698339, "swamp",                                 register=True)
    SWAMP_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x748500, "swamp-green",                           register=True)
    SWEET_BROWN                          : ColorRGB = lambda *_: ColorRGB(0xa83731, "sweet-brown",                           register=True)
    SWEET_SIXTEEN                        : ColorRGB = lambda *_: ColorRGB(0xf29eab, "sweet-sixteen",                         register=True)
    TAMARAMA                             : ColorRGB = lambda *_: ColorRGB(0x00b8d9, "tamarama",                              register=True, aliases=['jira-cyan'])
    TAN                                  : ColorRGB = lambda *_: ColorRGB(0xd1b26f, "tan",                                   register=True, variation_map={
             0xd99a6c:   "crayola" })
    TAN_BROWN                            : ColorRGB = lambda *_: ColorRGB(0xab7e4c, "tan-brown",                             register=True)
    TAN_GREEN                            : ColorRGB = lambda *_: ColorRGB(0xa9be70, "tan-green",                             register=True)
    TANGELO                              : ColorRGB = lambda *_: ColorRGB(0xf94d00, "tangelo",                               register=True)
    TANGERINE                            : ColorRGB = lambda *_: ColorRGB(0xff9408, "tangerine",                             register=True)
    TANGERINE_YELLOW                     : ColorRGB = lambda *_: ColorRGB(0xffcc00, "tangerine-yellow",                      register=True)
    TAOBAO_ORANGE                        : ColorRGB = lambda *_: ColorRGB(0xff4200, "taobao-orange",                         register=True)
    TARDIS_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x003b6f, "tardis-blue",                           register=True)
    TART_ORANGE                          : ColorRGB = lambda *_: ColorRGB(0xfb4d46, "tart-orange",                           register=True)
    TATARIAN_ASTER                       : ColorRGB = lambda *_: ColorRGB(0x976e9a, "tatarian-aster",                        register=True)
    TAUPE                                : ColorRGB = lambda *_: ColorRGB(0xb9a281, "taupe",                                 register=True)
    TAUPE_GRAY                           : ColorRGB = lambda *_: ColorRGB(0x8b8589, "taupe-gray",                            register=True)
    TEA                                  : ColorRGB = lambda *_: ColorRGB(0x65ab7c, "tea",                                   register=True)
    TEA_GREEN                            : ColorRGB = lambda *_: ColorRGB(0xbdf8a3, "tea-green",                             register=True)
    TEAL                                 : ColorRGB = lambda *_: ColorRGB(0x029386, "teal",                                  register=True)
    TEAL_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x01889f, "teal-blue",                             register=True)
    TEAL_DEER                            : ColorRGB = lambda *_: ColorRGB(0x99e6b3, "teal-deer",                             register=True)
    TEAL_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x25a36f, "teal-green",                            register=True)
    TEALISH                              : ColorRGB = lambda *_: ColorRGB(0x24bca8, "tealish",                               register=True)
    TEALISH_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x0cdc73, "tealish-green",                         register=True)
    TELEMAGENTA                          : ColorRGB = lambda *_: ColorRGB(0xcf3476, "telemagenta",                           register=True)
    TEMPTRESS                            : ColorRGB = lambda *_: ColorRGB(0x3c2126, "temptress",                             register=True)
    TENNE                                : ColorRGB = lambda *_: ColorRGB(0xcd5700, "tenne",                                 register=True)
    TERRA_COTTA                          : ColorRGB = lambda *_: ColorRGB(0xc9643b, "terra-cotta",                           register=True)
    TERRACOTA                            : ColorRGB = lambda *_: ColorRGB(0xcb6843, "terracota",                             register=True)
    TERRACOTTA                           : ColorRGB = lambda *_: ColorRGB(0xca6641, "terracotta",                            register=True)
    THE_COLOR_OF_AN_UNDRIED_WALL         : ColorRGB = lambda *_: ColorRGB(0x785e49, "the-color-of-an-undried-wall",          register=True)
    THE_SMELL                            : ColorRGB = lambda *_: ColorRGB(0xe3fcef, "the-smell",                             register=True)
    THIN                                 : ColorRGB = lambda *_: ColorRGB(0xa87ca0, "thin",                                  register=True)
    THISTLE                              : ColorRGB = lambda *_: ColorRGB(0xd8bfd8, "thistle",                               register=True, variation_map={
             0xebb0d7:   "crayola" })
    THOUSAND_HERB                        : ColorRGB = lambda *_: ColorRGB(0x317589, "thousand-herb",                         register=True)
    THOUSAND_YEAR_OLD_BROWN              : ColorRGB = lambda *_: ColorRGB(0x3b3429, "thousand-year-old-brown",               register=True)
    THOUSAND_YEAR_OLD_GREEN              : ColorRGB = lambda *_: ColorRGB(0x374231, "thousand-year-old-green",               register=True)
    THRICE_DYED_CRIMSON                  : ColorRGB = lambda *_: ColorRGB(0xf7665a, "thrice-dyed-crimson",                   register=True)
    TICKLE_ME_PINK                       : ColorRGB = lambda *_: ColorRGB(0xfc89ac, "tickle-me-pink",                        register=True)
    TIFFANY_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x7bf2da, "tiffany-blue",                          register=True)
    TIGERS_EYE                           : ColorRGB = lambda *_: ColorRGB(0xe08d3c, "tigers-eye",                            register=True)
    TIMBERWOLF                           : ColorRGB = lambda *_: ColorRGB(0xdbd7d2, "timberwolf",                            register=True)
    TITANIUM                             : ColorRGB = lambda *_: ColorRGB(0x878681, "titanium",                              register=True)
    TITANIUM_YELLOW                      : ColorRGB = lambda *_: ColorRGB(0xeee600, "titanium-yellow",                       register=True)
    TOMATO                               : ColorRGB = lambda *_: ColorRGB(0xef4026, "tomato",                                register=True)
    TOMATO_RED                           : ColorRGB = lambda *_: ColorRGB(0xec2d01, "tomato-red",                            register=True)
    TOMATO_SAUCE                         : ColorRGB = lambda *_: ColorRGB(0xb21807, "tomato-sauce",                          register=True)
    TOOLBOX                              : ColorRGB = lambda *_: ColorRGB(0x746cc0, "toolbox",                               register=True)
    TOPAZ                                : ColorRGB = lambda *_: ColorRGB(0x13bbaf, "topaz",                                 register=True)
    TOUPE                                : ColorRGB = lambda *_: ColorRGB(0xc7ac7d, "toupe",                                 register=True)
    TOXIC_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x61de2a, "toxic-green",                           register=True)
    TRACTOR_RED                          : ColorRGB = lambda *_: ColorRGB(0xfd0e35, "tractor-red",                           register=True)
    TREE_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x2a7e19, "tree-green",                            register=True)
    TREE_PEONY                           : ColorRGB = lambda *_: ColorRGB(0xa4345d, "tree-peony",                            register=True)
    TROLLEY_GRAY                         : ColorRGB = lambda *_: ColorRGB(0x808080, "trolley-gray",                          register=True)
    TROPICAL_RAIN_FOREST                 : ColorRGB = lambda *_: ColorRGB(0x00755e, "tropical-rain-forest",                  register=True)
    TROPICAL_VIOLET                      : ColorRGB = lambda *_: ColorRGB(0xcda4de, "tropical-violet",                       register=True)
    TRUE_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x010fcc, "true-blue",                             register=True, variation_map={
             0x2d68c4:   "2" })
    TRUE_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x089404, "true-green",                            register=True)
    TRUE_RED                             : ColorRGB = lambda *_: ColorRGB(0x8f1d21, "true-red",                              register=True)
    TUFTS_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x3e8ede, "tufts-blue",                            register=True)
    TULIP                                : ColorRGB = lambda *_: ColorRGB(0xff878d, "tulip",                                 register=True)
    TUMBLEWEED                           : ColorRGB = lambda *_: ColorRGB(0xdeaa88, "tumbleweed",                            register=True)
    TURKISH_ROSE                         : ColorRGB = lambda *_: ColorRGB(0xb57281, "turkish-rose",                          register=True)
    TURMERIC                             : ColorRGB = lambda *_: ColorRGB(0xe69b3a, "turmeric",                              register=True)
    TURQUOISE                            : ColorRGB = lambda *_: ColorRGB(0x06c2ac, "turquoise",                             register=True)
    TURQUOISE_BLUE                       : ColorRGB = lambda *_: ColorRGB(0x06b1c4, "turquoise-blue",                        register=True)
    TURQUOISE_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x04f489, "turquoise-green",                       register=True)
    TURQUOISE_SURF                       : ColorRGB = lambda *_: ColorRGB(0x00c5cd, "turquoise-surf",                        register=True)
    TURTLE_GREEN                         : ColorRGB = lambda *_: ColorRGB(0x75b84f, "turtle-green",                          register=True)
    TUSCAN_RED                           : ColorRGB = lambda *_: ColorRGB(0x7c4848, "tuscan-red",                            register=True)
    TUSCANY                              : ColorRGB = lambda *_: ColorRGB(0xc09999, "tuscany",                               register=True)
    TWILIGHT                             : ColorRGB = lambda *_: ColorRGB(0x4e518b, "twilight",                              register=True)
    TWILIGHT_BLUE                        : ColorRGB = lambda *_: ColorRGB(0x0a437a, "twilight-blue",                         register=True)
    TWILIGHT_LAVENDER                    : ColorRGB = lambda *_: ColorRGB(0x8a496b, "twilight-lavender",                     register=True)
    TWITTER_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x26a7de, "twitter-blue",                          register=True)
    UA_BLUE                              : ColorRGB = lambda *_: ColorRGB(0x0033aa, "ua-blue",                               register=True)
    UA_RED                               : ColorRGB = lambda *_: ColorRGB(0xd9004c, "ua-red",                                register=True)
    UBE                                  : ColorRGB = lambda *_: ColorRGB(0x8878c3, "ube",                                   register=True)
    UCLA_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x536895, "ucla-blue",                             register=True)
    UCLA_GOLD                            : ColorRGB = lambda *_: ColorRGB(0xffb300, "ucla-gold",                             register=True)
    UE_RED                               : ColorRGB = lambda *_: ColorRGB(0xba0001, "ue-red",                                register=True)
    UFO_GREEN                            : ColorRGB = lambda *_: ColorRGB(0x3cd070, "ufo-green",                             register=True)
    UGLY_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x31668a, "ugly-blue",                             register=True)
    UGLY_BROWN                           : ColorRGB = lambda *_: ColorRGB(0x7d7103, "ugly-brown",                            register=True)
    UGLY_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x7a9703, "ugly-green",                            register=True)
    UGLY_PINK                            : ColorRGB = lambda *_: ColorRGB(0xcd7584, "ugly-pink",                             register=True)
    UGLY_PURPLE                          : ColorRGB = lambda *_: ColorRGB(0xa442a0, "ugly-purple",                           register=True)
    UGLY_YELLOW                          : ColorRGB = lambda *_: ColorRGB(0xd0c101, "ugly-yellow",                           register=True)
    ULTRA_RED                            : ColorRGB = lambda *_: ColorRGB(0xfc6c85, "ultra-red",                             register=True)
    ULTRAMARINE                          : ColorRGB = lambda *_: ColorRGB(0x2000b1, "ultramarine",                           register=True, variation_map={
             0x5d8cae:   "gunjo-iro" })
    ULTRAMARINE_BLUE                     : ColorRGB = lambda *_: ColorRGB(0x1805db, "ultramarine-blue",                      register=True)
    UMBER                                : ColorRGB = lambda *_: ColorRGB(0xb26400, "umber",                                 register=True)
    UNBLEACHED_SILK                      : ColorRGB = lambda *_: ColorRGB(0xffddca, "unbleached-silk",                       register=True)
    UNDERSIDE_OF_WILLOW_LEAVES           : ColorRGB = lambda *_: ColorRGB(0xbcb58c, "underside-of-willow-leaves",            register=True)
    UNITED_NATIONS_BLUE                  : ColorRGB = lambda *_: ColorRGB(0x5b92e5, "united-nations-blue",                   register=True)
    UNIVERSITY_OF_CALIFORNIA_GOLD        : ColorRGB = lambda *_: ColorRGB(0xb78727, "university-of-california-gold",         register=True)
    UNIVERSITY_OF_TENNESSEE_ORANGE       : ColorRGB = lambda *_: ColorRGB(0xf77f00, "university-of-tennessee-orange",        register=True)
    UP_MAROON                            : ColorRGB = lambda *_: ColorRGB(0x7b1113, "up-maroon",                             register=True)
    UPSDELL_RED                          : ColorRGB = lambda *_: ColorRGB(0xae2029, "upsdell-red",                           register=True)
    UROBILIN                             : ColorRGB = lambda *_: ColorRGB(0xe1ad21, "urobilin",                              register=True)
    USAFA_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x004f98, "usafa-blue",                            register=True)
    UTAH_CRIMSON                         : ColorRGB = lambda *_: ColorRGB(0xd3003f, "utah-crimson",                          register=True)
    VAMPIRE_BLACK                        : ColorRGB = lambda *_: ColorRGB(0x080808, "vampire-black",                         register=True)
    VAN_DYKE_BROWN                       : ColorRGB = lambda *_: ColorRGB(0x664228, "van-dyke-brown",                        register=True)
    VANILLA_ICE                          : ColorRGB = lambda *_: ColorRGB(0xf38fa9, "vanilla-ice",                           register=True)
    VEGAS_GOLD                           : ColorRGB = lambda *_: ColorRGB(0xc5b358, "vegas-gold",                            register=True)
    VELVET                               : ColorRGB = lambda *_: ColorRGB(0x750851, "velvet",                                register=True, variation_map={
             0x224634:   "birodo" })
    VENETIAN_RED                         : ColorRGB = lambda *_: ColorRGB(0xc80815, "venetian-red",                          register=True)
    VERDIGRIS                            : ColorRGB = lambda *_: ColorRGB(0x43b3ae, "verdigris",                             register=True)
    VERIZON_RED                          : ColorRGB = lambda *_: ColorRGB(0xcd040b, "verizon-red",                           register=True, variation_map={
             0xec1c24:   "2000-2015" })
    VERMILION                            : ColorRGB = lambda *_: ColorRGB(0xd9381e, "vermilion",                             register=True)
    VERMILLION                           : ColorRGB = lambda *_: ColorRGB(0xf4320c, "vermillion",                            register=True)
    VERSE_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x18880d, "verse-green",                           register=True)
    VERY_DARK_BLUE                       : ColorRGB = lambda *_: ColorRGB(0x000133, "very-dark-blue",                        register=True)
    VERY_DARK_BROWN                      : ColorRGB = lambda *_: ColorRGB(0x1d0200, "very-dark-brown",                       register=True)
    VERY_DARK_GREEN                      : ColorRGB = lambda *_: ColorRGB(0x062e03, "very-dark-green",                       register=True)
    VERY_DARK_PURPLE                     : ColorRGB = lambda *_: ColorRGB(0x2a0134, "very-dark-purple",                      register=True)
    VERY_LIGHT_AZURE                     : ColorRGB = lambda *_: ColorRGB(0x74bbfb, "very-light-azure",                      register=True)
    VERY_LIGHT_BLUE                      : ColorRGB = lambda *_: ColorRGB(0xd5ffff, "very-light-blue",                       register=True)
    VERY_LIGHT_BROWN                     : ColorRGB = lambda *_: ColorRGB(0xd3b683, "very-light-brown",                      register=True)
    VERY_LIGHT_GREEN                     : ColorRGB = lambda *_: ColorRGB(0xd1ffbd, "very-light-green",                      register=True)
    VERY_LIGHT_MALACHITE_GREEN           : ColorRGB = lambda *_: ColorRGB(0x64e986, "very-light-malachite-green",            register=True)
    VERY_LIGHT_PINK                      : ColorRGB = lambda *_: ColorRGB(0xfff4f2, "very-light-pink",                       register=True)
    VERY_LIGHT_PURPLE                    : ColorRGB = lambda *_: ColorRGB(0xf6cefc, "very-light-purple",                     register=True)
    VERY_LIGHT_TANGELO                   : ColorRGB = lambda *_: ColorRGB(0xffb077, "very-light-tangelo",                    register=True)
    VERY_PALE_BLUE                       : ColorRGB = lambda *_: ColorRGB(0xd6fffe, "very-pale-blue",                        register=True)
    VERY_PALE_GREEN                      : ColorRGB = lambda *_: ColorRGB(0xcffdbc, "very-pale-green",                       register=True)
    VERY_PALE_ORANGE                     : ColorRGB = lambda *_: ColorRGB(0xffdfbf, "very-pale-orange",                      register=True)
    VERY_PALE_YELLOW                     : ColorRGB = lambda *_: ColorRGB(0xffffbf, "very-pale-yellow",                      register=True)
    VIBRANT_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x0339f8, "vibrant-blue",                          register=True)
    VIBRANT_GREEN                        : ColorRGB = lambda *_: ColorRGB(0x0add08, "vibrant-green",                         register=True)
    VIBRANT_PURPLE                       : ColorRGB = lambda *_: ColorRGB(0xad03de, "vibrant-purple",                        register=True)
    VICTORIAN_WHITE                      : ColorRGB = lambda *_: ColorRGB(0xf4e9ce, "victorian-white",                       register=True)
    VINE_GRAPE                           : ColorRGB = lambda *_: ColorRGB(0x6d2b50, "vine-grape",                            register=True)
    VINE_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x38a32a, "vine-green",                            register=True)
    VIOLET                               : ColorRGB = lambda *_: ColorRGB(0x9a0eea, "violet",                                register=True, variation_map={
             0x7f00ff:   "color-wheel", 
             0x963d7f:   "crayola", 
             0x8601af:   "ryb" })
    VIOLET_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x510ac9, "violet-blue",                           register=True, variation_map={
             0x766ec8:   "crayola" })
    VIOLET_PINK                          : ColorRGB = lambda *_: ColorRGB(0xfb5ffc, "violet-pink",                           register=True)
    VIOLET_RED                           : ColorRGB = lambda *_: ColorRGB(0xa50055, "violet-red",                            register=True, variation_map={
             0x891446:   "2" })
    VIOLETS_ARE_BLUE                     : ColorRGB = lambda *_: ColorRGB(0x8366f4, "violets-are-blue",                      register=True)
    VIOLIN_BROWN                         : ColorRGB = lambda *_: ColorRGB(0x674403, "violin-brown",                          register=True)
    VIRIDIAN                             : ColorRGB = lambda *_: ColorRGB(0x1e9167, "viridian",                              register=True)
    VIRIDIAN_GREEN                       : ColorRGB = lambda *_: ColorRGB(0x009698, "viridian-green",                        register=True)
    VISTA_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x7c9ed9, "vista-blue",                            register=True)
    VIVALDI_RED                          : ColorRGB = lambda *_: ColorRGB(0xef3939, "vivaldi-red",                           register=True)
    VIVID_AUBURN                         : ColorRGB = lambda *_: ColorRGB(0x922724, "vivid-auburn",                          register=True)
    VIVID_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x152eff, "vivid-blue",                            register=True)
    VIVID_BURGUNDY                       : ColorRGB = lambda *_: ColorRGB(0x9f1d35, "vivid-burgundy",                        register=True)
    VIVID_CERISE                         : ColorRGB = lambda *_: ColorRGB(0xda1d81, "vivid-cerise",                          register=True)
    VIVID_CERULEAN                       : ColorRGB = lambda *_: ColorRGB(0x00aaee, "vivid-cerulean",                        register=True)
    VIVID_CRIMSON                        : ColorRGB = lambda *_: ColorRGB(0xcc0033, "vivid-crimson",                         register=True)
    VIVID_GAMBOGE                        : ColorRGB = lambda *_: ColorRGB(0xff9900, "vivid-gamboge",                         register=True)
    VIVID_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x2fef10, "vivid-green",                           register=True)
    VIVID_LIME_GREEN                     : ColorRGB = lambda *_: ColorRGB(0xa6d608, "vivid-lime-green",                      register=True)
    VIVID_MALACHITE                      : ColorRGB = lambda *_: ColorRGB(0x00cc33, "vivid-malachite",                       register=True)
    VIVID_MULBERRY                       : ColorRGB = lambda *_: ColorRGB(0xb80ce3, "vivid-mulberry",                        register=True)
    VIVID_ORANGE                         : ColorRGB = lambda *_: ColorRGB(0xff5f00, "vivid-orange",                          register=True)
    VIVID_ORANGE_PEEL                    : ColorRGB = lambda *_: ColorRGB(0xffa000, "vivid-orange-peel",                     register=True)
    VIVID_ORCHID                         : ColorRGB = lambda *_: ColorRGB(0xcc00ff, "vivid-orchid",                          register=True)
    VIVID_PURPLE                         : ColorRGB = lambda *_: ColorRGB(0x9900fa, "vivid-purple",                          register=True)
    VIVID_RASPBERRY                      : ColorRGB = lambda *_: ColorRGB(0xff006c, "vivid-raspberry",                       register=True)
    VIVID_RED                            : ColorRGB = lambda *_: ColorRGB(0xf70d1a, "vivid-red",                             register=True)
    VIVID_RED_TANGELO                    : ColorRGB = lambda *_: ColorRGB(0xdf6124, "vivid-red-tangelo",                     register=True)
    VIVID_SKY_BLUE                       : ColorRGB = lambda *_: ColorRGB(0x00ccff, "vivid-sky-blue",                        register=True)
    VIVID_TANGELO                        : ColorRGB = lambda *_: ColorRGB(0xf07427, "vivid-tangelo",                         register=True)
    VIVID_TANGERINE                      : ColorRGB = lambda *_: ColorRGB(0xffa089, "vivid-tangerine",                       register=True)
    VIVID_VERMILION                      : ColorRGB = lambda *_: ColorRGB(0xe56024, "vivid-vermilion",                       register=True)
    VIVID_VIOLET                         : ColorRGB = lambda *_: ColorRGB(0x9f00ff, "vivid-violet",                          register=True)
    VIVID_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xffe302, "vivid-yellow",                          register=True)
    VODKA                                : ColorRGB = lambda *_: ColorRGB(0xbfc0ee, "vodka",                                 register=True)
    VOLT                                 : ColorRGB = lambda *_: ColorRGB(0xceff00, "volt",                                  register=True)
    VOMIT                                : ColorRGB = lambda *_: ColorRGB(0xa2a415, "vomit",                                 register=True)
    VOMIT_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x89a203, "vomit-green",                           register=True)
    VOMIT_YELLOW                         : ColorRGB = lambda *_: ColorRGB(0xc7c10c, "vomit-yellow",                          register=True)
    WAGENINGEN_GREEN                     : ColorRGB = lambda *_: ColorRGB(0x34b233, "wageningen-green",                      register=True)
    WALNUT_DYED                          : ColorRGB = lambda *_: ColorRGB(0x9f7462, "walnut-dyed",                           register=True)
    WARM_BLACK                           : ColorRGB = lambda *_: ColorRGB(0x004242, "warm-black",                            register=True)
    WARM_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x4b57db, "warm-blue",                             register=True)
    WARM_BROWN                           : ColorRGB = lambda *_: ColorRGB(0x964e02, "warm-brown",                            register=True)
    WARM_GREY                            : ColorRGB = lambda *_: ColorRGB(0x978a84, "warm-grey",                             register=True)
    WARM_PINK                            : ColorRGB = lambda *_: ColorRGB(0xfb5581, "warm-pink",                             register=True)
    WARM_PURPLE                          : ColorRGB = lambda *_: ColorRGB(0x952e8f, "warm-purple",                           register=True)
    WASH_ME                              : ColorRGB = lambda *_: ColorRGB(0xfafbfc, "wash-me",                               register=True)
    WASHED_OUT_CRIMSON                   : ColorRGB = lambda *_: ColorRGB(0xffb3a7, "washed-out-crimson",                    register=True)
    WASHED_OUT_GREEN                     : ColorRGB = lambda *_: ColorRGB(0xbcf5a6, "washed-out-green",                      register=True)
    WASHED_OUT_PERSIMMON                 : ColorRGB = lambda *_: ColorRGB(0xec8254, "washed-out-persimmon",                  register=True)
    WATER                                : ColorRGB = lambda *_: ColorRGB(0xd4f1f9, "water",                                 register=True)
    WATER_BLUE                           : ColorRGB = lambda *_: ColorRGB(0x0e87cc, "water-blue",                            register=True)
    WATER_PERSIMMON                      : ColorRGB = lambda *_: ColorRGB(0xb56c60, "water-persimmon",                       register=True)
    WATERMELON                           : ColorRGB = lambda *_: ColorRGB(0xfd4659, "watermelon",                            register=True)
    WATERMELON_YELLOW                    : ColorRGB = lambda *_: ColorRGB(0xeeff1b, "watermelon-yellow",                     register=True)
    WATERSPOUT                           : ColorRGB = lambda *_: ColorRGB(0xa4f4f9, "waterspout",                            register=True)
    WEATHERED_BAMBOO                     : ColorRGB = lambda *_: ColorRGB(0x593a27, "weathered-bamboo",                      register=True)
    WEEBLY_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x1f8eed, "weebly-blue",                           register=True)
    WEEBLY_ORANGE                        : ColorRGB = lambda *_: ColorRGB(0xff9a00, "weebly-orange",                         register=True)
    WEIRD_GREEN                          : ColorRGB = lambda *_: ColorRGB(0x3ae57f, "weird-green",                           register=True)
    WELDON_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x7c98ab, "weldon-blue",                           register=True)
    WENGE                                : ColorRGB = lambda *_: ColorRGB(0x645452, "wenge",                                 register=True)
    WHEAT                                : ColorRGB = lambda *_: ColorRGB(0xfbdd7e, "wheat",                                 register=True)
    WHITE                                : ColorRGB = lambda *_: ColorRGB(0xffffff, "white",                                 register=True)
    WHITE_CHOCOLATE                      : ColorRGB = lambda *_: ColorRGB(0xede6d6, "white-chocolate",                       register=True)
    WHITE_COFFEE                         : ColorRGB = lambda *_: ColorRGB(0xe6e0d4, "white-coffee",                          register=True)
    WHITE_KALM                           : ColorRGB = lambda *_: ColorRGB(0xc5c8c6, "white-kalm",                            register=True)
    WHITE_MOUSE                          : ColorRGB = lambda *_: ColorRGB(0xb9a193, "white-mouse",                           register=True)
    WHITE_OAK                            : ColorRGB = lambda *_: ColorRGB(0xce9f6f, "white-oak",                             register=True)
    WHITE_PANIK                          : ColorRGB = lambda *_: ColorRGB(0xececec, "white-panik",                           register=True)
    WHITE_TEA                            : ColorRGB = lambda *_: ColorRGB(0xc48e69, "white-tea",                             register=True)
    WHITISH_GREEN                        : ColorRGB = lambda *_: ColorRGB(0xa5ba93, "whitish-green",                         register=True)
    WILD_BLUE_YONDER                     : ColorRGB = lambda *_: ColorRGB(0xa2add0, "wild-blue-yonder",                      register=True)
    WILD_ORCHID                          : ColorRGB = lambda *_: ColorRGB(0xd470a2, "wild-orchid",                           register=True)
    WILD_STRAWBERRY                      : ColorRGB = lambda *_: ColorRGB(0xff43a4, "wild-strawberry",                       register=True)
    WILLOW_DYE                           : ColorRGB = lambda *_: ColorRGB(0x8c9e5e, "willow-dye",                            register=True)
    WILLOW_GRAY                          : ColorRGB = lambda *_: ColorRGB(0x817b69, "willow-gray",                           register=True)
    WILLOW_TEA                           : ColorRGB = lambda *_: ColorRGB(0x9c8a4d, "willow-tea",                            register=True)
    WILLPOWER_ORANGE                     : ColorRGB = lambda *_: ColorRGB(0xfd5800, "willpower-orange",                      register=True)
    WILTED_BROWN                         : ColorRGB = lambda *_: ColorRGB(0xab4c3d, "wilted-brown",                          register=True)
    WINDOWS_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x3778bf, "windows-blue",                          register=True)
    WINDSOR_TAN                          : ColorRGB = lambda *_: ColorRGB(0xa75502, "windsor-tan",                           register=True)
    WINE                                 : ColorRGB = lambda *_: ColorRGB(0x80013f, "wine",                                  register=True)
    WINE_RED                             : ColorRGB = lambda *_: ColorRGB(0x7b0323, "wine-red",                              register=True)
    WINTER_SKY                           : ColorRGB = lambda *_: ColorRGB(0xff007c, "winter-sky",                            register=True)
    WINTER_WIZARD                        : ColorRGB = lambda *_: ColorRGB(0xa0e6ff, "winter-wizard",                         register=True)
    WINTERGREEN                          : ColorRGB = lambda *_: ColorRGB(0x20f986, "wintergreen",                           register=True)
    WINTERGREEN_DREAM                    : ColorRGB = lambda *_: ColorRGB(0x56887d, "wintergreen-dream",                     register=True)
    WISTERIA                             : ColorRGB = lambda *_: ColorRGB(0xa87dc2, "wisteria",                              register=True, variation_map={
             0x89729e:   "fuji-iro" })
    WISTERIA_AND_BURNT_BAMBOO            : ColorRGB = lambda *_: ColorRGB(0x4d3b3c, "wisteria-and-burnt-bamboo",             register=True)
    WISTERIA_PURPLE                      : ColorRGB = lambda *_: ColorRGB(0x875f9a, "wisteria-purple",                       register=True)
    XANADU                               : ColorRGB = lambda *_: ColorRGB(0x738678, "xanadu",                                register=True)
    XBOX_GREEN                           : ColorRGB = lambda *_: ColorRGB(0x0e7a0d, "xbox-green",                            register=True)
    XIAOMI_ORANGE                        : ColorRGB = lambda *_: ColorRGB(0xfd4900, "xiaomi-orange",                         register=True)
    YALE_BLUE                            : ColorRGB = lambda *_: ColorRGB(0x0f4d92, "yale-blue",                             register=True)
    YANKEES_BLUE                         : ColorRGB = lambda *_: ColorRGB(0x1c2841, "yankees-blue",                          register=True)
    YELLOW                               : ColorRGB = lambda *_: ColorRGB(0xffff14, "yellow",                                register=True, variation_map={
             0xfce883:   "crayola", 
             0xefcc00:   "munsell", 
             0xfedf00:   "pantone", 
             0xfefe33:   "ryb" })
    YELLOW_BROWN                         : ColorRGB = lambda *_: ColorRGB(0xb79400, "yellow-brown",                          register=True)
    YELLOW_GREEN                         : ColorRGB = lambda *_: ColorRGB(0xc0fb2d, "yellow-green",                          register=True, variation_map={
             0x30b21a:   "color-wheel", 
             0xc5e384:   "crayola" })
    YELLOW_KALM                          : ColorRGB = lambda *_: ColorRGB(0xde935f, "yellow-kalm",                           register=True)
    YELLOW_OCHRE                         : ColorRGB = lambda *_: ColorRGB(0xcb9d06, "yellow-ochre",                          register=True)
    YELLOW_ORANGE                        : ColorRGB = lambda *_: ColorRGB(0xfcb001, "yellow-orange",                         register=True, variation_map={
             0xff9505:   "color-wheel" })
    YELLOW_PANIK                         : ColorRGB = lambda *_: ColorRGB(0xf3f99c, "yellow-panik",                          register=True)
    YELLOW_ROSE                          : ColorRGB = lambda *_: ColorRGB(0xfff000, "yellow-rose",                           register=True)
    YELLOW_SEA_PINE_BROWN                : ColorRGB = lambda *_: ColorRGB(0x896c39, "yellow-sea-pine-brown",                 register=True)
    YELLOW_TAN                           : ColorRGB = lambda *_: ColorRGB(0xffe36e, "yellow-tan",                            register=True)
    YELLOWGREEN                          : ColorRGB = lambda *_: ColorRGB(0xbbf90f, "yellowgreen",                           register=True)
    YELLOWISH                            : ColorRGB = lambda *_: ColorRGB(0xfaee66, "yellowish",                             register=True)
    YELLOWISH_BROWN                      : ColorRGB = lambda *_: ColorRGB(0x9b7a01, "yellowish-brown",                       register=True)
    YELLOWISH_GREEN                      : ColorRGB = lambda *_: ColorRGB(0xb0dd16, "yellowish-green",                       register=True)
    YELLOWISH_ORANGE                     : ColorRGB = lambda *_: ColorRGB(0xffab0f, "yellowish-orange",                      register=True)
    YELLOWISH_TAN                        : ColorRGB = lambda *_: ColorRGB(0xfcfc81, "yellowish-tan",                         register=True)
    YELLOWY_BROWN                        : ColorRGB = lambda *_: ColorRGB(0xae8b0c, "yellowy-brown",                         register=True)
    YELLOWY_GREEN                        : ColorRGB = lambda *_: ColorRGB(0xbff128, "yellowy-green",                         register=True)
    YIN_MN_BLUE                          : ColorRGB = lambda *_: ColorRGB(0x2e5090, "yin-mn-blue",                           register=True)
    YOUNG_BAMBOO                         : ColorRGB = lambda *_: ColorRGB(0x6b9362, "young-bamboo",                          register=True)
    YOUTUBE_RED                          : ColorRGB = lambda *_: ColorRGB(0xb2071d, "youtube-red",                           register=True)
    ZAFFRE                               : ColorRGB = lambda *_: ColorRGB(0x0014a8, "zaffre",                                register=True)
    ZINNWALDITE                          : ColorRGB = lambda *_: ColorRGB(0xebc2af, "zinnwaldite",                           register=True)
    ZINNWALDITE_BROWN                    : ColorRGB = lambda *_: ColorRGB(0x2c1608, "zinnwaldite-brown",                     register=True)
    ZOMP                                 : ColorRGB = lambda *_: ColorRGB(0x39a78e, "zomp",                                  register=True)
    # fmt: on


_CVT = t.TypeVar("_CVT", _ColorValuesXterm, _ColorValuesRGB)


class _ColorValuesLoader:
    """
    Actual colors
    """
    def __init__(self, target_cls: t.Type[_CVT]):
        self._target_cls: t.Type[_CVT] = target_cls
        self._actual: _CVT | None = None

    def load(self):
        if not self._actual:
            self._actual = self._target_cls()
            self._actual.load()

    def __getattr__(self, item):
        # when preset color is accessed
        self.load()
        return getattr(self._actual, item)


cv: _ColorValuesXterm = _ColorValuesXterm()
cvr: _ColorValuesRGB|_ColorValuesLoader = _ColorValuesLoader(_ColorValuesRGB)
